package demo.barcharts.category ;

import java.awt.Color ;
import java.awt.Font ;
import java.awt.GradientPaint ;
import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.AxisLocation ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.Bar2DRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.title.TextTitle ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class BarChartDemo11 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  public BarChartDemo11() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createCategoryBar2DChart(
      "Open Source Projects By Licence", "Licence", "Project Count", dataset, orientation, false, true) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    plot.setValueAxisLocation(AxisLocation.TOP_OR_LEFT);

    TextTitle texttitle = new TextTitle("Source: Freshmeat (http://www.freshmeat.net/)",
                                        new Font("Dialog", 0, 12)) ;
    texttitle.setPosition(RectangleEdge.BOTTOM) ;
    chart.addSubtitle(texttitle) ;

    NumberValueAxis numberaxis = (NumberValueAxis) plot.getValueAxis() ;
    numberaxis.setTickUnit(new LongTickUnit()) ;
    Bar2DRender render = (Bar2DRender) plot.getRender() ;
    render.setToolTipGenerator(new DefaultTooltipGenerator() {
      public String generateToolTip(IMatrixDataset dataset, int row, int column) {
        return dataset.getRowKey(row) + ": "
          + ((NumberChartData) dataset.getData(row, column)).getValue() + " projects" ;
      }
    }) ;

    GradientPaint gradientpaint = new GradientPaint(0.0F, 0.0F, Color.blue, 0.0F, 0.0F,
                                                    new Color(0, 0, 64)) ;
    render.setAutoSelectFillPaint(false);
    render.setColumnItemFillPaint(0, gradientpaint) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addColKey("S1") ;

    String[] keys = {"GNUGeneralPublicLicence", "GNULesserGeneralPublicLicence",
                    "BSDLicence(Original)", "BSDLicence(Revised)", "MIT/XConsortiumLicence",
                    "ArtisticLicence", "PublicDomain", "ApacheLicence2.0", "ApacheLicence",
                    "MozillaPublicLicence", "Other"} ;
    for (int i = 0 ; i < keys.length ; i++) {
      dataset.addRowKey(keys[i]) ;
    }

    double data[] = {23192D, 3157D, 1506D, 1283D, 738D, 630D, 585D, 349D, 317D, 309D, 918D} ;
    dataset.addRowData(data.length) ;

    for (int i = 0 ; i < data.length ; i++) {
      dataset.addData(i, new NumberChartData(data[i])) ;
    }

    return dataset ;
  }

}
