package demo.combinecharts ;

import java.awt.BorderLayout ;
import java.awt.Font ;

import javax.swing.JPanel ;

import com.ijchart.xychart.annotation.XYTextAnnotation ;
import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.plot.combine.CombineDomainAxisXYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CombinedXYPlotDemo4 extends JPanel {

  public CombinedXYPlotDemo4() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    ChartOrientation orientation = ChartOrientation.VERTICAL ;

    IMatrixDataset dataset1 = createDataset1() ;
    IMatrixDataset dataset2 = createDataset2() ;

    XYNumberValueAxis xAxis1 = new XYNumberValueAxis("Domain", dataset1,
                                                     XYValueAxisType.XY_VALUE_AXIS_X, false) ;
    XYNumberValueAxis yAxis1 = new XYNumberValueAxis("Range 1", dataset1,
                                                     XYValueAxisType.XY_VALUE_AXIS_Y, false) ;
    XYNumberValueAxis yAxis3 = new XYNumberValueAxis("Range Axis 2", dataset2,
                                                     XYValueAxisType.XY_VALUE_AXIS_Y, false) ;

    XYLineAndShapeRender render1 = new XYLineAndShapeRender(dataset1) ;
    render1.setBaseItemShapeVisible(false) ;
    XYTextAnnotation xytextannotation = new XYTextAnnotation("Hello!", 50D, 10000D) ;
    xytextannotation.setTextFont(new Font("SansSerif", 0, 9)) ;
    xytextannotation.setRotationAngle(0.78539816339744828D) ;
    render1.addAnnotation(xytextannotation) ;

    XYLineAndShapeRender render3 = new XYLineAndShapeRender(dataset2) ;
    render3.setBaseItemShapeVisible(false) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis1.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis1.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
      yAxis3.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }

    XYPlot plot1 = new XYPlot(xAxis1, yAxis1, render1) ;
    plot1.setOrientation(orientation) ;
    plot1.setValueAxis(1, yAxis3) ;
    plot1.setRender(1, render3) ;
    plot1.mapRenderToValueAxis(1, 1) ;

    XYNumberValueAxis xAxis2 = new XYNumberValueAxis("Domain", dataset2,
                                                     XYValueAxisType.XY_VALUE_AXIS_X, false) ;
    XYNumberValueAxis yAxis2 = new XYNumberValueAxis("Range 1", dataset2,
                                                     XYValueAxisType.XY_VALUE_AXIS_Y, false) ;

    XYLineAndShapeRender render2 = new XYLineAndShapeRender(dataset2) ;
    render2.setBaseItemShapeVisible(false) ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      xAxis2.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      yAxis2.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    }

    XYPlot plot2 = new XYPlot(xAxis2, yAxis2, render2) ;
    plot2.setOrientation(orientation) ;

    CombineDomainAxisXYPlot combinePlot = new CombineDomainAxisXYPlot() ;
    combinePlot.addSubPlot(plot2, 1) ;
    combinePlot.addSubPlot(plot1, 1) ;
    combinePlot.setPlotGap(15) ;

    IJChart chart = new IJChart("CombinedDomainXYPlot Demo", IJChart.DEFAULT_TITLE_FONT,
                                combinePlot, true) ;
    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static DefaultMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Series 1a") ;
    dataset.addRowKey("Series 1b") ;
    dataset.addRowData(2) ;

    dataset.addData(0, new XYNumberChartData(10D, 12353.299999999999D)) ;
    dataset.addData(0, new XYNumberChartData(20D, 13734.4D)) ;
    dataset.addData(0, new XYNumberChartData(30D, 14525.299999999999D)) ;
    dataset.addData(0, new XYNumberChartData(40D, 13984.299999999999D)) ;
    dataset.addData(0, new XYNumberChartData(50D, 12999.4D)) ;
    dataset.addData(0, new XYNumberChartData(60D, 14274.299999999999D)) ;
    dataset.addData(0, new XYNumberChartData(70D, 15943.5D)) ;
    dataset.addData(0, new XYNumberChartData(80D, 14845.299999999999D)) ;
    dataset.addData(0, new XYNumberChartData(90D, 14645.4D)) ;
    dataset.addData(0, new XYNumberChartData(100D, 16234.6D)) ;
    dataset.addData(0, new XYNumberChartData(110D, 17232.299999999999D)) ;
    dataset.addData(0, new XYNumberChartData(120D, 14232.200000000001D)) ;
    dataset.addData(0, new XYNumberChartData(130D, 13102.200000000001D)) ;
    dataset.addData(0, new XYNumberChartData(140D, 14230.200000000001D)) ;
    dataset.addData(0, new XYNumberChartData(150D, 11235.200000000001D)) ;

    dataset.addData(1, new XYNumberChartData(10D, 15000.299999999999D)) ;
    dataset.addData(1, new XYNumberChartData(20D, 11000.4D)) ;
    dataset.addData(1, new XYNumberChartData(30D, 17000.299999999999D)) ;
    dataset.addData(1, new XYNumberChartData(40D, 15000.299999999999D)) ;
    dataset.addData(1, new XYNumberChartData(50D, 14000.4D)) ;
    dataset.addData(1, new XYNumberChartData(60D, 12000.299999999999D)) ;
    dataset.addData(1, new XYNumberChartData(70D, 11000.5D)) ;
    dataset.addData(1, new XYNumberChartData(80D, 12000.299999999999D)) ;
    dataset.addData(1, new XYNumberChartData(90D, 13000.4D)) ;
    dataset.addData(1, new XYNumberChartData(100D, 12000.6D)) ;
    dataset.addData(1, new XYNumberChartData(110D, 13000.299999999999D)) ;
    dataset.addData(1, new XYNumberChartData(120D, 17000.200000000001D)) ;
    dataset.addData(1, new XYNumberChartData(130D, 18000.200000000001D)) ;
    dataset.addData(1, new XYNumberChartData(140D, 16000.200000000001D)) ;
    dataset.addData(1, new XYNumberChartData(150D, 17000.200000000001D)) ;

    return dataset ;
  }

  private static DefaultMatrixDataset createDataset2() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Series 2") ;
    dataset.addRowData() ;

    dataset.addData(0, new XYNumberChartData(10D, 6853.1999999999998D)) ;
    dataset.addData(0, new XYNumberChartData(20D, 9642.2999999999993D)) ;
    dataset.addData(0, new XYNumberChartData(30D, 8253.5D)) ;
    dataset.addData(0, new XYNumberChartData(40D, 5352.3000000000002D)) ;
    dataset.addData(0, new XYNumberChartData(50D, 3532D)) ;
    dataset.addData(0, new XYNumberChartData(60D, 2635.3000000000002D)) ;
    dataset.addData(0, new XYNumberChartData(70D, 3998.1999999999998D)) ;
    dataset.addData(0, new XYNumberChartData(80D, 1943.2D)) ;
    dataset.addData(0, new XYNumberChartData(90D, 6943.8999999999996D)) ;
    dataset.addData(0, new XYNumberChartData(100D, 7843.1999999999998D)) ;
    dataset.addData(0, new XYNumberChartData(105D, 6495.3000000000002D)) ;
    dataset.addData(0, new XYNumberChartData(110D, 7943.6000000000004D)) ;
    dataset.addData(0, new XYNumberChartData(115D, 8500.7000000000007D)) ;
    dataset.addData(0, new XYNumberChartData(120D, 9595.8999999999996D)) ;

    return dataset ;
  }

}
