package demo.granttcharts ;

import java.awt.GridLayout ;
import java.text.SimpleDateFormat ;
import java.util.Calendar ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.IntervalXYValueAxis ;
import com.ijchart.xychart.axis.value.SymbolValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.data.IntervalXYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.tickunit.LongTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYTaskDatasetDemo1 extends JPanel {

  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  private Calendar calendar = Calendar.getInstance() ;

  public XYTaskDatasetDemo1() {
    calendar.clear() ;
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    IMatrixDataset dataset = createDataset1() ;
    IJChart chart = ChartFactory.createIntervalXYBar2DChart(
      "XYTaskDatasetDemo1", "Resource", "Timing", dataset, orientation, true, true, false, false) ;

    XYPlot plot = (XYPlot) chart.getPlot() ;
    IntervalXYValueAxis xAxis = (IntervalXYValueAxis) plot.getDomainAxis() ;
    xAxis.setTickUnit(new LongTickUnit()) ;
    xAxis.setTickMarkVisible(false) ;

    IntervalXYValueAxis valueAxis = (IntervalXYValueAxis) plot.getValueAxis() ;
    valueAxis.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_HOUR,
                                           new SimpleDateFormat("HH:SS"))) ;

    String[] symbols = {null, "Team A", "Team B", "Team C", "Team D", null} ;
    SymbolValueAxis symbolAxis = new SymbolValueAxis("Resource", xAxis, symbols) ;
    symbolAxis.setAxisLabelAngle(IAngle.ANGLE_UP_90) ;
    symbolAxis.setGridBandVisible(false) ;
    plot.setDomainAxis(symbolAxis) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private IMatrixDataset createDataset1() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    String[] rowKeys = {"Team A", "Team B", "Team C", "Team D"} ;
    for (int i = 0 ; i < rowKeys.length ; i++) {
      dataset.addRowKey(rowKeys[i]) ;
    }

    dataset.addRowData(rowKeys.length) ;
    dataset.addData(0, new IntervalXYNumberChartData(0, 0, getMills(11), getMills(12))) ;
    dataset.addData(0, new IntervalXYNumberChartData(5, 5, getMills(11), getMills(12))) ;
    dataset.addData(0, createData(1, 11, 12)) ;
    dataset.addData(0, createData(1, 14, 15)) ;
    dataset.addData(0, createData(1, 16, 17)) ;

    dataset.addData(1, createData(2, 13, 14)) ;
    dataset.addData(1, createData(2, 19, 20)) ;
    dataset.addData(1, createData(2, 21, 22)) ;

    dataset.addData(2, createData(3, 13, 14)) ;
    dataset.addData(2, createData(3, 19, 20)) ;
    dataset.addData(2, createData(3, 21, 22)) ;

    dataset.addData(3, createData(4, 0, 24)) ;

    return dataset ;
  }

  private IntervalXYNumberChartData createData(double centerX, int startHour, int endHour) {
    return new IntervalXYNumberChartData(centerX - 0.25, centerX + 0.25, getMills(startHour),
                                         getMills(endHour)) ;
  }

  private long getMills(int hour) {
    calendar.set(Calendar.HOUR_OF_DAY, hour) ;
    return calendar.getTimeInMillis() ;
  }

}
