package demo.miscellaneous.annotations ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.GridLayout ;
import java.awt.Paint ;
import java.awt.Rectangle ;
import java.awt.Shape ;
import java.awt.geom.Rectangle2D ;

import javax.swing.JPanel ;

import com.ijchart.xychart.annotation.XYShapeAnnotation ;
import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.MillisecondsToDateFormatter ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.ui.supplier.DefaultDrawingSupplier ;
import com.ijchart.xychart.ui.supplier.IDrawingSupplier ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYBoxAnnotationDemo1 extends JPanel {
  private ChartOrientation vertical = ChartOrientation.VERTICAL ;

  private final long halfDayMills = 12 * 60 * 60 * 1000L ;

  public XYBoxAnnotationDemo1() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Breakdowns", "Production Date",
                                                           "Hours of Operation", dataset, vertical,
                                                           true, true, true, false) ;

    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    xyplot.setDomainGridlineStroke(new BasicStroke(1.0F)) ;
    xyplot.setValueGridlineStroke(new BasicStroke(1.0F)) ;

    XYLineAndShapeRender render = (XYLineAndShapeRender) xyplot.getRender() ;
    render.setBaseItemLineVisible(false) ;
    IDrawingSupplier supplier = new DefaultDrawingSupplier() {
      private int shapeCount = 0 ;

      private int paintCount = 0 ;

      private Shape[] shapes = {new Rectangle(2, 2), new Rectangle(2, 2)} ;

      private Paint[] paints = {Color.blue, Color.red} ;

      public Shape getNextShape() {
        return shapes[shapeCount++ % shapes.length] ;
      }

      public Paint getNextFillPaint() {
        return paints[paintCount++ % paints.length] ;
      }

    } ;

    xyplot.setDrawingSupplier(supplier) ;
    XYNumberValueAxis xAxis = (XYNumberValueAxis) xyplot.getDomainAxis() ;
    xAxis.setTickUnit(new DateTickUnit(DateTickUnitType.DATE_UNIT_MONTH)) ;
    xAxis.setTickLabelAngle( -Math.PI / 2) ;
    xAxis.setTickValueFormatter(new MillisecondsToDateFormatter("MMM.yy")) ;

    XYNumberValueAxis yAxis = (XYNumberValueAxis) xyplot.getValueAxis() ;
    yAxis.setTickUnit(new LongTickUnit(50000L)) ;

    Color color = new Color(255, 0, 0, 60) ;
    Color color1 = new Color(0, 255, 0, 60) ;

    long t1 = DateUtil.dateToTimeInMillis(2004, 1, 1) + halfDayMills ;
    long t2 = DateUtil.dateToTimeInMillis(2004, 1, 31) + halfDayMills ;
    Rectangle2D rect = new Rectangle2D.Double(t1, 0, t2 - t1, 50000D) ;
    XYShapeAnnotation annotation = new XYShapeAnnotation(rect) ;
    annotation.setTooltipText("Value: 2.9") ;
    annotation.setFillPaint(color) ;
    annotation.setBorderPaint(color) ;
    annotation.setBorderStroke(new BasicStroke(0F)) ;
    render.addAnnotation(annotation) ;

    t1 = DateUtil.dateToTimeInMillis(2004, 2, 1) + halfDayMills ;
    t2 = DateUtil.dateToTimeInMillis(2004, 2, 29) + halfDayMills ;
    rect = new Rectangle2D.Double(t1, 0, t2 - t1, 50000D) ;
    XYShapeAnnotation annotation2 = new XYShapeAnnotation(rect) ;
    annotation2.setTooltipText("Value: 2.5") ;
    annotation2.setFillPaint(color) ;
    annotation2.setBorderPaint(color) ;
    annotation2.setBorderStroke(new BasicStroke(0F)) ;
    render.addAnnotation(annotation2) ;

    t1 = DateUtil.dateToTimeInMillis(2004, 5, 1) + halfDayMills ;
    t2 = DateUtil.dateToTimeInMillis(2004, 5, 31) + halfDayMills ;
    rect = new Rectangle2D.Double(t1, 50000, t2 - t1, 100000D - 50000) ;
    XYShapeAnnotation annotation3 = new XYShapeAnnotation(rect) ;
    annotation3.setTooltipText("Value: 1.8") ;
    annotation3.setFillPaint(color) ;
    annotation3.setBorderPaint(color) ;
    annotation3.setBorderStroke(new BasicStroke(0F)) ;
    render.addAnnotation(annotation3) ;

    t1 = DateUtil.dateToTimeInMillis(2005, 6, 1) + halfDayMills ;
    t2 = DateUtil.dateToTimeInMillis(2005, 6, 30) + halfDayMills ;
    rect = new Rectangle2D.Double(t1, 0, t2 - t1, 50000D) ;
    XYShapeAnnotation annotation4 = new XYShapeAnnotation(rect) ;
    annotation4.setTooltipText("Value: 3.7") ;
    annotation4.setFillPaint(color1) ;
    annotation4.setBorderPaint(color1) ;
    annotation4.setBorderStroke(new BasicStroke(0F)) ;
    render.addAnnotation(annotation4) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Old") ;
    dataset.addRowKey("New") ;
    dataset.addRowData(2) ;

    dataset.addData(0, createData(6, 11, 2003, 617D)) ;
    dataset.addData(0, createData(7, 11, 2003, 3065D)) ;
    dataset.addData(0, createData(14, 11, 2003, 1964D)) ;
    dataset.addData(0, createData(17, 12, 2003, 204D)) ;
    dataset.addData(0, createData(13, 11, 2003, 10320D)) ;
    dataset.addData(0, createData(5, 11, 2003, 17892D)) ;
    dataset.addData(0, createData(11, 12, 2003, 4269D)) ;
    dataset.addData(0, createData(12, 12, 2003, 48D)) ;
    dataset.addData(0, createData(19, 12, 2003, 7335D)) ;
    dataset.addData(0, createData(6, 11, 2003, 30887D)) ;
    dataset.addData(0, createData(8, 12, 2003, 7767D)) ;
    dataset.addData(0, createData(29, 11, 2003, 11701D)) ;
    dataset.addData(0, createData(7, 11, 2003, 2202D)) ;
    dataset.addData(0, createData(2, 2, 2004, 233D)) ;
    dataset.addData(0, createData(20, 1, 2004, 8572D)) ;
    dataset.addData(0, createData(26, 11, 2003, 16607D)) ;
    dataset.addData(0, createData(20, 11, 2003, 5945D)) ;
    dataset.addData(0, createData(16, 1, 2004, 1322D)) ;
    dataset.addData(0, createData(12, 1, 2004, 6600D)) ;
    dataset.addData(0, createData(27, 11, 2003, 19080D)) ;
    dataset.addData(0, createData(5, 11, 2003, 39000D)) ;
    dataset.addData(0, createData(25, 11, 2003, 28871D)) ;
    dataset.addData(0, createData(4, 2, 2004, 1387D)) ;
    dataset.addData(0, createData(19, 1, 2004, 1750D)) ;
    dataset.addData(0, createData(10, 12, 2003, 340D)) ;
    dataset.addData(0, createData(2, 2, 2004, 1100D)) ;
    dataset.addData(0, createData(11, 2, 2004, 1586D)) ;
    dataset.addData(0, createData(23, 1, 2004, 482D)) ;
    dataset.addData(0, createData(20, 2, 2004, 1520D)) ;
    dataset.addData(0, createData(19, 1, 2004, 213D)) ;
    dataset.addData(0, createData(8, 12, 2003, 7398D)) ;
    dataset.addData(0, createData(24, 1, 2004, 2903D)) ;
    dataset.addData(0, createData(14, 2, 2004, 724D)) ;
    dataset.addData(0, createData(3, 11, 2003, 58936D)) ;
    dataset.addData(0, createData(7, 11, 2003, 24494D)) ;
    dataset.addData(0, createData(25, 1, 2004, 994D)) ;
    dataset.addData(0, createData(15, 1, 2004, 6610D)) ;
    dataset.addData(0, createData(2, 2, 2004, 1081D)) ;
    dataset.addData(0, createData(22, 1, 2004, 4350D)) ;
    dataset.addData(0, createData(27, 11, 2003, 45009D)) ;
    dataset.addData(0, createData(3, 11, 2003, 23746D)) ;
    dataset.addData(0, createData(25, 11, 2003, 3082D)) ;
    dataset.addData(0, createData(24, 1, 2004, 12425D)) ;
    dataset.addData(0, createData(4, 12, 2003, 90D)) ;
    dataset.addData(0, createData(7, 11, 2003, 6500D)) ;
    dataset.addData(0, createData(2, 12, 2003, 36350D)) ;
    dataset.addData(0, createData(10, 2, 2004, 7983D)) ;
    dataset.addData(0, createData(16, 1, 2004, 7250D)) ;
    dataset.addData(0, createData(10, 2, 2004, 4881D)) ;
    dataset.addData(0, createData(25, 2, 2004, 6616D)) ;
    dataset.addData(0, createData(20, 1, 2004, 4700D)) ;
    dataset.addData(0, createData(12, 12, 2003, 47317D)) ;
    dataset.addData(0, createData(24, 3, 2004, 314D)) ;
    dataset.addData(0, createData(4, 12, 2003, 20885D)) ;
    dataset.addData(0, createData(7, 1, 2004, 13477D)) ;
    dataset.addData(0, createData(19, 1, 2004, 23527D)) ;
    dataset.addData(0, createData(26, 1, 2004, 22224D)) ;
    dataset.addData(0, createData(2, 2, 2004, 17467D)) ;
    dataset.addData(0, createData(20, 2, 2004, 5474D)) ;
    dataset.addData(0, createData(18, 11, 2003, 14897D)) ;
    dataset.addData(0, createData(9, 1, 2004, 2459D)) ;
    dataset.addData(0, createData(20, 2, 2004, 5712D)) ;
    dataset.addData(0, createData(13, 3, 2004, 1796D)) ;
    dataset.addData(0, createData(6, 4, 2004, 258D)) ;
    dataset.addData(0, createData(31, 1, 2004, 19701D)) ;
    dataset.addData(0, createData(3, 2, 2004, 5936D)) ;
    dataset.addData(0, createData(2, 2, 2004, 22601D)) ;
    dataset.addData(0, createData(8, 4, 2004, 1977D)) ;
    dataset.addData(0, createData(2, 2, 2004, 17801D)) ;
    dataset.addData(0, createData(2, 2, 2004, 17321D)) ;
    dataset.addData(0, createData(6, 2, 2004, 17261D)) ;
    dataset.addData(0, createData(2, 4, 2004, 3091D)) ;
    dataset.addData(0, createData(2, 2, 2004, 17901D)) ;
    dataset.addData(0, createData(2, 2, 2004, 14236D)) ;
    dataset.addData(0, createData(4, 2, 2004, 18601D)) ;
    dataset.addData(0, createData(7, 2, 2004, 18901D)) ;
    dataset.addData(0, createData(9, 2, 2004, 18722D)) ;
    dataset.addData(0, createData(4, 3, 2004, 8812D)) ;
    dataset.addData(0, createData(18, 11, 2003, 42080D)) ;
    dataset.addData(0, createData(8, 3, 2004, 8988D)) ;
    dataset.addData(0, createData(15, 3, 2004, 654D)) ;
    dataset.addData(0, createData(25, 3, 2004, 704D)) ;
    dataset.addData(0, createData(24, 1, 2004, 267D)) ;
    dataset.addData(0, createData(1, 3, 2004, 770D)) ;
    dataset.addData(0, createData(16, 2, 2004, 19855D)) ;
    dataset.addData(0, createData(15, 12, 2003, 40548D)) ;
    dataset.addData(0, createData(21, 2, 2004, 23701D)) ;
    dataset.addData(0, createData(19, 11, 2003, 2604D)) ;
    dataset.addData(0, createData(26, 1, 2004, 31154D)) ;
    dataset.addData(0, createData(11, 2, 2004, 21423D)) ;
    dataset.addData(0, createData(10, 12, 2003, 22132D)) ;
    dataset.addData(0, createData(22, 4, 2004, 5423D)) ;
    dataset.addData(0, createData(19, 11, 2003, 2626D)) ;
    dataset.addData(0, createData(2, 3, 2004, 6700D)) ;
    dataset.addData(0, createData(14, 1, 2004, 332D)) ;
    dataset.addData(0, createData(5, 4, 2004, 4980D)) ;
    dataset.addData(0, createData(31, 1, 2004, 22655D)) ;
    dataset.addData(0, createData(9, 2, 2004, 26510D)) ;
    dataset.addData(0, createData(27, 11, 2003, 44228D)) ;
    dataset.addData(0, createData(4, 3, 2004, 2406D)) ;
    dataset.addData(0, createData(2, 2, 2004, 34801D)) ;
    dataset.addData(0, createData(13, 2, 2004, 13332D)) ;
    dataset.addData(0, createData(19, 4, 2004, 2722D)) ;
    dataset.addData(0, createData(18, 2, 2004, 31212D)) ;
    dataset.addData(0, createData(4, 3, 2004, 17351D)) ;
    dataset.addData(0, createData(15, 3, 2004, 12784D)) ;
    dataset.addData(0, createData(5, 12, 2003, 24157D)) ;
    dataset.addData(0, createData(12, 3, 2004, 1053D)) ;
    dataset.addData(0, createData(18, 12, 2003, 56533D)) ;
    dataset.addData(0, createData(19, 12, 2003, 38394D)) ;
    dataset.addData(0, createData(19, 12, 2003, 21997D)) ;
    dataset.addData(0, createData(2, 2, 2004, 31113D)) ;
    dataset.addData(0, createData(17, 12, 2003, 54535D)) ;
    dataset.addData(0, createData(8, 3, 2004, 17451D)) ;
    dataset.addData(0, createData(8, 3, 2004, 17451D)) ;
    dataset.addData(0, createData(4, 2, 2004, 1070D)) ;
    dataset.addData(0, createData(4, 11, 2003, 14501D)) ;
    dataset.addData(0, createData(4, 2, 2004, 11119D)) ;
    dataset.addData(0, createData(19, 4, 2004, 2059D)) ;
    dataset.addData(0, createData(16, 1, 2004, 12268D)) ;
    dataset.addData(0, createData(22, 12, 2003, 44210D)) ;
    dataset.addData(0, createData(7, 11, 2003, 1240D)) ;
    dataset.addData(0, createData(10, 2, 2004, 32734D)) ;
    dataset.addData(0, createData(19, 12, 2003, 85800D)) ;
    dataset.addData(0, createData(18, 11, 2003, 21576D)) ;
    dataset.addData(0, createData(24, 5, 2004, 3978D)) ;
    dataset.addData(0, createData(13, 1, 2004, 4526D)) ;
    dataset.addData(0, createData(8, 3, 2004, 12463D)) ;
    dataset.addData(0, createData(2, 2, 2004, 48991D)) ;
    dataset.addData(0, createData(15, 12, 2003, 72193D)) ;
    dataset.addData(0, createData(4, 12, 2003, 89024D)) ;
    dataset.addData(0, createData(6, 5, 2004, 2001D)) ;
    dataset.addData(0, createData(2, 2, 2004, 30610D)) ;
    dataset.addData(0, createData(4, 3, 2004, 246D)) ;
    dataset.addData(0, createData(27, 11, 2003, 63135D)) ;
    dataset.addData(0, createData(1, 3, 2004, 38865D)) ;
    dataset.addData(0, createData(3, 5, 2004, 389D)) ;
    dataset.addData(0, createData(28, 4, 2004, 3723D)) ;
    dataset.addData(0, createData(10, 12, 2003, 58745D)) ;
    dataset.addData(0, createData(18, 5, 2004, 2635D)) ;
    dataset.addData(0, createData(11, 3, 2004, 348D)) ;
    dataset.addData(0, createData(31, 3, 2004, 5085D)) ;
    dataset.addData(0, createData(2, 2, 2004, 40608D)) ;
    dataset.addData(0, createData(25, 2, 2004, 9864D)) ;
    dataset.addData(0, createData(6, 5, 2004, 20239D)) ;
    dataset.addData(0, createData(14, 6, 2004, 280D)) ;
    dataset.addData(0, createData(28, 4, 2004, 14873D)) ;
    dataset.addData(0, createData(16, 12, 2003, 30845D)) ;
    dataset.addData(0, createData(19, 5, 2004, 729D)) ;
    dataset.addData(0, createData(29, 1, 2004, 983D)) ;
    dataset.addData(0, createData(8, 4, 2004, 619D)) ;
    dataset.addData(0, createData(4, 11, 2003, 18037D)) ;
    dataset.addData(0, createData(2, 12, 2003, 9772D)) ;
    dataset.addData(0, createData(15, 1, 2004, 125837D)) ;
    dataset.addData(0, createData(27, 1, 2004, 84704D)) ;
    dataset.addData(0, createData(27, 1, 2004, 10400D)) ;
    dataset.addData(0, createData(18, 5, 2004, 16425D)) ;
    dataset.addData(0, createData(6, 4, 2004, 30D)) ;
    dataset.addData(0, createData(27, 2, 2004, 104978D)) ;
    dataset.addData(0, createData(5, 3, 2004, 17359D)) ;
    dataset.addData(0, createData(7, 4, 2004, 55074D)) ;
    dataset.addData(0, createData(19, 12, 2003, 55630D)) ;
    dataset.addData(0, createData(10, 12, 2003, 47055D)) ;
    dataset.addData(0, createData(8, 3, 2004, 16900D)) ;
    dataset.addData(0, createData(26, 1, 2004, 34819D)) ;
    dataset.addData(0, createData(21, 6, 2004, 514D)) ;
    dataset.addData(0, createData(28, 5, 2004, 8195D)) ;
    dataset.addData(0, createData(20, 3, 2004, 61870D)) ;
    dataset.addData(0, createData(13, 3, 2004, 13000D)) ;
    dataset.addData(0, createData(20, 1, 2004, 43152D)) ;
    dataset.addData(0, createData(5, 3, 2004, 49653D)) ;
    dataset.addData(0, createData(25, 1, 2004, 23311D)) ;
    dataset.addData(0, createData(14, 11, 2003, 40166D)) ;
    dataset.addData(0, createData(1, 12, 2003, 59504D)) ;
    dataset.addData(0, createData(15, 1, 2004, 113349D)) ;
    dataset.addData(0, createData(5, 6, 2004, 32110D)) ;
    dataset.addData(0, createData(25, 1, 2004, 24693D)) ;
    dataset.addData(0, createData(25, 2, 2004, 11668D)) ;
    dataset.addData(0, createData(23, 4, 2004, 651D)) ;
    dataset.addData(0, createData(3, 3, 2004, 24409D)) ;
    dataset.addData(0, createData(11, 3, 2004, 43478D)) ;
    dataset.addData(0, createData(24, 3, 2004, 13000D)) ;
    dataset.addData(0, createData(9, 7, 2004, 4370D)) ;
    dataset.addData(0, createData(12, 1, 2004, 67648D)) ;
    dataset.addData(0, createData(5, 4, 2004, 44427D)) ;
    dataset.addData(0, createData(28, 4, 2004, 35210D)) ;
    dataset.addData(0, createData(30, 6, 2004, 784D)) ;
    dataset.addData(0, createData(8, 4, 2004, 41645D)) ;
    dataset.addData(0, createData(1, 12, 2003, 46919D)) ;
    dataset.addData(0, createData(4, 6, 2004, 19853D)) ;
    dataset.addData(0, createData(6, 5, 2004, 18070D)) ;
    dataset.addData(0, createData(15, 1, 2004, 5213D)) ;
    dataset.addData(0, createData(14, 11, 2003, 47288D)) ;
    dataset.addData(0, createData(24, 3, 2004, 18262D)) ;
    dataset.addData(0, createData(2, 12, 2003, 63186D)) ;
    dataset.addData(0, createData(30, 7, 2004, 8980D)) ;
    dataset.addData(0, createData(22, 6, 2004, 1627D)) ;
    dataset.addData(0, createData(8, 12, 2003, 33833D)) ;
    dataset.addData(0, createData(5, 3, 2004, 68608D)) ;
    dataset.addData(0, createData(12, 5, 2004, 785D)) ;
    dataset.addData(0, createData(30, 7, 2004, 12055D)) ;
    dataset.addData(0, createData(11, 6, 2004, 11902D)) ;
    dataset.addData(0, createData(9, 8, 2004, 102D)) ;
    dataset.addData(0, createData(19, 1, 2004, 69549D)) ;
    dataset.addData(0, createData(27, 5, 2004, 4037D)) ;
    dataset.addData(0, createData(21, 11, 2003, 94967D)) ;
    dataset.addData(0, createData(19, 11, 2003, 99063D)) ;
    dataset.addData(0, createData(26, 4, 2004, 365D)) ;
    dataset.addData(0, createData(5, 4, 2004, 30681D)) ;
    dataset.addData(0, createData(6, 7, 2004, 15798D)) ;
    dataset.addData(0, createData(13, 3, 2004, 111394D)) ;
    dataset.addData(0, createData(27, 5, 2004, 781D)) ;
    dataset.addData(0, createData(9, 6, 2004, 30862D)) ;
    dataset.addData(0, createData(7, 4, 2004, 58378D)) ;
    dataset.addData(0, createData(10, 11, 2003, 27414D)) ;
    dataset.addData(0, createData(28, 7, 2004, 1264D)) ;
    dataset.addData(0, createData(27, 5, 2004, 3700D)) ;
    dataset.addData(0, createData(30, 7, 2004, 23915D)) ;
    dataset.addData(0, createData(21, 5, 2004, 3140D)) ;
    dataset.addData(0, createData(13, 8, 2004, 9925D)) ;
    dataset.addData(0, createData(24, 5, 2004, 45225D)) ;
    dataset.addData(0, createData(3, 2, 2004, 2816D)) ;
    dataset.addData(0, createData(11, 2, 2004, 84916D)) ;
    dataset.addData(0, createData(19, 4, 2004, 10025D)) ;
    dataset.addData(0, createData(27, 11, 2003, 25607D)) ;
    dataset.addData(0, createData(21, 6, 2004, 6961D)) ;
    dataset.addData(0, createData(13, 8, 2004, 758D)) ;
    dataset.addData(0, createData(11, 2, 2004, 53802D)) ;
    dataset.addData(0, createData(19, 12, 2003, 125030D)) ;
    dataset.addData(0, createData(4, 10, 2004, 3056D)) ;
    dataset.addData(0, createData(13, 8, 2004, 1700D)) ;
    dataset.addData(0, createData(11, 5, 2004, 12967D)) ;
    dataset.addData(0, createData(15, 12, 2003, 4549D)) ;
    dataset.addData(0, createData(17, 11, 2003, 152336D)) ;
    dataset.addData(0, createData(15, 12, 2003, 110483D)) ;
    dataset.addData(0, createData(22, 6, 2004, 27274D)) ;
    dataset.addData(0, createData(28, 9, 2004, 8161D)) ;
    dataset.addData(0, createData(27, 11, 2003, 103805D)) ;
    dataset.addData(0, createData(11, 10, 2004, 32D)) ;
    dataset.addData(0, createData(19, 1, 2004, 1847D)) ;
    dataset.addData(0, createData(11, 6, 2004, 40917D)) ;
    dataset.addData(0, createData(13, 8, 2004, 6125D)) ;
    dataset.addData(0, createData(8, 10, 2004, 7822D)) ;
    dataset.addData(0, createData(10, 9, 2004, 28213D)) ;
    dataset.addData(0, createData(16, 9, 2004, 15285D)) ;
    dataset.addData(0, createData(18, 10, 2004, 4015D)) ;
    dataset.addData(0, createData(25, 8, 2004, 13929D)) ;
    dataset.addData(0, createData(20, 3, 2004, 55132D)) ;
    dataset.addData(0, createData(26, 10, 2004, 3473D)) ;
    dataset.addData(0, createData(27, 10, 2004, 2D)) ;
    dataset.addData(0, createData(11, 6, 2004, 34557D)) ;
    dataset.addData(0, createData(4, 5, 2004, 55015D)) ;
    dataset.addData(0, createData(7, 6, 2004, 1750D)) ;
    dataset.addData(0, createData(23, 2, 2004, 109978D)) ;
    dataset.addData(0, createData(13, 8, 2004, 43759D)) ;
    dataset.addData(0, createData(12, 8, 2004, 42897D)) ;
    dataset.addData(0, createData(10, 3, 2004, 91480D)) ;
    dataset.addData(0, createData(19, 2, 2004, 141200D)) ;
    dataset.addData(0, createData(28, 5, 2004, 5437D)) ;
    dataset.addData(0, createData(9, 8, 2004, 2750D)) ;
    dataset.addData(0, createData(13, 4, 2004, 25170D)) ;
    dataset.addData(0, createData(12, 5, 2004, 45996D)) ;
    dataset.addData(0, createData(17, 12, 2003, 61903D)) ;
    dataset.addData(0, createData(15, 4, 2004, 75129D)) ;
    dataset.addData(0, createData(26, 10, 2004, 15217D)) ;
    dataset.addData(0, createData(27, 7, 2004, 15352D)) ;
    dataset.addData(0, createData(5, 5, 2004, 74223D)) ;
    dataset.addData(0, createData(27, 2, 2004, 97847D)) ;
    dataset.addData(0, createData(24, 11, 2003, 13082D)) ;
    dataset.addData(0, createData(17, 11, 2004, 4019D)) ;
    dataset.addData(0, createData(25, 10, 2004, 186D)) ;
    dataset.addData(0, createData(7, 1, 2004, 10595D)) ;
    dataset.addData(0, createData(4, 11, 2004, 240D)) ;
    dataset.addData(0, createData(9, 8, 2004, 35085D)) ;
    dataset.addData(0, createData(17, 7, 2004, 59925D)) ;
    dataset.addData(0, createData(17, 8, 2004, 43200D)) ;
    dataset.addData(0, createData(3, 11, 2004, 217D)) ;
    dataset.addData(0, createData(30, 10, 2004, 637D)) ;
    dataset.addData(0, createData(20, 9, 2004, 12296D)) ;
    dataset.addData(0, createData(17, 11, 2004, 4299D)) ;
    dataset.addData(0, createData(29, 11, 2004, 3137D)) ;
    dataset.addData(0, createData(6, 5, 2004, 62702D)) ;
    dataset.addData(0, createData(24, 6, 2004, 23243D)) ;
    dataset.addData(0, createData(9, 9, 2004, 873D)) ;
    dataset.addData(0, createData(8, 9, 2004, 23100D)) ;
    dataset.addData(0, createData(20, 8, 2004, 207D)) ;
    dataset.addData(0, createData(1, 4, 2004, 105779D)) ;
    dataset.addData(0, createData(30, 3, 2004, 69874D)) ;
    dataset.addData(0, createData(8, 9, 2004, 19287D)) ;
    dataset.addData(0, createData(26, 7, 2004, 59130D)) ;
    dataset.addData(0, createData(14, 9, 2004, 13343D)) ;
    dataset.addData(0, createData(3, 3, 2004, 89040D)) ;
    dataset.addData(0, createData(13, 12, 2004, 4752D)) ;
    dataset.addData(0, createData(16, 1, 2004, 2321D)) ;
    dataset.addData(0, createData(26, 4, 2004, 24831D)) ;
    dataset.addData(0, createData(15, 1, 2004, 3390D)) ;
    dataset.addData(0, createData(8, 11, 2004, 2340D)) ;
    dataset.addData(0, createData(15, 12, 2004, 1553D)) ;
    dataset.addData(0, createData(21, 9, 2004, 7186D)) ;
    dataset.addData(0, createData(30, 10, 2004, 16444D)) ;
    dataset.addData(0, createData(26, 5, 2004, 54688D)) ;
    dataset.addData(0, createData(13, 10, 2004, 12562D)) ;
    dataset.addData(0, createData(7, 10, 2004, 4023D)) ;
    dataset.addData(0, createData(11, 11, 2004, 12999D)) ;
    dataset.addData(0, createData(25, 10, 2004, 5378D)) ;
    dataset.addData(0, createData(29, 6, 2004, 49340D)) ;
    dataset.addData(0, createData(14, 1, 2004, 22112D)) ;
    dataset.addData(0, createData(9, 2, 2004, 79733D)) ;
    dataset.addData(0, createData(23, 9, 2004, 490D)) ;
    dataset.addData(0, createData(13, 11, 2004, 59D)) ;
    dataset.addData(0, createData(13, 10, 2004, 31160D)) ;
    dataset.addData(0, createData(18, 2, 2004, 104655D)) ;
    dataset.addData(0, createData(26, 7, 2004, 2956D)) ;
    dataset.addData(0, createData(10, 12, 2004, 80D)) ;
    dataset.addData(0, createData(13, 3, 2004, 192969D)) ;
    dataset.addData(0, createData(9, 8, 2004, 45940D)) ;
    dataset.addData(0, createData(8, 4, 2004, 109268D)) ;
    dataset.addData(0, createData(4, 10, 2004, 12716D)) ;
    dataset.addData(0, createData(9, 12, 2004, 8430D)) ;
    dataset.addData(0, createData(18, 11, 2004, 24994D)) ;
    dataset.addData(0, createData(17, 3, 2004, 96378D)) ;
    dataset.addData(0, createData(20, 12, 2004, 11023D)) ;
    dataset.addData(0, createData(20, 12, 2004, 11023D)) ;
    dataset.addData(0, createData(14, 6, 2004, 49043D)) ;
    dataset.addData(0, createData(6, 11, 2004, 21168D)) ;
    dataset.addData(0, createData(4, 9, 2004, 64336D)) ;
    dataset.addData(0, createData(12, 1, 2005, 522D)) ;
    dataset.addData(0, createData(17, 6, 2004, 84255D)) ;
    dataset.addData(0, createData(11, 11, 2004, 4106D)) ;
    dataset.addData(0, createData(13, 9, 2004, 69040D)) ;
    dataset.addData(0, createData(9, 6, 2004, 44534D)) ;
    dataset.addData(0, createData(29, 10, 2004, 1556D)) ;
    dataset.addData(0, createData(21, 9, 2004, 28048D)) ;
    dataset.addData(0, createData(14, 1, 2005, 482D)) ;
    dataset.addData(0, createData(12, 2, 2004, 137490D)) ;
    dataset.addData(0, createData(9, 10, 2004, 19965D)) ;
    dataset.addData(0, createData(5, 1, 2005, 1625D)) ;
    dataset.addData(0, createData(12, 11, 2004, 17972D)) ;
    dataset.addData(0, createData(23, 9, 2004, 23591D)) ;
    dataset.addData(0, createData(8, 4, 2004, 44562D)) ;
    dataset.addData(0, createData(4, 10, 2004, 40126D)) ;
    dataset.addData(0, createData(12, 8, 2004, 84683D)) ;
    dataset.addData(0, createData(9, 11, 2004, 11943D)) ;
    dataset.addData(0, createData(27, 7, 2004, 114763D)) ;
    dataset.addData(0, createData(9, 8, 2004, 61353D)) ;
    dataset.addData(0, createData(4, 1, 2005, 8746D)) ;
    dataset.addData(0, createData(13, 10, 2004, 43273D)) ;
    dataset.addData(0, createData(17, 9, 2004, 36157D)) ;
    dataset.addData(0, createData(8, 12, 2004, 19701D)) ;
    dataset.addData(0, createData(2, 6, 2004, 58987D)) ;
    dataset.addData(0, createData(6, 7, 2004, 17320D)) ;
    dataset.addData(0, createData(18, 12, 2004, 6923D)) ;
    dataset.addData(0, createData(13, 3, 2004, 49538D)) ;
    dataset.addData(0, createData(20, 1, 2005, 350D)) ;
    dataset.addData(0, createData(19, 3, 2004, 45218D)) ;
    dataset.addData(0, createData(16, 12, 2004, 9122D)) ;
    dataset.addData(0, createData(12, 1, 2004, 150321D)) ;
    dataset.addData(0, createData(17, 11, 2004, 44811D)) ;
    dataset.addData(0, createData(24, 11, 2004, 5229D)) ;
    dataset.addData(0, createData(19, 5, 2004, 103090D)) ;
    dataset.addData(0, createData(9, 6, 2004, 57797D)) ;
    dataset.addData(0, createData(29, 6, 2004, 20401D)) ;
    dataset.addData(0, createData(15, 10, 2004, 22720D)) ;
    dataset.addData(0, createData(3, 2, 2005, 7430D)) ;
    dataset.addData(0, createData(16, 8, 2004, 1450D)) ;
    dataset.addData(0, createData(2, 11, 2004, 44595D)) ;
    dataset.addData(0, createData(11, 2, 2004, 165301D)) ;
    dataset.addData(0, createData(28, 6, 2004, 70065D)) ;
    dataset.addData(0, createData(23, 6, 2004, 18394D)) ;
    dataset.addData(0, createData(26, 7, 2004, 45087D)) ;
    dataset.addData(0, createData(20, 10, 2004, 6971D)) ;
    dataset.addData(0, createData(29, 11, 2004, 971D)) ;
    dataset.addData(0, createData(2, 3, 2005, 4241D)) ;
    dataset.addData(0, createData(7, 10, 2004, 2671D)) ;
    dataset.addData(0, createData(14, 4, 2004, 78014D)) ;
    dataset.addData(0, createData(13, 8, 2004, 81107D)) ;
    dataset.addData(0, createData(15, 11, 2004, 37558D)) ;
    dataset.addData(0, createData(2, 3, 2005, 6128D)) ;
    dataset.addData(0, createData(17, 12, 2004, 6478D)) ;
    dataset.addData(0, createData(27, 11, 2003, 39804D)) ;
    dataset.addData(0, createData(13, 4, 2004, 25591D)) ;
    dataset.addData(0, createData(18, 5, 2004, 2297D)) ;
    dataset.addData(0, createData(2, 2, 2005, 3785D)) ;
    dataset.addData(0, createData(18, 8, 2004, 5256D)) ;
    dataset.addData(0, createData(24, 2, 2005, 6100D)) ;
    dataset.addData(0, createData(13, 5, 2004, 86738D)) ;
    dataset.addData(0, createData(31, 3, 2005, 278D)) ;
    dataset.addData(0, createData(6, 8, 2004, 110455D)) ;
    dataset.addData(0, createData(30, 9, 2004, 99828D)) ;
    dataset.addData(0, createData(22, 10, 2004, 64827D)) ;
    dataset.addData(0, createData(13, 10, 2004, 149269D)) ;
    dataset.addData(0, createData(16, 8, 2004, 5948D)) ;
    dataset.addData(0, createData(13, 4, 2005, 6025D)) ;
    dataset.addData(0, createData(28, 2, 2005, 7755D)) ;
    dataset.addData(0, createData(8, 12, 2004, 44226D)) ;
    dataset.addData(0, createData(8, 7, 2004, 89481D)) ;
    dataset.addData(0, createData(19, 5, 2004, 85650D)) ;
    dataset.addData(0, createData(28, 9, 2004, 50713D)) ;
    dataset.addData(0, createData(13, 8, 2004, 94151D)) ;
    dataset.addData(0, createData(20, 1, 2005, 38905D)) ;
    dataset.addData(0, createData(7, 4, 2005, 437D)) ;
    dataset.addData(0, createData(31, 1, 2004, 155322D)) ;
    dataset.addData(0, createData(26, 10, 2004, 80867D)) ;
    dataset.addData(0, createData(10, 1, 2005, 22530D)) ;
    dataset.addData(0, createData(18, 3, 2005, 7280D)) ;
    dataset.addData(0, createData(3, 3, 2005, 199D)) ;
    dataset.addData(0, createData(13, 9, 2004, 93475D)) ;
    dataset.addData(0, createData(11, 8, 2004, 74984D)) ;
    dataset.addData(0, createData(21, 12, 2004, 28153D)) ;
    dataset.addData(0, createData(18, 3, 2005, 6725D)) ;
    dataset.addData(0, createData(5, 4, 2005, 2095D)) ;
    dataset.addData(0, createData(15, 7, 2004, 140022D)) ;
    dataset.addData(0, createData(12, 11, 2004, 63582D)) ;
    dataset.addData(0, createData(2, 4, 2004, 5117D)) ;
    dataset.addData(0, createData(17, 5, 2004, 93320D)) ;
    dataset.addData(0, createData(15, 12, 2004, 20393D)) ;
    dataset.addData(0, createData(1, 3, 2004, 29900D)) ;
    dataset.addData(0, createData(27, 4, 2005, 832D)) ;
    dataset.addData(0, createData(15, 6, 2004, 69073D)) ;
    dataset.addData(0, createData(20, 5, 2005, 2625D)) ;
    dataset.addData(0, createData(21, 4, 2005, 2956D)) ;
    dataset.addData(0, createData(12, 2, 2005, 881D)) ;
    dataset.addData(0, createData(8, 10, 2004, 95544D)) ;
    dataset.addData(0, createData(9, 8, 2004, 103419D)) ;
    dataset.addData(0, createData(23, 6, 2004, 99280D)) ;
    dataset.addData(0, createData(27, 8, 2004, 47822D)) ;
    dataset.addData(0, createData(9, 3, 2005, 1336D)) ;
    dataset.addData(0, createData(23, 7, 2004, 78313D)) ;
    dataset.addData(0, createData(26, 10, 2004, 111320D)) ;
    dataset.addData(0, createData(3, 11, 2004, 6187D)) ;
    dataset.addData(0, createData(18, 8, 2004, 88757D)) ;
    dataset.addData(0, createData(8, 11, 2004, 64621D)) ;
    dataset.addData(0, createData(13, 4, 2005, 5005D)) ;
    dataset.addData(0, createData(14, 1, 2004, 18246D)) ;
    dataset.addData(0, createData(15, 10, 2004, 50016D)) ;
    dataset.addData(0, createData(24, 3, 2005, 24179D)) ;
    dataset.addData(0, createData(4, 3, 2005, 24203D)) ;
    dataset.addData(0, createData(1, 3, 2005, 6079D)) ;
    dataset.addData(0, createData(19, 11, 2004, 77738D)) ;
    dataset.addData(0, createData(28, 9, 2004, 49283D)) ;
    dataset.addData(0, createData(24, 11, 2004, 4880D)) ;
    dataset.addData(0, createData(25, 2, 2005, 30705D)) ;
    dataset.addData(0, createData(22, 10, 2004, 89693D)) ;
    dataset.addData(0, createData(1, 4, 2005, 2865D)) ;
    dataset.addData(0, createData(20, 1, 2005, 46560D)) ;
    dataset.addData(0, createData(4, 1, 2005, 16570D)) ;
    dataset.addData(0, createData(13, 4, 2005, 2602D)) ;
    dataset.addData(0, createData(17, 6, 2005, 551D)) ;
    dataset.addData(0, createData(22, 10, 2004, 103984D)) ;
    dataset.addData(0, createData(4, 1, 2005, 18078D)) ;
    dataset.addData(0, createData(7, 4, 2005, 11146D)) ;
    dataset.addData(0, createData(22, 4, 2005, 2266D)) ;
    dataset.addData(0, createData(18, 5, 2004, 78472D)) ;
    dataset.addData(0, createData(22, 10, 2004, 83375D)) ;
    dataset.addData(0, createData(25, 8, 2004, 1265D)) ;
    dataset.addData(0, createData(7, 4, 2005, 380D)) ;
    dataset.addData(0, createData(20, 4, 2005, 30071D)) ;
    dataset.addData(0, createData(2, 2, 2005, 2020D)) ;
    dataset.addData(0, createData(24, 3, 2005, 30547D)) ;
    dataset.addData(1, createData(22, 2, 2005, 57743D)) ;
    dataset.addData(1, createData(5, 11, 2003, 21135D)) ;
    dataset.addData(1, createData(28, 5, 2005, 4988D)) ;
    dataset.addData(1, createData(5, 4, 2004, 51760D)) ;
    dataset.addData(1, createData(16, 3, 2005, 2283D)) ;
    dataset.addData(1, createData(16, 3, 2005, 12927D)) ;
    dataset.addData(1, createData(26, 4, 2005, 9223D)) ;
    dataset.addData(1, createData(15, 12, 2004, 88625D)) ;
    dataset.addData(1, createData(19, 10, 2004, 106741D)) ;
    dataset.addData(1, createData(11, 7, 2005, 1802D)) ;
    dataset.addData(1, createData(20, 1, 2005, 43372D)) ;
    dataset.addData(1, createData(30, 3, 2005, 67224D)) ;
    dataset.addData(1, createData(6, 6, 2005, 8204D)) ;
    dataset.addData(1, createData(2, 3, 2005, 38504D)) ;
    dataset.addData(1, createData(1, 1, 2005, 47599D)) ;
    dataset.addData(1, createData(30, 9, 2004, 64959D)) ;
    dataset.addData(1, createData(7, 5, 2005, 30701D)) ;
    dataset.addData(1, createData(3, 5, 2004, 150586D)) ;
    dataset.addData(1, createData(5, 8, 2005, 713D)) ;
    dataset.addData(1, createData(21, 6, 2005, 8091D)) ;
    dataset.addData(1, createData(21, 6, 2005, 891D)) ;
    dataset.addData(1, createData(8, 7, 2005, 5940D)) ;
    dataset.addData(1, createData(18, 6, 2005, 453D)) ;
    dataset.addData(1, createData(11, 8, 2005, 715D)) ;
    dataset.addData(1, createData(23, 7, 2004, 45792D)) ;
    dataset.addData(1, createData(11, 11, 2004, 83422D)) ;
    dataset.addData(1, createData(3, 5, 2004, 216825D)) ;
    dataset.addData(1, createData(26, 7, 2004, 3443D)) ;
    dataset.addData(1, createData(25, 6, 2005, 21288D)) ;
    dataset.addData(1, createData(28, 6, 2004, 64987D)) ;
    dataset.addData(1, createData(15, 7, 2005, 113D)) ;
    dataset.addData(1, createData(1, 12, 2004, 18000D)) ;
    dataset.addData(1, createData(22, 6, 2005, 23581D)) ;
    dataset.addData(1, createData(13, 1, 2005, 90667D)) ;
    dataset.addData(1, createData(24, 6, 2004, 11599D)) ;
    dataset.addData(1, createData(8, 12, 2004, 47985D)) ;
    dataset.addData(1, createData(27, 6, 2005, 6545D)) ;
    dataset.addData(1, createData(10, 6, 2005, 29100D)) ;
    dataset.addData(1, createData(31, 8, 2004, 388D)) ;
    dataset.addData(1, createData(10, 12, 2004, 73404D)) ;
    dataset.addData(1, createData(16, 8, 2005, 516D)) ;
    dataset.addData(1, createData(17, 12, 2004, 34289D)) ;
    dataset.addData(1, createData(19, 10, 2004, 53409D)) ;
    dataset.addData(1, createData(6, 1, 2005, 102640D)) ;
    dataset.addData(1, createData(22, 4, 2005, 1741D)) ;
    dataset.addData(1, createData(1, 10, 2004, 104899D)) ;
    dataset.addData(1, createData(29, 7, 2005, 785D)) ;
    dataset.addData(1, createData(20, 6, 2005, 24834D)) ;
    dataset.addData(1, createData(10, 6, 2005, 34197D)) ;

    return dataset ;
  }

  private XYNumberChartData createData(int day, int month, int year, double value) {
    long mills = halfDayMills + DateUtil.dateToTimeInMillis(year, month, day) ;
    return new XYNumberChartData(mills, value) ;
  }
}
