package demo.timeseriescharts;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.GridLayout ;
import java.util.Calendar ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.YIntervalValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.YIntervalChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.DeviationRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class DeviationRenderDemo2 extends JPanel {

  public DeviationRenderDemo2() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createDeviationChart("Projected Values - Test", "Date",
                                                      "Index Projection", dataset,
                                                      ChartOrientation.VERTICAL,
                                                      true, true, true, false) ;
    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    xyplot.setDomainGridlinePaint(Color.white) ;
    xyplot.setValueGridlinePaint(Color.white) ;

    YIntervalValueAxis yAxis = (YIntervalValueAxis) xyplot.getValueAxis() ;
    yAxis.setAxisRangeIncludeZero(false) ;
    yAxis.setTickUnit(new LongTickUnit()) ;

    DeviationRender render = (DeviationRender) xyplot.getRender() ;
    render.setBaseItemShapeVisible(false) ;
    render.setRowItemBorderStroke(0, new BasicStroke(3F, 1, 1)) ;
    render.setRowItemBorderStroke(1, new BasicStroke(3F, 1, 1)) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Series 1") ;
    dataset.addRowKey("Series 2") ;
    dataset.addRowData(2) ;

    double d = 100D ;
    double d1 = 100D ;
    Calendar cal = Calendar.getInstance() ;
    for (int i = 0 ; i <= 52 ; i++) {
      int year = cal.get(Calendar.YEAR) ;
      int month = cal.get(Calendar.MONTH) + 1 ;
      int day = cal.get(Calendar.DAY_OF_MONTH) ;

      double d2 = 0.050000000000000003D * (double) i ;
      dataset.addData(0, createData(day, month, year, d, d - d2, d + d2)) ;
      d = (d + Math.random()) - 0.45000000000000001D ;
      double d3 = 0.070000000000000007D * (double) i ;
      dataset.addData(1, createData(day, month, year, d1, d1 - d3, d1 + d3)) ;
      d1 = (d1 + Math.random()) - 0.55000000000000004D ;

      cal.add(Calendar.DAY_OF_MONTH, 7) ;
    }

    return dataset ;
  }

  private static YIntervalChartData createData(int day, int month, int year,
                                               double y, double yLow, double yHigh) {
    return new YIntervalChartData(DateUtil.dateToTimeInMillis(year, month, day), y, yLow, yHigh) ;
  }
}
