package com.ijchart.xychart.axis ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.util.Arrays ;
import java.util.EventListener ;
import java.util.HashMap ;
import java.util.List ;
import java.util.Map ;

import javax.swing.event.EventListenerList ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.chartinfo.AxisShapeCollection ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.AxisLabelType ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.IAngle ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.event.AxisChangeEvent ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.listener.IAxisChangeListener ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LineChartShape ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.ui.IEffect3D ;
import com.ijchart.xychart.ui.Rectangle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.AngleUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StringUtil ;
import com.ijchart.xychart.util.TextUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʵIJChartеĸĳʵ,ɷΪ,һʾͼݷֵ(x),һʾ
 * ֵ(y).</zh>
 * <en>The base class for all axes in IJChart.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractAxis implements IAxis, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -4392656039369985447L ;

  /**
   * <zh>ȱʡͼǷʾ.</zh>
   * <en>The default axis visibility.</en>
   */
  public static final boolean DEFAULT_AXIS_VISIBLE = true ;

  /**
   * <zh>ȱʡǩ.</zh>
   * <en>The default axis label font.</en>
   */
  public static final Font DEFAULT_AXIS_LABEL_FONT = new Font("SansSerif", Font.BOLD, 12) ;

  /**
   * <zh>ȱʡǩPaint</zh>
   * <en>The default axis label paint.</en>
   */
  public static final Paint DEFAULT_AXIS_LABEL_PAINT = Color.black ;

  /**
   * <zh>ȱʡհ.</zh>
   * <en>The default axis label insets.</en>
   */
  public static final RectangleInsets DEFAULT_AXIS_LABEL_INSETS = RectangleInsets.INSETS_TWO ;

  /**
   * <zh>ȱʡǷʾı.</zh>
   * <en>A default flag that controls whether or not the axis line is visible. </en>
   */
  public static final boolean DEFAULT_AXIS_LINE_VISIBLE = true ;

  /**
   * <zh>ȱʡPaint.</zh>
   * <en>The default axis line paint.</en>
   */
  public static final Paint DEFAULT_AXIS_LINE_PAINT = Color.gray ;

  /**
   * <zh>ȱʡ3DPaint.</zh>
   * <en>The default paint used for fill 3d axis. </en>
   */
  public static final Paint DEFAULT_3D_AXIS_FILL_PAINT = new Color(221, 221, 221) ;

  /**
   * <zh>ȱʡStroke.</zh>
   * <en>The default axis line stroke.</en>
   */
  public static final Stroke DEFAULT_AXIS_LINE_STROKE = new BasicStroke(1.0f) ;

  /**
   * <zh>ȱʡ̶ֵǷʾ.</zh>
   * <en>The default tick labels visibility. </en>
   */
  public static final boolean DEFAULT_TICK_LABELS_VISIBLE = true ;

  /**
   * <zh>ȱʡ̶ֵ.</zh>
   * <en>The default tick label font. </en>
   */
  public static final Font DEFAULT_TICK_LABEL_FONT = new Font("SansSerif", Font.PLAIN, 10) ;

  /**
   * <zh>ȱʡ̶ֵPaint.</zh>
   * <en>The default tick label paint.</en>
   */
  public static final Paint DEFAULT_TICK_LABEL_PAINT = Color.black ;

  /**
   * <zh>ȱʡ̶ֵհ.</zh>
   * <en>The default tick label insets. </en>
   */
  public static final RectangleInsets DEFAULT_TICK_LABEL_INSETS
    = IChartShape.DEFAULT_MARGIN ;

  /**
   * <zh>ȱʡ̶ֵǷʾ.</zh>
   * <en>The default tick marks visible. </en>
   */
  public static final boolean DEFAULT_TICK_MARKS_VISIBLE = true ;

  /**
   * <zh>ȱʡǩǷʾ.</zh>
   * <en>The default axis label visible. </en>
   */
  public static final boolean DEFAULT_AXIS_LABEL_VISIBLE = true ;

  /**
   * <zh>ȱʡ̶ֵStroke.</zh>
   * <en>The default tick stroke. </en>
   */
  public static final Stroke DEFAULT_TICK_MARK_STROKE = new BasicStroke(1) ;

  /**
   * <zh>ȱʡ̶Paint.</zh>
   * <en>The default tick paint. </en>
   */
  public static final Paint DEFAULT_TICK_MARK_PAINT = Color.gray ;

  /**
   * <zh>ȱʡǷʾ.</zh>
   * <en>A default flag indicate whether or not grid line is visible.</en>
   */
  public static final boolean DEFAULT_GRID_LINE_VISIBLE = true ;

  /**
   * <zh>ȱʡ̶ڲʾĳ.</zh>
   * <en>The default tick mark inside length.</en>
   */
  public static final float DEFAULT_TICK_MARK_INSIDE_LENGTH = 0.0f ;

  /**
   * <zh>ȱʡ̶ⲿʾĳ.</zh>
   * <en>The default tick mark outside length. </en>
   */
  public static final float DEFAULT_TICK_MARK_OUTSIDE_LENGTH = 2.0f ;

  /**
   * <zh>ȱʡ̶ֵˮƽ뷽ʽ.</zh>
   * <en>The default tick label horizontal alignment.</en>
   */
  public static final HorizontalAlignment DEFAULT_TICK_LABEL_HORIZONTAL_ALIGNMENT =
    HorizontalAlignment.LEFT ;

  /**
   * <zh>ȱʡ̶ֱֵ뷽ʽ.</zh>
   * <en>The default tick label vertical alignment.</en>
   */
  public static final VerticalAlignment DEFAULT_TICK_LABEL_VERTICAL_ALIGNMENT =
    VerticalAlignment.CENTER ;

  /**
   * <zh>ȱʡǩˮƽ뷽ʽ.</zh>
   * <en>The default label horizontal alignment.</en>
   */
  public static final HorizontalAlignment DEFAULT_LABEL_HORIZONTAL_ALIGNMENT =
    HorizontalAlignment.CENTER ;

  /**
   * <zh>ȱʡǷԶѡǩĶ뷽ʽ.</zh>
   * <en>The default auto selec label alignment value.</en>
   */
  public static final boolean DEFAULT_AUTO_SELECT_TICK_LABEL_ALIGN = true ;

  /**
   * <zh>˵Ŀհȱʡֵ,λΪٷֱ.</zh>
   * <en>The default margin for the axis (used for both lower and upper margins).</en>
   */
  public static final double DEFAULT_AXIS_MARGIN = 0.0 ;

  /**
   * <zh>̶ȺͿ̶ֵ֮ľ.</zh>
   * <en>The space width between tick mark and tick label.</en>
   */
  public static final double WIDTH_TICK_MARK_AND_TICK_LABEL = 3 ;

  /**
   * <zh>ȱʡǩ.</zh>
   * <en>The default axis label type.</en>
   */
  public static final AxisLabelType DEFAULT_AXIS_LABEL_TYPE = AxisLabelType.ROW ;

  /**
   * <zh>ȱʡĹ̶ƿȻ߶.</zh>
   * <en>The default fixed axis space.</en>
   */
  public static final double DEFAULT_FIXED_AXIS_SPACE = 0 ;

  /**
   * <zh>ǩռͼȻ򳤶ȵٷֱȵȱʡֵ.</zh>
   * <en>The default ratio that is multiplied by the width of one category to determine the
   * maximum label width.</en>
   */
  public static final float DEFAULT_MAXIMUM_CATEGORY_LABEL_WIDTH_RATIO = 0.4F ;

  /**
   * <zh>ȱʡǷƿձǩĿ̶.</zh>
   * <en>A default flag the controls whether or not draw tick mark with empty label.</en>
   */
  public static final boolean DEFAULT_DRAW_EMPTY_LABEL_TICK_MARK = true ;

  /**
   * <zh>Ƿʾı.</zh>
   * <en>A flag indicating whether or not the axis is visible. </en>
   */
  private boolean axisVisible ;

  /**
   * <zh>ıǩ.</zh>
   * <en>The label for the axis. </en>
   */
  private String axisLabel ;

  /**
   * <zh>ıǩ.</zh>
   * <en>The font for displaying the axis label. </en>
   */
  private Font axisLabelFont ;

  /**
   * <zh>ǩPaint.</zh>
   * <en>The paint for drawing the axis label.</en>
   */
  private transient Paint axisLabelPaint ;

  /**
   * <zh>ǩĿհ.</zh>
   * <en>The insets for the axis label.</en>
   */
  private RectangleInsets axisLabelInsets ;

  /**
   * <zh>ǩתĻ.</zh>
   * <en>The label angle (in radians).</en>
   */
  private double axisLabelAngle ;

  /**
   * <zh>ǩǷʾı.</zh>
   * <en>A flag indicating whether or not the axis label is visible. </en>
   */
  private boolean axisLabelVisible ;

  /**
   * <zh>Ƿʾı.</zh>
   * <en>A flag that controls whether or not the axis line is visible. </en>
   */
  private boolean axisLineVisible ;

  /**
   * <zh>ߵStroke.</zh>
   * <en>The stroke used for the axis line.</en>
   */
  private transient Stroke axisLineStroke ;

  /**
   * <zh>ߵPaint.</zh>
   * <en>The paint used for the axis line. </en>
   */
  private transient Paint axisLinePaint ;

  /**
   * <zh>3DPaint.</zh>
   * <en>The paint used for fill 3d axis. </en>
   */
  private transient Paint axis3DFillPaint ;

  /**
   * <zh>ϵĿ̶ȱǩǷʾ.</zh>
   * <en>A flag that indicates whether or not tick labels are visible for the axis.</en>
   */
  private boolean tickLabelsVisible ;

  /**
   * <zh>̶ֵתĻ.</zh>
   * <en>The tick label angle (in radians).</en>
   */
  private double tickLabelAngle ;

  /**
   * <zh>ϵĿ̶ֵǩ</zh>
   * <en>The font used to display the tick labels.</en>
   */
  private Font tickLabelFont ;

  /**
   * <zh>洢̶ֵĶӦϵ,ÿ̶ֵе.</zh>
   * <en> Storage for tick label font overrides (if any).</en>
   */
  private Map tickLabelFontMap ;

  /**
   * <zh>洢̶ֵPaintĶӦϵ,ÿ̶ֵеPaint.</zh>
   * <en>Storage for tick label paint overrides (if any). </en>
   */
  private transient Map tickLabelPaintMap ;

  /**
   * <zh>ϵĿ̶ֵǩPaint.</zh>
   * <en>The color used to display the tick labels. </en>
   */
  private transient Paint tickLabelPaint ;

  /**
   * <zh>ϵĿ̶ֵǩĿհ.</zh>
   * <en>The blank space around each tick label.</en>
   */
  private RectangleInsets tickLabelInsets ;

  /**
   * <zh>ϵĿ̶ֵǷʾ.</zh>
   * <en>A flag that indicates whether or not tick marks are visible for the axis.</en>
   */
  private boolean tickMarkVisible ;

  /**
   * <zh>ϵĿ̶ֵڵĳ.</zh>
   * <en>The length of the tick mark inside the data area (zero permitted). </en>
   */
  private float tickMarkInsideLength ;

  /**
   * <zh>ϵĿ̶ֵĳ.</zh>
   * <en>The length of the tick mark outside the data area (zero permitted).</en>
   */
  private float tickMarkOutsideLength ;

  /**
   * <zh>ϵĿ̶ֵStroke.</zh>
   * <en>The stroke used to draw tick marks.</en>
   */
  private transient Stroke tickMarkStroke ;

  /**
   * <zh>ϵĿ̶ֵPaint.</zh>
   * <en>The paint used to draw tick marks.</en>
   */
  private transient Paint tickMarkPaint ;

  /**
   * <zh>Ƿʾ.</zh>
   * <en>A flag indicate whether or not grid line is visible.</en>
   */
  private boolean gridLineVisible ;

  /**
   * <zh>̶ֵˮƽ뷽ʽ.</zh>
   * <en>The tick label horizontal align.</en>
   */
  private HorizontalAlignment tickLabelHorizontalAlign ;

  /**
   * <zh>̶ֱֵ뷽ʽ.</zh>
   * <en>The tick label vertical align.</en>
   */
  private VerticalAlignment tickLabelVerticalAlign ;

  /**
   * <zh>ǩˮƽ뷽ʽ.</zh>
   * <en>The label horizontal alignment.</en>
   */
  private HorizontalAlignment axisLabelHorizontalAlign ;

  /**
   * <zh>ǷԶѡǩĶ뷽ʽ.</zh>
   * <en>A flag that indicates whether or not auto select label alignment.</en>
   */
  private boolean autoSelectTickLabelAlign ;

  /**
   * <zh>Ὺʼ˵Ŀհ,λΪٷֱ.</zh>
   * <en>The amount of space reserved at the start of the axis.</en>
   */
  private double lowerMargin ;

  /**
   * <zh>˵Ŀհ,λΪٷֱ.</zh>
   * <en>The amount of space reserved at the end of the axis.</en>
   */
  private double upperMargin ;

  /**
   * <zh>ᲿռͼȻ򳤶ȵٷֱ.</zh>
   * <en>A ratio that is multiplied by the width of one category to determine the
   * maximum label width.</en>
   */
  private float maximumCategoryLabelWidthRatio ;

  /**
   * <zh>ڸʽݼеrowKeyֵ.</zh>
   * <en>The formatter for format the rowKeys in dataset.</en>
   */
  private IFormatter rowKeyFormatter ;

  /**
   * <zh>ڸʽݼеcolKeyֵ.</zh>
   * <en>The formatter for format the colKeys in dataset.</en>
   */
  private IFormatter colKeyFormatter ;

  /**
   * <zh>ڸʽݼеֵ.</zh>
   * <en>The formatter for format the data in dataset.</en>
   */
  private IFormatter dataFormatter ;

  /**
   * <zh>¼߶.</zh>
   * <en>Storage for registered listeners.</en>
   */
  private transient EventListenerList listenerList ;

  /**
   * <zh>άݼ.</zh>
   * <en>The dataset.</en>
   */
  private IMatrixDataset dataset ;

  /**
   * <zh>ǩ.</zh>
   * <en>The axis label type.</en>
   */
  private AxisLabelType axisLabelType ;

  /**
   * <zh>̶ƿȻ߶.</zh>
   * <en>The fixed axis space.</en>
   */
  private double fixedAxisSpace ;

  /**
   * <zh>ǷƿձǩĿ̶.</zh>
   * <en>A boolean flag controls whether or not draw tick mark with empty label.</en>
   */
  private boolean drawEmptyLabelTickMark ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  protected AbstractAxis(IMatrixDataset dataset) {
    this(null, dataset) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  protected AbstractAxis(String label, IMatrixDataset dataset) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }

    this.maximumCategoryLabelWidthRatio = DEFAULT_MAXIMUM_CATEGORY_LABEL_WIDTH_RATIO ;
    this.lowerMargin = DEFAULT_AXIS_MARGIN ;
    this.upperMargin = DEFAULT_AXIS_MARGIN ;

    this.axisLabel = label ;
    this.axisVisible = DEFAULT_AXIS_VISIBLE ;

    this.axisLabelVisible = DEFAULT_AXIS_LABEL_VISIBLE ;
    this.axisLabelFont = DEFAULT_AXIS_LABEL_FONT ;
    this.axisLabelPaint = DEFAULT_AXIS_LABEL_PAINT ;
    this.axisLabelInsets = DEFAULT_AXIS_LABEL_INSETS ;
    this.axisLabelAngle = 0.0 ;
    this.axisLabelHorizontalAlign = DEFAULT_LABEL_HORIZONTAL_ALIGNMENT ;

    this.axisLineVisible = DEFAULT_AXIS_LINE_VISIBLE ;
    this.axisLinePaint = DEFAULT_AXIS_LINE_PAINT ;
    this.axis3DFillPaint = DEFAULT_3D_AXIS_FILL_PAINT ;
    this.axisLineStroke = DEFAULT_AXIS_LINE_STROKE ;

    this.tickLabelsVisible = DEFAULT_TICK_LABELS_VISIBLE ;
    this.tickLabelAngle = 0 ;
    this.tickLabelFont = DEFAULT_TICK_LABEL_FONT ;
    this.tickLabelFontMap = new HashMap() ;
    this.tickLabelPaint = DEFAULT_TICK_LABEL_PAINT ;
    this.tickLabelPaintMap = new HashMap() ;
    this.tickLabelInsets = DEFAULT_TICK_LABEL_INSETS ;
    this.tickLabelHorizontalAlign = DEFAULT_TICK_LABEL_HORIZONTAL_ALIGNMENT ;
    this.tickLabelVerticalAlign = DEFAULT_TICK_LABEL_VERTICAL_ALIGNMENT ;
    this.autoSelectTickLabelAlign = DEFAULT_AUTO_SELECT_TICK_LABEL_ALIGN ;

    this.tickMarkVisible = DEFAULT_TICK_MARKS_VISIBLE ;
    this.tickMarkStroke = DEFAULT_TICK_MARK_STROKE ;
    this.tickMarkPaint = DEFAULT_TICK_MARK_PAINT ;
    this.gridLineVisible = DEFAULT_GRID_LINE_VISIBLE ;
    this.tickMarkInsideLength = DEFAULT_TICK_MARK_INSIDE_LENGTH ;
    this.tickMarkOutsideLength = DEFAULT_TICK_MARK_OUTSIDE_LENGTH ;
    this.drawEmptyLabelTickMark = DEFAULT_DRAW_EMPTY_LABEL_TICK_MARK ;

    this.rowKeyFormatter = null ;
    this.colKeyFormatter = null ;
    this.dataFormatter = null ;

    this.dataset = dataset ;

    this.listenerList = new EventListenerList() ;
    this.axisLabelType = DEFAULT_AXIS_LABEL_TYPE ;
    this.fixedAxisSpace = DEFAULT_FIXED_AXIS_SPACE ;
  }

  /**
   * <zh>Ƿɼ,ɼtrue,򷵻false.</zh>
   * <en>Returns true if the axis is visible, and false otherwise.</en>
   *
   * @return A boolean.
   */
  public boolean isVisible() {
    return this.axisVisible ;
  }

  /**
   * <zh>Ƿɼ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets a flag that controls whether or not the axis is visible and sends
   * an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param visible  the flag.
   */
  public void setVisible(boolean visible) {
    if (visible != this.axisVisible) {
      this.axisVisible = visible ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ǩǷɼ,ɼtrue,򷵻false.</zh>
   * <en>Returns true if the axis label is visible,
   * and false otherwise.</en>
   *
   * @return A boolean.
   */
  public boolean isAxisLabelVisible() {
    return this.axisLabelVisible ;
  }

  /**
   * <zh>ǩǷɼ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets a flag that controls whether or not the axis label is visible and sends
   * an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param visible  the flag.
   */
  public void setAxisLabelVisible(boolean visible) {
    if (visible != this.axisLabelVisible) {
      this.axisLabelVisible = visible ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ıǩ.</zh>
   * <en>Returns the label for the axis.</en>
   *
   * @return <zh>ıǩ(Ϊnull).</zh>
   * <en>The label for the axis (null possible).</en>
   */
  public String getAxisLabel() {
    return this.axisLabel ;
  }

  /**
   * <zh>ıǩ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the label for the axis and sends an {@link AxisChangeEvent} to
   * all registered listeners.</en>
   *
   * @param label <zh>µıǩ (Ϊnull).</zh>
   * <en>The new label (null permitted).</en>
   */
  public void setAxisLabel(String label) {
    String existing = this.axisLabel ;
    if (existing != null) {
      if (!existing.equals(label)) {
        this.axisLabel = label ;
        fireAxisChanged() ;
      }
    } else {
      if (label != null) {
        this.axisLabel = label ;
        fireAxisChanged() ;
      }
    }
  }

  /**
   * <zh>رǩ.</zh>
   * <en>Returns the font for the axis label.</en>
   *
   * @return <zh>ǩ.</zh>
   * <en>The font (never null).</en>
   */
  public Font getAxisLabelFont() {

    return axisLabelFont ;
  }

  /**
   * <zh>ǩ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the font for the axis label and sends an {@link AxisChangeEvent}
   * to all registered listeners.</en>
   *
   * @param font <zh>ǩ (Ϊnull).</zh>
   * <en>The font (null not permitted).</en>
   */
  public void setAxisLabelFont(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }
    if (!this.axisLabelFont.equals(font)) {
      this.axisLabelFont = font ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ǩPaint.</zh>
   * <en>Returns the color/shade used to draw the axis label.</en>
   *
   * @return <zh>ǩPaint.</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getAxisLabelPaint() {
    return this.axisLabelPaint ;
  }

  /**
   * <zh>ǩPaint,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the axis label and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param paint <zh>ǩPaint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setAxisLabelPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.axisLabelPaint = paint ;
    fireAxisChanged() ;
  }

  /**
   * <zh>ǩĿհ.</zh>
   * <en>Returns the insets for the label (that is, the amount of blank space
   * that should be left around the label).</en>
   *
   * @return <zh>ǩĿհ (Ϊnull).</zh>
   * <en>The label insets (never null).</en>
   */
  public RectangleInsets getAxisLabelInsets() {
    return this.axisLabelInsets ;
  }

  /**
   * <zh>ñǩĿհ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the insets for the axis label, and sends an {@link AxisChangeEvent}
   * to all registered listeners.</en>
   *
   * @param insets <zh>ǩĿհ (Ϊnull).</zh>
   * <en>The insets (null not permitted).</en>
   */
  public void setAxisLabelInsets(RectangleInsets insets) {
    if (insets == null) {
      throw new IllegalArgumentException("Null 'insets' argument.") ;
    }

    if (!insets.equals(this.axisLabelInsets)) {
      this.axisLabelInsets = insets ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>رǩתǶ,λΪ.</zh>
   * <en>Returns the angle of the axis label.</en>
   *
   * @return <zh>ǩתǶ,λΪ.</zh>
   * <en>The angle (in radians).</en>
   */
  public double getAxisLabelAngle() {
    return this.axisLabelAngle ;
  }

  /**
   * <zh>ñǩתǶ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the angle for the label and sends an {@link AxisChangeEvent} to all
   * registered listeners.</en>
   *
   * @param angle <zh>ǩתǶ,λΪ.</zh>
   * <en>The angle (in radians).</en>
   */
  public void setAxisLabelAngle(double angle) {
    if (this.axisLabelAngle != angle) {
      this.axisLabelAngle = angle ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>Ƿʾ.</zh>
   * <en>A flag that controls whether or not the axis line is drawn.</en>
   *
   * @return A boolean.
   */
  public boolean isAxisLineVisible() {
    return this.axisLineVisible ;
  }

  /**
   * <zh>Ƿʾ.</zh>
   * <en>Sets a flag that controls whether or not the axis line is visible and
   * sends an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param visible  the flag.
   */
  public void setAxisLineVisible(boolean visible) {
    if (this.axisLineVisible != visible) {
      this.axisLineVisible = visible ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ߵPaint.</zh>
   * <en>Returns the paint used to draw the axis line.</en>
   *
   * @return <zh>ߵPaint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getAxisLinePaint() {
    return this.axisLinePaint ;
  }

  /**
   * <zh>ߵPaint,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the axis line and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param paint <zh>ߵPaint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setAxisLinePaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.axisLinePaint = paint ;
    fireAxisChanged() ;
  }

  /**
   * <zh>3DPaint.</zh>
   * <en>Returns the paint used for fill 3d axis. </en>
   *
   * @return Paint <zh>3DPaint.</zh>
   * <en>The paint used for fill 3d axis. </en>
   */
  public Paint getAxis3DFillPaint() {
    return this.axis3DFillPaint ;
  }

  /**
   * <zh>3DPaint,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the paint used for fill 3d axis. </en>
   *
   * @param axis3DFillPaint Paint <zh>3DPaint.</zh>
   * <en>The paint used for fill 3d axis. </en>
   */
  public void setAxis3DFillPaint(Paint axis3DFillPaint) {
    if (this.axis3DFillPaint != axis3DFillPaint) {
      this.axis3DFillPaint = axis3DFillPaint ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ߵStroke.</zh>
   * <en>Returns the stroke used to draw the axis line.</en>
   *
   * @return <zh>ߵStroke.</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getAxisLineStroke() {
    return this.axisLineStroke ;
  }

  /**
   * <zh>ߵPaint,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw the axis line and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param stroke <zh>Stroke.</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setAxisLineStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    this.axisLineStroke = stroke ;
    fireAxisChanged() ;
  }

  /**
   * <zh>̶ֵǷʾ.</zh>
   * <en>Returns a flag indicating whether or not the tick labels are visible.</en>
   *
   * @return The flag.
   */
  public boolean isTickLabelsVisible() {
    return this.tickLabelsVisible ;
  }

  /**
   * <zh>̶ֵǷɼ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the flag that determines whether or not the tick labels are visible and sends
   * an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param flag  the flag.
   */
  public void setTickLabelsVisible(boolean flag) {
    if (flag != this.tickLabelsVisible) {
      this.tickLabelsVisible = flag ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>̶ȵ.</zh>
   * <en>Returns the font used for the tick labels (if showing).</en>
   *
   * @return <zh>̶ȵ (Ϊnull).</zh>
   * <en>The font (never null).</en>
   */
  public Font getTickLabelFont() {
    return this.tickLabelFont ;
  }

  /**
   * <zh>̶ֵ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the font for the tick labels and sends an {@link AxisChangeEvent}
   * to all registered listeners.</en>
   *
   * @param font <zh>̶ֵ (Ϊnull).</zh>
   * <en>The font (null not allowed).</en>
   */
  public void setTickLabelFont(Font font) {
    if (font == null) {
      throw new IllegalArgumentException("Null 'font' argument.") ;
    }

    if (!this.tickLabelFont.equals(font)) {
      this.tickLabelFont = font ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>̶ֵPaint.</zh>
   * <en>Returns the color/shade used for the tick labels.</en>
   *
   * @return <zh>̶ֵPaint (Ϊnull).</zh>
   * <en>The paint used for the tick labels.</en>
   */
  public Paint getTickLabelPaint() {
    return this.tickLabelPaint ;
  }

  /**
   * <zh>̶ֵPaint,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw tick labels (if they are showing) and
   * sends an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param paint <zh>̶ֵPaint.</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setTickLabelPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }
    this.tickLabelPaint = paint ;
    fireAxisChanged() ;
  }

  /**
   * <zh>̶ֵPaint.</zh>
   * <en>Returns the color/shade used for the tick labels.</en>
   *
   * @param tickLabel Object <zh>̶ֵǩ (Ϊnull).</zh>
   * <en>The axis tick label (null permitted).</en>
   * @return <zh>̶ֵPaint.</zh>
   * <en>The paint used for the specified tick label.</en>
   */
  public Paint getTickLabelPaint(Object tickLabel) {
    if (tickLabel == null) {
      return getTickLabelPaint() ;
    }

    Paint paint = (Paint)this.tickLabelPaintMap.get(tickLabel) ;
    if (paint == null) {
      return getTickLabelPaint() ;
    }

    return paint ;
  }

  /**
   * <zh>̶ֵǩPaint,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the paint for the axis tick label and sends
   * an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param tickLabel Object <zh>̶ֵǩ (Ϊnull).</zh>
   * <en>The axis tick label (null not permitted).</en>
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setTickLabelPaint(Object tickLabel, Paint paint) {
    if (tickLabel == null) {
      throw new IllegalArgumentException("Null 'rowKey' argument.") ;
    }

    if (paint == null) {
      this.tickLabelPaintMap.remove(tickLabel) ;
    } else {
      this.tickLabelPaintMap.put(tickLabel, paint) ;
    }
    fireAxisChanged() ;
  }

  /**
   * <zh>̶ֵĿհ.</zh>
   * <en>Returns the insets for the tick labels.</en>
   *
   * @return <zh>̶ֵĿհ (Ϊnull).</zh>
   * <en>The insets (never null).</en>
   */
  public RectangleInsets getTickLabelInsets() {
    return this.tickLabelInsets ;
  }

  /**
   * <zh>̶ֵĿհ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the insets for the tick labels and sends an {@link AxisChangeEvent}
   * to all registered listeners.</en>
   *
   * @param insets <zh>̶ֵĿհ (Ϊnull).</zh>
   * <en>The insets (null not permitted).</en>
   */
  public void setTickLabelInsets(RectangleInsets insets) {
    if (insets == null) {
      throw new IllegalArgumentException("Null 'insets' argument.") ;
    }
    if (!this.tickLabelInsets.equals(insets)) {
      this.tickLabelInsets = insets ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ؿ̶ֵǷɼ.</zh>
   * <en>Returns the flag that indicates whether or not the tick marks are showing.</en>
   *
   * @return <zh>ǷɼĲֵ.</zh>
   * <en>The flag that indicates whether or not the tick marks are showing.</en>
   */
  public boolean isTickMarkVisible() {
    return this.tickMarkVisible ;
  }

  /**
   * <zh>̶ֵǷɼ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the flag that indicates whether or not the tick marks are showing
   * and sends an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param flag  the flag.
   */
  public void setTickMarkVisible(boolean flag) {
    if (flag != this.tickMarkVisible) {
      this.tickMarkVisible = flag ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>̶ֵڲĳ.</zh>
   * <en>Returns the inside length of the tick marks.</en>
   *
   * @return <zh>.</zh>
   * <en>The length.</en>
   */
  public float getTickMarkInsideLength() {
    return this.tickMarkInsideLength ;
  }

  /**
   * <zh>̶ڲĳ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the inside length of the tick marks and sends
   * an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param length <zh>µĳ.</zh>
   * <en>The new length.</en>
   */
  public void setTickMarkInsideLength(float length) {
    if (this.tickMarkInsideLength != length) {
      this.tickMarkInsideLength = length ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>̶ֵⲿĳ.</zh>
   * <en>Returns the outside length of the tick marks.</en>
   *
   * @return <zh>.</zh>
   * <en>The length.</en>
   */
  public float getTickMarkOutsideLength() {
    return this.tickMarkOutsideLength ;
  }

  /**
   * <zh>̶ⲿĳ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the outside length of the tick marks and sends
   * an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param length <zh>µĳ.</zh>
   * <en>The new length.</en>
   */
  public void setTickMarkOutsideLength(float length) {
    if (this.tickMarkOutsideLength != length) {
      this.tickMarkOutsideLength = length ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>̶ȵStroke.</zh>
   * <en>Returns the stroke used to draw tick marks.</en>
   *
   * @return <zh>̶ȵStroke (Ϊnull).</zh>
   * <en>The stroke (never null).</en>
   */
  public Stroke getTickMarkStroke() {
    return this.tickMarkStroke ;
  }

  /**
   * <zh>̶ȵStroke,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw tick marks and sends
   * an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param stroke <zh>̶ȵStroke (Ϊnull).</zh>
   * <en>The stroke (null not permitted).</en>
   */
  public void setTickMarkStroke(Stroke stroke) {
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }

    if (!this.tickMarkStroke.equals(stroke)) {
      this.tickMarkStroke = stroke ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>̶ȵPaint.</zh>
   * <en>Returns the paint used to draw tick marks (if they are showing).</en>
   *
   * @return <zh>̶ȵPaint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getTickMarkPaint() {
    return this.tickMarkPaint ;
  }

  /**
   * <zh>̶ȵPaint,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw tick marks and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param paint <zh>̶ȵPaint</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setTickMarkPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }
    this.tickMarkPaint = paint ;
    fireAxisChanged() ;
  }

  /**
   * <zh>Ƿʾ.</zh>
   * <en>Returns the flag indicate whether or not grid line is visible.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isGridLineVisible() {
    return this.gridLineVisible ;
  }

  /**
   * <zh>Ƿʾ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets a flag indicate whether or not grid line is visible and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setGridLineVisible(boolean flag) {
    if (this.gridLineVisible != flag) {
      this.gridLineVisible = flag ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>һ.</zh>
   * <en>Registers an object for notification of changes to the axis.</en>
   *
   * @param listener <zh>.</zh>
   * <en>The object that is being registered.</en>
   */
  public void addChangeListener(IAxisChangeListener listener) {
    this.listenerList.add(IAxisChangeListener.class, listener) ;
  }

  /**
   * <zh>ݼ.</zh>
   * <en>Return the dataset.</en>
   *
   * @return IMatrixDataset
   */
  public IMatrixDataset getDataset() {
    return this.dataset ;
  }

  /**
   * <zh>ݼ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets dataset for this axis and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ{@link IMatrixDataset} (Ϊnull).</zh>
   * <en>A instance of {@link IMatrixDataset} (null not permitted).</en>
   */
  public void setDataset(IMatrixDataset dataset) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }

    this.dataset = dataset ;
    fireAxisChanged() ;
  }

  /**
   * <zh>ڸʽݼеrowKeyֵ.</zh>
   * <en>Returns the formatter for format the rowKeys in dataset.</en>
   *
   * @return IFormatter
   */
  public IFormatter getRowKeyFormatter() {
    return rowKeyFormatter ;
  }

  /**
   * <zh>ڸʽݼеrowKeyֵ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the formatter for format the rowKeys in dataset and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param rowKeyFormatter IFormatter <zh>{@link IFormatter}.</zh>
   * <en>A {@link IFormatter}.</en>
   */
  public void setRowKeyFormatter(IFormatter rowKeyFormatter) {
    this.rowKeyFormatter = rowKeyFormatter ;
    fireAxisChanged() ;
  }

  /**
   * <zh>ڸʽݼеcolKeyֵ.</zh>
   * <en>Returns the formatter for format the colKeys in dataset.</en>
   *
   * @return IFormatter
   */
  public IFormatter getColKeyFormatter() {
    return colKeyFormatter ;
  }

  /**
   * <zh>ڸʽݼеcolKeyֵ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the formatter for format the colKeys in dataset and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param colKeyFormatter IFormatter <zh>{@link IFormatter}.</zh>
   * <en>A {@link IFormatter}.</en>
   */
  public void setColKeyFormatter(IFormatter colKeyFormatter) {
    this.colKeyFormatter = colKeyFormatter ;
    fireAxisChanged() ;
  }

  /**
   * <zh>ڸʽݼеֵ.</zh>
   * <en>Returns the formatter for format the data in dataset.</en>
   *
   * @return IFormatter
   */
  public IFormatter getDataFormatter() {
    return dataFormatter ;
  }

  /**
   * <zh>ڸʽݼеֵ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the formatter for format the data in dataset and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param dataFormatter IFormatter <zh>{@link IFormatter}.</zh>
   * <en>A {@link IFormatter}.</en>
   */
  public void setDataFormatter(IFormatter dataFormatter) {
    this.dataFormatter = dataFormatter ;
    fireAxisChanged() ;
  }

  /**
   * <zh>̶ֵתĻ.</zh>
   * <en>Returns the tick label angle (in radians).</en>
   *
   * @return double <zh>̶ֵתĻ.</zh>
   * <en>The tick label angle (in radians).</en>
   */
  public double getTickLabelAngle() {
    return this.tickLabelAngle ;
  }

  /**
   * * <zh>̶ֵתĻ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the tick label angle (in radians) and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param tickLabelAngle double <zh>̶ֵתĻ.</zh>
   * <en>The tick label angle (in radians).</en>
   */
  public void setTickLabelAngle(double tickLabelAngle) {
    if (this.tickLabelAngle != tickLabelAngle) {
      this.tickLabelAngle = tickLabelAngle ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ÿ̶ֵˮƽ뷽ʽ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the tick label horizontal alignment and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param horizontalAlign HorizontalAlignment <zh>̶ֵˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The tick label horizontal alignment (null not permitted).</en>
   */
  public void setTickLabelHorizontalAlign(HorizontalAlignment horizontalAlign) {
    if (horizontalAlign == null) {
      throw new IllegalArgumentException("Null 'horizontalAlign' argument.") ;
    }

    if (!this.tickLabelHorizontalAlign.equals(horizontalAlign)) {
      this.tickLabelHorizontalAlign = horizontalAlign ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ؿ̶ֵˮƽ뷽ʽ.</zh>
   * <en>Returns the tick label horizontal alignment.</en>
   *
   * @return HorizontalAlignment <zh>̶ֵˮƽ뷽ʽ.</zh>
   * <en>The tick label horizontal alignment.</en>
   */
  public HorizontalAlignment getTickLabelHorizontalAlign() {
    return this.tickLabelHorizontalAlign ;
  }

  /**
   * <zh>ÿ̶ֱֵ뷽ʽ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the tick label vertical alignment and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param verticalAlign VerticalAlignment <zh>̶ֱֵ뷽ʽ (Ϊnull).</zh>
   * <en>The tick label vertical alignment (null not permitted).</en>
   */
  public void setTickLabelVerticalAlign(VerticalAlignment verticalAlign) {
    if (verticalAlign == null) {
      throw new IllegalArgumentException("Null 'verticalAlign' argument.") ;
    }

    if (!this.tickLabelVerticalAlign.equals(verticalAlign)) {
      this.tickLabelVerticalAlign = verticalAlign ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ؿ̶ֵĴֱ뷽ʽ.</zh>
   * <en>Returns the tick label vertical alignment.</en>
   *
   * @return VerticalAlignment <zh>̶ֱֵ뷽ʽ.</zh>
   * <en>The tick label vertical alignment.</en>
   */
  public VerticalAlignment getTickLabelVerticalAlign() {
    return this.tickLabelVerticalAlign ;
  }

  /**
   * <zh>ñǩˮƽ뷽ʽ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the label horizontal alignment and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param horizontalAlign HorizontalAlignment <zh>ǩˮƽ뷽ʽ (Ϊnull).</zh>
   * <en>The label horizontal alignment (null not permitted).</en>
   */
  public void setAxisLabelHorizontalAlign(HorizontalAlignment horizontalAlign) {
    if (horizontalAlign == null) {
      throw new IllegalArgumentException("Null 'horizontalAlign' argument.") ;
    }

    if (!this.axisLabelHorizontalAlign.equals(horizontalAlign)) {
      this.axisLabelHorizontalAlign = horizontalAlign ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>رǩˮƽ뷽ʽ.</zh>
   * <en>Returns the label horizontal alignment.</en>
   *
   * @return HorizontalAlignment <zh>ǩˮƽ뷽ʽ.</zh>
   * <en>The label horizontal alignment.</en>
   */
  public HorizontalAlignment getAxisLabelHorizontalAlign() {
    return axisLabelHorizontalAlign ;
  }

  /**
   * <zh>ǷԶѡǩĶ뷽ʽ.</zh>
   * <en>Sets whether or not auto select label alignment.</en>
   *
   * @param autoSelectTickLabelAlign boolean <zh>booleanֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setAutoSelectTickLabelAlign(boolean autoSelectTickLabelAlign) {
    if (this.autoSelectTickLabelAlign != autoSelectTickLabelAlign) {
      this.autoSelectTickLabelAlign = autoSelectTickLabelAlign ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ǷԶѡǩĶ뷽ʽ.</zh>
   * <en>Sets whether or not auto select label alignment.</en>
   *
   * @return boolean <zh>booleanֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isAutoSelectTickLabelAlign() {
    return this.autoSelectTickLabelAlign ;
  }

  /**
   * <zh>ǩռͼȻ򳤶ȵٷֱ.</zh>
   * <en>Returns the category label width ratio.</en>
   *
   * @return <zh>ٷֱ.</zh>
   * <en>The ratio.</en>
   */
  public float getMaximumCategoryLabelWidthRatio() {
    return this.maximumCategoryLabelWidthRatio ;
  }

  /**
   * <zh>ǩռͼȻ򳤶ȵٷֱ,
   * м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the maximum category label width ratio and sends an
   * {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param ratio <zh>ٷֱ.</zh>
   * <en>The ratio.</en>
   */
  public void setMaximumCategoryLabelWidthRatio(float ratio) {
    if (this.maximumCategoryLabelWidthRatio != ratio) {
      this.maximumCategoryLabelWidthRatio = ratio ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>̶ȵ.</zh>
   * <en>Returns the font used for the specified tick label (if showing).</en>
   *
   * @param tickLabel <zh>̶ֵǩ (Ϊnull).</zh>
   * <en>The axis tick label (null permitted).</en>
   * @return <zh>̶ȵ (Ϊnull).</zh>
   * <en>The font (never null).</en>
   */
  public Font getTickLabelFont(Object tickLabel) {
    if (tickLabel == null) {
      return getTickLabelFont() ;
    }

    Font font = (Font)this.tickLabelFontMap.get(tickLabel) ;
    if (font == null) {
      return getTickLabelFont() ;
    }

    return font ;
  }

  /**
   * <zh>̶ֵǩ,м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the font for the tick label and sends
   * an {@link AxisChangeEvent} to all registered listeners.</en>
   *
   * @param tickLabel Object <zh>̶ֵǩ (Ϊnull).</zh>
   * <en>The axis tick label (null not permitted).</en>
   * @param font Font <zh> (Ϊnull).</zh>
   * <en>The font (null permitted).</en>
   */
  public void setTickLabelFont(Object tickLabel, Font font) {
    if (tickLabel == null) {
      throw new IllegalArgumentException("Null 'rowKey' argument.") ;
    }

    if (font == null) {
      this.tickLabelFontMap.remove(tickLabel) ;
    } else {
      this.tickLabelFontMap.put(tickLabel, font) ;
    }

    fireAxisChanged() ;
  }

  /**
   * <zh>ǩ.</zh>
   * <en>Returns the axis label type.</en>
   *
   * @return AxisLabelType <zh>ǩ (Ϊnull).</zh>
   * <en>The axis label type (never null).</en>
   */
  public AxisLabelType getAxisLabelType() {
    return this.axisLabelType ;
  }

  /**
   * <zh>ǩ,м{@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets the axis label type and sends an {@link com.ijchart.xychart.event.AxisChangeEvent} to
   * all registered listeners.</en>
   *
   * @param axisLabelType AxisLabelType <zh>ǩ (Ϊnull).</zh>
   * <en>The axis label type (never null).</en>
   */
  public void setAxisLabelType(AxisLabelType axisLabelType) {
    if (axisLabelType == null) {
      throw new IllegalArgumentException("Null 'axisLabelType' argument.") ;
    }

    if (!this.axisLabelType.equals(axisLabelType)) {
      this.axisLabelType = axisLabelType ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ع̶ƿȻ߶.</zh>
   * <en>Returns the fixed axis space.</en>
   *
   * @return double <zh>̶ƿȻ߶ (Ϊnull).</zh>
   * <en>The fixed axis space (maybe null).</en>
   */
  public double getFixedAxisSpace() {
    return this.fixedAxisSpace ;
  }

  /**
   * <zh>ù̶ƿȻ߶,м
   * {@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets the fixed axis space and sends an {@link com.ijchart.xychart.event.AxisChangeEvent} to
   * all registered listeners.</en>
   *
   * @param space double <zh>̶ (Ϊnull).</zh>
   * <en>The fixed axis space (null permitted).</en>
   */
  public void setFixedAxisSpace(double space) {
    if (this.fixedAxisSpace != space) {
      this.fixedAxisSpace = space ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ǷƿձǩĿ̶.</zh>
   * <en>Returns a boolean flag controls whether or not draw tick mark with empty label.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isDrawEmptyLabelTickMark() {
    return this.drawEmptyLabelTickMark ;
  }

  /**
   * <zh>ǷƿձǩĿ̶,м
   * {@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets a boolean flag controls whether or not draw tick mark with empty label and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setDrawEmptyLabelTickMark(boolean flag) {
    if (this.drawEmptyLabelTickMark != flag) {
      this.drawEmptyLabelTickMark = flag ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ҪĿȻ߶.</zh>
   * <en>Calculation the width or height of painting coordinate axis.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param maxAxisArea Rectangle2D <zh>,϶СplotArea.</zh>
   * <en>The maximum area within which the axis should be drawn.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   *
   * @return double <zh>Ȼ߶.</zh>
   * <en>The width or height.</en>
   */
  public double calculateAxisSpace(Graphics2D g2, Rectangle2D plotArea,
                                   Rectangle2D maxAxisArea, RectangleEdge edge) {
    if (!isVisible()) {
      return 0 ;
    }

    double result = 0 ;
    if (isAxisLineVisible()) { //axis line
      float strokeWidth = 1F ;
      Stroke stroke = getAxisLineStroke() ;
      if (stroke != null && stroke instanceof BasicStroke) {
        strokeWidth = ((BasicStroke) stroke).getLineWidth() ;
      }

      result = strokeWidth ;
    }

    if (isTickMarkVisible()) { //tick mark
      result += getTickMarkOutsideLength() ;
      result += AbstractAxis.WIDTH_TICK_MARK_AND_TICK_LABEL ;
    }

    if (isTickLabelsVisible()) { //tick label
      String[] tickLabels = getFormatedTickLabels(g2, maxAxisArea, edge) ;
      if (!StringUtil.isEmpty(tickLabels, true)) {
        //save old graphics state
        Graphics2DState state = new Graphics2DState(g2) ;
        state.saveOldState() ;
        AffineTransform transform = AffineTransform.getRotateInstance(getTickLabelAngle()) ;
        RectangleInsets insets = getTickLabelInsets() ;

        double tickLabelMaxWidth = 0 ;
        for (int i = 0 ; i < tickLabels.length ; i++) {
          g2.setFont(getTickLabelFont(dataset.getRowKey(i))) ;
          Rectangle2D bound = TextUtil.getStringBound(g2, tickLabels[i]) ;
          if (insets != null) {
            insets.expandRect(bound) ;
          }

          Shape shape = transform.createTransformedShape(bound) ;
          bound = shape.getBounds() ;

          if (RectangleEdge.isLeftOrRight(edge)) {
            tickLabelMaxWidth = Math.max(bound.getWidth(), tickLabelMaxWidth) ;
          } else if (RectangleEdge.isTopOrBottom(edge)) {
            tickLabelMaxWidth = Math.max(bound.getHeight(), tickLabelMaxWidth) ;
          }
        }

        result += tickLabelMaxWidth * Math.max(0, getTickLabelLines()) ;
        state.restoreOldState() ;
      }
    }

    if (isAxisLabelVisible() && !StringUtil.isEmpty(getAxisLabel())) { //axis label
      if (RectangleEdge.isLeftOrRight(edge)) {
        //ƱǩҪ
        Rectangle2D labelRect = new Rectangle2D.Double(0, 0, plotArea.getHeight(), 0) ;
        RectangleTextChartShape shape = new RectangleTextChartShape(labelRect, getAxisLabel()) ;
        shape.setMargin(getAxisLabelInsets()) ;
        shape.setTextFont(getAxisLabelFont()) ;
        //תlabel
        Rectangle bound = shape.getMinimumBound(g2) ;
        AffineTransform transform = AffineTransform.getRotateInstance(getAxisLabelAngle()) ;
        double labelWidth = transform.createTransformedShape(bound.toRectangle2D())
                            .getBounds().getWidth() ;
        result += labelWidth ;
      } else if (RectangleEdge.isTopOrBottom(edge)) {
        //ƱǩҪ߶
        Rectangle2D labelRect = new Rectangle2D.Double(0, 0, plotArea.getWidth(), 0) ;
        RectangleTextChartShape shape = new RectangleTextChartShape(labelRect, getAxisLabel()) ;
        shape.setMargin(getAxisLabelInsets()) ;
        shape.setTextFont(getAxisLabelFont()) ;
        //תlabel
        Rectangle bound = shape.getMinimumBound(g2) ;
        AffineTransform transform = AffineTransform.getRotateInstance(getAxisLabelAngle()) ;
        double labelHeight = transform.createTransformedShape(bound.toRectangle2D())
                             .getBounds().getHeight() ;
        result += labelHeight ;
      }
    }

    if (RectangleEdge.isLeftOrRight(edge)) {
      result = Math.min(result, plotArea.getWidth() * getMaximumCategoryLabelWidthRatio()) ;
    } else if (RectangleEdge.isTopOrBottom(edge)) {
      result = Math.min(result, plotArea.getHeight() * getMaximumCategoryLabelWidthRatio()) ;
    }

    return result ;
  }

  /**
   * <zh>̶ֵ.</zh>
   * <en>Returns the line count of tick labels.</en>
   *
   * @return int <zh>.</zh>
   * <en>The line count.</en>
   */
  protected int getTickLabelLines() {
    return 1 ;
  }

  /**
   * <zh>ظʽʾĿ̶ֵ.</zh>
   * <en>Returns the formated tick labels.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param maxAxisArea Rectangle2D <zh>,϶СplotArea (Ϊnull).</zh>
   * <en>The maximum area within which the axis should be drawn (null not permitted).</en>
   * @param edge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   *
   * @return String[] <zh>̶ֵ.</zh>
   * <en>The array of tick labels.</en>
   */
  protected abstract String[] getFormatedTickLabels(Graphics2D g2, Rectangle2D maxAxisArea,
                                                    RectangleEdge edge) ;

  /**
   * <zh>ͼplot(plotArea)XY.ͼеͼδshapeCollection.</zh>
   * <en>Draw the X-axis or Y-axis on plot area(plotArea).All axis shape is stored in
   * shapeCollection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param axisArea Rectangle2D <zh>ʣĿհ.</zh>
   * <en>The empty chart area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param alignEdge boolean <zh>־ʵʻС,ǷԶ߽.</zh>
   * <en>Controls whether or not translate shapes when the draw area less than the axis area.</en>
   *
   * @return double <zh>ʵʻƵĿȻ߶.</zh>
   * <en>Drawing the actual coordinates axis width or height.</en>
   */
  public double draw(Graphics2D g2, Rectangle2D plotArea, Rectangle2D axisArea,
                     RectangleEdge edge, AxisShapeCollection shapeCollection, boolean alignEdge) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (plotArea == null) {
      throw new IllegalArgumentException("Null 'plotArea' argument.") ;
    }
    if (axisArea == null) {
      throw new IllegalArgumentException("Null 'axisArea' argument.") ;
    }
    if (edge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }
    if (shapeCollection == null) {
      throw new IllegalArgumentException("Null 'shapeCollection' argument.") ;
    }

    // if the axis is not visible, don't draw it...
    if (!isVisible()) {
      drawTickMark(g2, axisArea, edge, new AxisShapeCollection(), 0) ; //ǿȼ̶
      if (this instanceof CategoryAxis) {
        drawTickLabel(g2, plotArea, axisArea, edge, shapeCollection, 0) ;
      }
      return 0 ;
    }

    //cursorΪǰλ
    double cursor = drawAxisLine(axisArea, edge, shapeCollection) ;
    cursor = drawTickMark(g2, axisArea, edge, shapeCollection, cursor) ;
    if (isTickMarkVisible()) {
      if (edge.equals(RectangleEdge.TOP)) { //ƫ3,̶֤ȺͿ̶ֵһľ
        cursor -= WIDTH_TICK_MARK_AND_TICK_LABEL ;
      } else if (edge.equals(RectangleEdge.BOTTOM)) {
        cursor += WIDTH_TICK_MARK_AND_TICK_LABEL ;
      } else if (edge.equals(RectangleEdge.LEFT)) {
        cursor -= WIDTH_TICK_MARK_AND_TICK_LABEL ;
      } else if (edge.equals(RectangleEdge.RIGHT)) {
        cursor += WIDTH_TICK_MARK_AND_TICK_LABEL ;
      }
    }
    cursor = drawTickLabel(g2, plotArea, axisArea, edge, shapeCollection, cursor) ;
    cursor = dawAxisLabel(g2, axisArea, edge, shapeCollection, cursor) ;

    double axisHeight = 0 ;
    double offsetX = 0, offsetY = 0 ; //ƫֵ
    if (edge.equals(RectangleEdge.TOP)) {
      offsetX = 0 ;
      offsetY = -(cursor - axisArea.getMinY()) ;
      axisHeight = axisArea.getMaxY() - cursor ;
    } else if (edge.equals(RectangleEdge.BOTTOM)) {
      offsetX = 0 ;
      offsetY = axisArea.getMaxY() - cursor ;
      axisHeight = cursor - axisArea.getMinY() ;
    } else if (edge.equals(RectangleEdge.LEFT)) {
      offsetX = -(axisArea.getWidth() - (axisArea.getMaxX() - cursor)) ;
      offsetY = 0 ;
      axisHeight = axisArea.getMaxX() - cursor ;
    } else if (edge.equals(RectangleEdge.RIGHT)) {
      offsetX = axisArea.getWidth() - (cursor - axisArea.getMinX()) ;
      offsetY = 0 ;
      axisHeight = cursor - axisArea.getMinX() ;
    }

    if (alignEdge) {
      shapeCollection.translate(offsetX, offsetY) ;
    }

    return axisHeight ;
  }

  /**
   * <zh>ǩ.</zh>
   * <en>Draw the axis label.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param axisArea Rectangle2D <zh>ʣĿհ.</zh>
   * <en>The empty chart area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param cursor <zh>λ.</zh>
   * <en>The cursor location.</en>
   *
   * @return double <zh>ǩλ.</zh>
   * <en>The cursor location after draw axis label.</en>
   */
  protected double dawAxisLabel(Graphics2D g2, Rectangle2D axisArea, RectangleEdge edge,
                                AxisShapeCollection shapeCollection, double cursor) {
    if (isAxisLabelVisible()) {
      if (edge.equals(RectangleEdge.TOP)) {
        Rectangle2D labelRect = calculateLabelBound(
          g2, axisArea.getWidth(), axisArea.getHeight() - (axisArea.getMaxY() - cursor),
          getAxisLabelAngle(), edge, new String[] {getAxisLabel()}, getAxisLabelFont(),
          getAxisLabelInsets()) ;

        Rectangle2D tmpRect = new Rectangle2D.Double(0, 0, labelRect.getWidth(),
                                                     labelRect.getHeight()) ;
        Rectangle2D rotateRect = getTickLabelRotateBound(tmpRect, getAxisLabelAngle()) ;

        double startX = axisArea.getX() ; //ƫ
        if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.LEFT)) {
          if (rotateRect.getMinX() < 0) {
            startX += -rotateRect.getMinX() ;
          }
        } else if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.CENTER)) {
          startX += (axisArea.getWidth() - rotateRect.getWidth()) * 0.5
            + Math.abs(rotateRect.getMinX()) ;
        } else if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.RIGHT)) {
          startX += axisArea.getWidth() - rotateRect.getMaxX() ;
        }

        Rectangle2D axisLabelDrawRect = new Rectangle2D.Double(startX, cursor + labelRect.getY(),
                                                               labelRect.getWidth(),
                                                               labelRect.getHeight()) ;
        cursor -= rotateRect.getHeight() ;
        shapeCollection.addAxisLabelShape(createAxisLabelChartShape(axisLabelDrawRect)) ;
      } else if (edge.equals(RectangleEdge.BOTTOM)) {
        Rectangle2D labelRect = calculateLabelBound(
          g2, axisArea.getWidth(), axisArea.getHeight() - (cursor - axisArea.getMinY()),
          getAxisLabelAngle(), edge, new String[] {getAxisLabel()}, getAxisLabelFont(),
          getAxisLabelInsets()) ;

        Rectangle2D tmpRect = new Rectangle2D.Double(0, 0, labelRect.getWidth(),
                                                     labelRect.getHeight()) ;
        Rectangle2D rotateRect = getTickLabelRotateBound(tmpRect, getAxisLabelAngle()) ;

        double startX = axisArea.getX() ; //ƫ
        if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.LEFT)) {
          if (rotateRect.getMinX() < 0) {
            startX += -rotateRect.getMinX() ;
          }
        } else if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.CENTER)) {
          startX += (axisArea.getWidth() - rotateRect.getWidth()) * 0.5
            + Math.abs(rotateRect.getMinX()) ;
        } else if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.RIGHT)) {
          startX += axisArea.getWidth() - rotateRect.getMaxX() ;
        }

        Rectangle2D axisLabelDrawRect = new Rectangle2D.Double(startX, cursor + labelRect.getY(),
                                                               labelRect.getWidth(),
                                                               labelRect.getHeight()) ;
        cursor += rotateRect.getHeight() ;
        shapeCollection.addAxisLabelShape(createAxisLabelChartShape(axisLabelDrawRect)) ;
      } else if (edge.equals(RectangleEdge.LEFT)) {
        Rectangle2D labelRect = null ;
        if (AngleUtil.isVertical(getAxisLabelAngle())) { //ֱ
          labelRect = calculateLabelBound(
            g2, axisArea.getHeight(), axisArea.getWidth() - (axisArea.getMaxX() - cursor),
            getAxisLabelAngle(), edge, new String[] {getAxisLabel()}, getAxisLabelFont(),
            getAxisLabelInsets()) ;
        } else {
          labelRect = calculateLabelBound(
            g2, axisArea.getWidth(), axisArea.getWidth() - (axisArea.getMaxX() - cursor),
            getAxisLabelAngle(), edge, new String[] {getAxisLabel()}, getAxisLabelFont(),
            getAxisLabelInsets()) ;
        }

        Rectangle2D tmpRect = new Rectangle2D.Double(0, 0, labelRect.getWidth(),
                                                     labelRect.getHeight()) ;
        Rectangle2D rotateRect = getTickLabelRotateBound(tmpRect, getAxisLabelAngle()) ;

        double startY = axisArea.getMaxY() ; //ƫ
        if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.LEFT)) {
          startY -= rotateRect.getMaxY() ;
        } else if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.CENTER)) {
          startY = axisArea.getMinY() + 0.5 * axisArea.getHeight()
                   - (rotateRect.getMinY() + rotateRect.getMaxY()) * 0.5 ;
        } else if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.RIGHT)) {
          startY = axisArea.getMinY() + Math.abs(rotateRect.getMinY()) ;
        }

        Rectangle2D axisLabelDrawRect = new Rectangle2D.Double(cursor - rotateRect.getMaxX(),
                                                               startY, labelRect.getWidth(),
                                                               labelRect.getHeight()) ;
        cursor -= rotateRect.getWidth() ;
        shapeCollection.addAxisLabelShape(createAxisLabelChartShape(axisLabelDrawRect)) ;
      } else if (edge.equals(RectangleEdge.RIGHT)) {
        Rectangle2D labelRect = null ;
        if (AngleUtil.isVertical(getAxisLabelAngle())) { //ֱ
          labelRect = calculateLabelBound(
            g2, axisArea.getHeight(), axisArea.getWidth() - (cursor - axisArea.getX()),
            getAxisLabelAngle(), edge, new String[] {getAxisLabel()}, getAxisLabelFont(),
            getAxisLabelInsets()) ;
        } else {
          labelRect = calculateLabelBound(
            g2, axisArea.getWidth(), axisArea.getWidth() - (cursor - axisArea.getX()),
            getAxisLabelAngle(), edge, new String[] {getAxisLabel()}, getAxisLabelFont(),
            getAxisLabelInsets()) ;
        }

        Rectangle2D tmpRect = new Rectangle2D.Double(0, 0, labelRect.getWidth(),
                                                     labelRect.getHeight()) ;
        Rectangle2D rotateRect = getTickLabelRotateBound(tmpRect, getAxisLabelAngle()) ;

        double startY = axisArea.getMaxY() ; //ƫ
        if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.LEFT)) {
          startY -= rotateRect.getMaxY() ;
        } else if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.CENTER)) {
          startY = axisArea.getMinY() + 0.5 * axisArea.getHeight()
                   - (rotateRect.getMinY() + rotateRect.getMaxY()) * 0.5 ;
        } else if (getAxisLabelHorizontalAlign().equals(HorizontalAlignment.RIGHT)) {
          startY = axisArea.getMinY() + Math.abs(rotateRect.getMinY()) ;
        }

        Rectangle2D axisLabelDrawRect = new Rectangle2D.Double(cursor + Math.abs(rotateRect.getMinX()),
                                                               startY, labelRect.getWidth(),
                                                               labelRect.getHeight()) ;
        cursor += rotateRect.getWidth() ;
        shapeCollection.addAxisLabelShape(createAxisLabelChartShape(axisLabelDrawRect)) ;
      }
    }

    return cursor ;
  }

  /**
   * <zh>ǩľ.</zh>
   * <en>Calculate the bound of draw a label.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param labelMaxWidth double <zh>ÿǩռ.</zh>
   * <en>The max width of label.</en>
   * @param labelMaxHeight double <zh>ÿǩռ߶.</zh>
   * <en>The max height of label.</en>
   * @param labelAngle <zh>ǩƫתǶ.</zh>
   * <en>The rotate angle of label.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param seriesLabel String[] <zh>һϵеıǩֵ(̶ֵ),Сǩ߽.</zh>
   * <en>A series labels (for example: axis tick labels).</en>
   * @param labelFont Font <zh>Ʊǩ.</zh>
   * <en>The font of draw labels.</en>
   * @param inset <zh>ǩĿհ.</zh>
   * <en>The insets for the label</en>
   *
   * @return Rectangle2D <zh>̶ֵľα߽.</zh>
   * <en>The bound.</en>
   */
  protected Rectangle2D calculateLabelBound(Graphics2D g2, double labelMaxWidth,
                                            double labelMaxHeight, double labelAngle,
                                            RectangleEdge edge, String[] seriesLabel,
                                            Font labelFont, RectangleInsets inset) {
    double angle = AngleUtil.conversionAngle(labelAngle) ;

    double x = 0, y = 0, width = 0, height = 0 ;
    if (edge.equals(RectangleEdge.TOP)) { //
      if (AngleUtil.isHorizontal(angle)) { //0 or Math.PI
        width = labelMaxWidth ;
        height = labelMaxHeight ;

        //С
        Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
        x = (width - maxRect.getWidth()) * 0.5 ;
        width = maxRect.getWidth() ;
        height = maxRect.getHeight() ;
        y = -height ;

        if (isAutoSelectTickLabelAlign()) {
          setTickLabelHorizontalAlign(HorizontalAlignment.CENTER) ;
          setTickLabelVerticalAlign(VerticalAlignment.BOTTOM) ;
        }
      } else if (angle > 0) { //4
        double sinAngle = Math.sin(angle) ;
        double cosAngle = Math.cos(angle) ;

        if (angle == IAngle.ANGLE_DOWN_90) {
          width = labelMaxHeight ;
          height = labelMaxWidth ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          x = labelMaxWidth - (labelMaxWidth - maxRect.getHeight()) * 0.5 ;
          y = -maxRect.getWidth() ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.RIGHT) ;
            setTickLabelVerticalAlign(VerticalAlignment.CENTER) ;
          }
        } else {
          height = Math.min(labelMaxWidth, labelMaxHeight) ;
          if (height == labelMaxHeight) { //С߶
            height /= 2 ;
          }
          width = (labelMaxHeight - height * cosAngle) / sinAngle ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          x = 0.5 * labelMaxWidth - (width * cosAngle - height * sinAngle)
              - height * sinAngle * 0.5 ;
          y = -(width * sinAngle + height * cosAngle) ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.RIGHT) ;
            setTickLabelVerticalAlign(VerticalAlignment.CENTER) ;
          }
        }
      } else { //1
        angle = Math.abs(angle) ;
        double sinAngle = Math.sin(angle) ;
        double cosAngle = Math.cos(angle) ;

        if (angle == IAngle.ANGLE_DOWN_90) {
          width = labelMaxHeight ;
          height = labelMaxWidth ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          x = (labelMaxWidth - maxRect.getHeight()) * 0.5 ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.LEFT) ;
            setTickLabelVerticalAlign(VerticalAlignment.CENTER) ;
          }
        } else {
          height = Math.min(labelMaxWidth, labelMaxHeight) ;
          if (height == labelMaxHeight) { //С߶
            height /= 2 ;
          }
          width = (labelMaxHeight - height * cosAngle) / sinAngle ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;

          x = labelMaxWidth * 0.5 - height * sinAngle ;
          y = -height * cosAngle ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.LEFT) ;
            setTickLabelVerticalAlign(VerticalAlignment.BOTTOM) ;
          }
        }
      }
    } else if (edge.equals(RectangleEdge.BOTTOM)) { //ײ
      if (AngleUtil.isHorizontal(angle)) { //0 or Math.PI
        width = labelMaxWidth ;
        height = labelMaxHeight ;

        //С
        Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
        x = (width - maxRect.getWidth()) * 0.5 ;
        width = maxRect.getWidth() ;
        height = maxRect.getHeight() ;

        if (isAutoSelectTickLabelAlign()) {
          setTickLabelHorizontalAlign(HorizontalAlignment.CENTER) ;
          setTickLabelVerticalAlign(VerticalAlignment.TOP) ;
        }
      } else if (angle > 0) { //4
        double sinAngle = Math.sin(angle) ;
        double cosAngle = Math.cos(angle) ;

        if (angle == IAngle.ANGLE_DOWN_90) {
          width = labelMaxHeight ;
          height = labelMaxWidth ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          x = width - (width - maxRect.getHeight()) * 0.5 ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.LEFT) ;
            setTickLabelVerticalAlign(VerticalAlignment.CENTER) ;
          }
        } else {
          height = Math.min(labelMaxWidth, labelMaxHeight) ;
          if (height == labelMaxHeight) { //С߶
            height /= 2 ;
          }
          width = (labelMaxHeight - height * cosAngle) / sinAngle ;
          x = labelMaxWidth * 0.5 + 0.5 * height * sinAngle - height * sinAngle * 0.5 ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          x = labelMaxWidth * 0.5 + 0.5 * height * sinAngle - height * sinAngle * 0.5 ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.LEFT) ;
            setTickLabelVerticalAlign(VerticalAlignment.TOP) ;
          }
        }
      } else { //1
        angle = Math.abs(angle) ;
        double sinAngle = Math.sin(angle) ;
        double cosAngle = Math.cos(angle) ;

        if (angle == Math.PI * 0.5) {
          width = labelMaxHeight ;
          height = labelMaxWidth ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          x = (height - maxRect.getHeight()) * 0.5 ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          y = width ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.RIGHT) ;
            setTickLabelVerticalAlign(VerticalAlignment.CENTER) ;
          }
        } else {
          height = Math.min(labelMaxWidth, labelMaxHeight) ;
          if (height == labelMaxHeight) { //С߶
            height /= 2 ;
          }
          width = (labelMaxHeight - height * cosAngle) / sinAngle ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          y = width * sinAngle ;
          x = labelMaxWidth * 0.5 - width * cosAngle ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.RIGHT) ;
            setTickLabelVerticalAlign(VerticalAlignment.TOP) ;
          }
        }
      }
    } else if (edge.equals(RectangleEdge.LEFT)) { //
      if (AngleUtil.isHorizontal(angle)) { //0 or Math.PI
        width = labelMaxHeight ;
        height = labelMaxWidth ;
        x = -width ;

        //С
        Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
        y = (height - maxRect.getHeight()) * 0.5 ;
        width = maxRect.getWidth() ;
        height = maxRect.getHeight() ;
        x = -width ;

        if (isAutoSelectTickLabelAlign()) {
          setTickLabelHorizontalAlign(HorizontalAlignment.RIGHT) ;
          setTickLabelVerticalAlign(VerticalAlignment.CENTER) ;
        }
      } else if (angle > 0) { //4
        double sinAngle = Math.sin(angle) ;
        double cosAngle = Math.cos(angle) ;

        if (angle == IAngle.ANGLE_DOWN_90) {
          width = labelMaxWidth ;
          height = labelMaxHeight ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          y = (width - maxRect.getWidth()) * 0.5 ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.CENTER) ;
            setTickLabelVerticalAlign(VerticalAlignment.TOP) ;
          }
        } else {
          height = Math.min(labelMaxWidth, labelMaxHeight) ;
          if (height == labelMaxHeight) { //С߶
            height /= 2 ;
          }
          width = (labelMaxHeight - height * sinAngle) / cosAngle ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          x = -width * cosAngle ;
          y = 0.5 * labelMaxWidth - width * sinAngle ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.RIGHT) ;
            setTickLabelVerticalAlign(VerticalAlignment.CENTER) ;
          }
        }
      } else { //1
        angle = Math.abs(angle) ;
        double sinAngle = Math.sin(angle) ;
        double cosAngle = Math.cos(angle) ;

        if (angle == Math.PI * 0.5) {
          width = labelMaxWidth ;
          height = labelMaxHeight ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          x = -height ;
          y = labelMaxWidth - (labelMaxWidth - width) * 0.5 ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.CENTER) ;
            setTickLabelVerticalAlign(VerticalAlignment.BOTTOM) ;
          }
        } else {
          height = Math.min(labelMaxWidth, labelMaxHeight) ;
          if (height == labelMaxHeight) { //С߶
            height /= 2 ;
          }
          width = (labelMaxHeight - height * sinAngle) / cosAngle ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          x = -(width * cosAngle + height * sinAngle) ;
          y = (labelMaxHeight * 0.5 - height > 0) ? (labelMaxHeight * 0.5 - height) : 0 * cosAngle
              + width * sinAngle ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.RIGHT) ;
            setTickLabelVerticalAlign(VerticalAlignment.BOTTOM) ;
          }
        }
      }
    } else if (edge.equals(RectangleEdge.RIGHT)) { //Ҳ
      if (AngleUtil.isHorizontal(angle)) { //0 or Math.PI
        width = labelMaxHeight ;
        height = labelMaxWidth ;

        //С
        Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
        y = (height - maxRect.getHeight()) * 0.5 ;
        width = maxRect.getWidth() ;
        height = maxRect.getHeight() ;

        if (isAutoSelectTickLabelAlign()) {
          setTickLabelHorizontalAlign(HorizontalAlignment.LEFT) ;
          setTickLabelVerticalAlign(VerticalAlignment.CENTER) ;
        }
      } else if (angle > 0) { //4
        double sinAngle = Math.sin(angle) ;
        double cosAngle = Math.cos(angle) ;

        if (angle == IAngle.ANGLE_DOWN_90) {
          width = labelMaxWidth ;
          height = labelMaxHeight ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          y = (width - maxRect.getWidth()) * 0.5 ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          x = height ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.CENTER) ;
            setTickLabelVerticalAlign(VerticalAlignment.BOTTOM) ;
          }
        } else {
          height = Math.min(labelMaxWidth, labelMaxHeight) ;
          if (height == labelMaxHeight) { //С߶
            height /= 2 ;
          }
          width = (labelMaxHeight - height * sinAngle) / cosAngle ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          x = height * sinAngle ;
          y = 0.5 * labelMaxWidth - height * cosAngle ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.LEFT) ;
            setTickLabelVerticalAlign(VerticalAlignment.BOTTOM) ;
          }
        }
      } else { //1
        angle = Math.abs(angle) ;
        double sinAngle = Math.sin(angle) ;
        double cosAngle = Math.cos(angle) ;

        if (angle == Math.PI * 0.5) {
          width = labelMaxWidth ;
          height = labelMaxHeight ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          y = labelMaxWidth - (labelMaxWidth - width) * 0.5 ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.CENTER) ;
            setTickLabelVerticalAlign(VerticalAlignment.TOP) ;
          }
        } else {
          height = Math.min(labelMaxWidth, labelMaxHeight) ;
          if (height == labelMaxHeight) { //С߶
            height /= 2 ;
          }
          width = (labelMaxHeight - height * sinAngle) / cosAngle ;

          //С
          Rectangle maxRect = getMaximumBound(g2, width, height, seriesLabel, labelFont, inset) ;
          width = maxRect.getWidth() ;
          height = maxRect.getHeight() ;
          y = height * cosAngle * 0.5 ;

          if (isAutoSelectTickLabelAlign()) {
            setTickLabelHorizontalAlign(HorizontalAlignment.LEFT) ;
            setTickLabelVerticalAlign(VerticalAlignment.TOP) ;
          }
        }
      }

    }

    Rectangle2D bound = new Rectangle2D.Double(x, y, width, height) ;

    return bound ;
  }

  /**
   * <zh>̶ֵ߽תı߽.</zh>
   * <en>Returns the rotate bound of tick label bound.</en>
   *
   * @param rect Rectangle2D <zh>̶ֵ߽.</zh>
   * <en>The label bound.</en>
   * @param angle double <zh>ǩתĽǶ.</zh>
   * <en>The label rotate angle.</en>
   *
   * @return Rectangle2D <zh>תı߽.</zh>
   * <en>The rotate bound.</en>
   */
  protected Rectangle2D getTickLabelRotateBound(Rectangle2D rect, double angle) {
    double conversionAngle = AngleUtil.conversionAngle(angle) ;
    AffineTransform transform = AffineTransform.getRotateInstance(conversionAngle) ;

    return transform.createTransformedShape(rect).getBounds2D() ;
  }

  /**
   * <zh>һϵеıǩֵ߽(ɱǩȺ߶).</zh>
   * <en>Calculate the maximum bound of a series labels.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param maxWidth double <zh>Ʊǩ.</zh>
   * <en>The maximum width of draw this label.</en>
   * @param maxHeight double <zh>Ʊǩ߶.</zh>
   * <en>The maximum height of draw this label.</en>
   * @param labels String[] <zh>(Ϊnull).</zh>
   * <en>(null not permitted).</en>
   * @param font Font <zh>Ʊǩõ.</zh>
   * <en>The font of draw labels.</en>
   * @param inset <zh>ǩĿհ.</zh>
   * <en>The insets for the label</en>
   *
   * @return Rectangle <zh>ַ.</zh>
   * <en>The maximum bound of draw this labels.</en>
   */
  private static Rectangle getMaximumBound(Graphics2D g2, double maxWidth, double maxHeight,
                                           String[] labels, Font font, RectangleInsets inset) {
    if (labels == null) {
      throw new IllegalArgumentException("Null 'labels' argument.") ;
    }

    double minWidth = 0 ;
    double minHeight = 0 ;
    Rectangle2D rect = new Rectangle2D.Double(0, 0, maxWidth, maxHeight) ;
    for (int i = 0 ; i < labels.length ; i++) {
      RectangleTextChartShape shape = new RectangleTextChartShape(rect, labels[i], font) ;
      shape.setMargin(inset) ;
      Rectangle minRect = shape.getMinimumBound(g2) ;

      minWidth = Math.max(minWidth, minRect.getWidth()) ;
      minHeight = Math.max(minHeight, minRect.getHeight()) ;
    }

    minWidth = Math.min(minWidth, maxWidth) ;
    minHeight = Math.min(minHeight, maxHeight) ;

    return new Rectangle(minWidth, minHeight) ;
  }

  /**
   * <zh>ϵĿ̶.</zh>
   * <en>Draws tick marks in a specified area and edge.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param axisArea Rectangle2D <zh>ʣĿհ.</zh>
   * <en>The empty chart area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param cursor <zh>λ.</zh>
   * <en>The cursor location.</en>
   *
   * @return double <zh>̶ȹλ.</zh>
   * <en>The cursor location after draw tick mark.</en>
   */
  protected abstract double drawTickMark(Graphics2D g2, Rectangle2D axisArea, RectangleEdge edge,
                                         AxisShapeCollection shapeCollection, double cursor) ;

  /**
   * <zh>ϵĿ̶ֵıǩ.</zh>
   * <en>Draws tick mark labels in a specified area and edge.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param axisArea Rectangle2D <zh>ʣĿհ.</zh>
   * <en>The empty chart area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param cursor <zh>λ.</zh>
   * <en>The cursor location.</en>
   *
   * @return double <zh>̶ֵλ.</zh>
   * <en>The cursor location after draw tick label.</en>
   */
  protected abstract double drawTickLabel(Graphics2D g2, Rectangle2D plotArea,
                                          Rectangle2D axisArea, RectangleEdge edge,
                                          AxisShapeCollection shapeCollection,
                                          double cursor) ;

  /**
   * <zh></zh>
   * <en>Draws an axis line in a specified area and edge.</en>
   *
   * @param axisArea Rectangle2D <zh>ʣĿհ (Ϊnull).</zh>
   * <en>The empty chart area (null not permitted).</en>
   * @param edge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ (Ϊnull).</zh>
   * <en>Collects information about the axis (null not permitted).</en>
   *
   * @return double <zh>ǰλ.</zh>
   * <en>The cursor location.</en>
   */
  protected double drawAxisLine(Rectangle2D axisArea, RectangleEdge edge,
                                AxisShapeCollection shapeCollection) {
    if (axisArea == null) {
      throw new IllegalArgumentException("Null 'axisArea' argument.") ;
    }
    if (edge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }
    if (shapeCollection == null) {
      throw new IllegalArgumentException("Null 'shapeCollection' argument.") ;
    }

    double cursor = 0 ;
    LineChartShape axisLine = null ;
    float strokeWidth = 1F ;
    if (this.axisLineStroke != null && this.axisLineStroke instanceof BasicStroke) {
      BasicStroke stroke = (BasicStroke)this.axisLineStroke ;
      strokeWidth = stroke.getLineWidth() ;
    }

    if (edge.equals(RectangleEdge.TOP)) {
      cursor = axisArea.getMaxY() ;
      if (this.axisLineVisible) {
        axisLine = new LineChartShape(axisArea.getX(), axisArea.getMaxY(),
                                      axisArea.getMaxX(), axisArea.getMaxY()) ;
        cursor -= strokeWidth ;
      }
    } else if (edge.equals(RectangleEdge.BOTTOM)) {
      cursor = axisArea.getY() ;
      if (this.axisLineVisible) {
        axisLine = new LineChartShape(axisArea.getX(), axisArea.getY(),
                                      axisArea.getMaxX(), axisArea.getY()) ;
        cursor += strokeWidth ;
      }
    } else if (edge.equals(RectangleEdge.LEFT)) {
      cursor = axisArea.getMaxX() ;
      if (this.axisLineVisible) {
        axisLine = new LineChartShape(axisArea.getMaxX(), axisArea.getY(),
                                      axisArea.getMaxX(), axisArea.getMaxY()) ;
        cursor -= strokeWidth ;
      }
    } else if (edge.equals(RectangleEdge.RIGHT)) {
      cursor = axisArea.getX() ;
      if (this.axisLineVisible) {
        axisLine = new LineChartShape(axisArea.getX(), axisArea.getY(),
                                      axisArea.getX(), axisArea.getMaxY()) ;
        cursor += strokeWidth ;
      }
    }

    if (axisLine != null) {
      axisLine.setBorderPaint(this.axisLinePaint) ;
      axisLine.setBorderStroke(this.axisLineStroke) ;
      shapeCollection.addAxisLineShape(axisLine) ;
    }

    return cursor ;
  }

  /**
   * <zh></zh>
   * <en>Draws an axis line in a specified area and edge.</en>
   *
   * @param effect3D IEffect3D <zh>3DЧƫ.</zh>
   * <en>The 3D effect.</en>
   * @param axisArea Rectangle2D <zh>ʣĿհ (Ϊnull).</zh>
   * <en>The empty chart area (null not permitted).</en>
   * @param edge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ (Ϊnull).</zh>
   * <en>Collects information about the axis (null not permitted).</en>
   *
   * @return double <zh>ǰλ.</zh>
   * <en>The cursor location.</en>
   */
  protected double drawAxis3DLine(IEffect3D effect3D, Rectangle2D axisArea, RectangleEdge edge,
                                  AxisShapeCollection shapeCollection) {
    if (effect3D == null) {
      throw new IllegalArgumentException("Null 'effect3D' argument.") ;
    }
    if (axisArea == null) {
      throw new IllegalArgumentException("Null 'axisArea' argument.") ;
    }
    if (edge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }
    if (shapeCollection == null) {
      throw new IllegalArgumentException("Null 'shapeCollection' argument.") ;
    }

    double cursor = 0 ;
    double offsetX = effect3D.getXOffset() ;
    double offsetY = effect3D.getYOffset() ;

    IChartShape axisLine = null ;
    float strokeWidth = 1F ;
    if (this.axisLineStroke != null && this.axisLineStroke instanceof BasicStroke) {
      BasicStroke stroke = (BasicStroke)this.axisLineStroke ;
      strokeWidth = stroke.getLineWidth() ;
    }

    if (edge.equals(RectangleEdge.TOP)) {
      return this.drawAxisLine(axisArea, edge, shapeCollection) ;
    } else if (edge.equals(RectangleEdge.BOTTOM)) {
      cursor = axisArea.getY() ;
      if (isAxisLineVisible()) {
        GeneralPath path = new GeneralPath() ;
        path.moveTo((float) axisArea.getX(), (float) axisArea.getY()) ;
        path.lineTo((float) (axisArea.getX() + offsetX), (float) (axisArea.getY() - offsetY)) ;
        path.lineTo((float) (axisArea.getMaxX() + offsetX), (float) (axisArea.getY() - offsetY)) ;
        path.lineTo((float) axisArea.getMaxX(), (float) axisArea.getY()) ;
        path.closePath() ;

        axisLine = new GeneralPathChartShape(path) ;

        cursor += strokeWidth ;
      }
    } else if (edge.equals(RectangleEdge.LEFT)) {
      cursor = axisArea.getMaxX() ;
      if (isAxisLineVisible()) {
        GeneralPath path = new GeneralPath() ;
        path.moveTo((float) axisArea.getMaxX(), (float) axisArea.getY()) ;
        path.lineTo((float) (axisArea.getMaxX() + offsetX), (float) (axisArea.getY() - offsetY)) ;
        path.lineTo((float) (axisArea.getMaxX() + offsetX), (float) (axisArea.getMaxY() - offsetY)) ;
        path.lineTo((float) axisArea.getMaxX(), (float) axisArea.getMaxY()) ;
        path.closePath() ;

        axisLine = new GeneralPathChartShape(path) ;

        cursor -= strokeWidth ;
      }
    } else if (edge.equals(RectangleEdge.RIGHT)) {
      return this.drawAxisLine(axisArea, edge, shapeCollection) ;
    }

    if (axisLine != null) {
      Paint linePaint = getAxisLinePaint() ;
      axisLine.setBorderPaint(linePaint) ;
      axisLine.setBorderStroke(getAxisLineStroke()) ;
      axisLine.setDrawBorder(true) ;
      axisLine.setFillShape(true) ;
      axisLine.setFillPaint(this.axis3DFillPaint) ;

      shapeCollection.addAxisLine3DShape(axisLine) ;
    }

    return cursor ;
  }

  /**
   * <zh>һRectangleTextChartShape.</zh>
   * <en>Create a RectangleTextChartShape object.</en>
   *
   * @param axisLabelDrawRect Rectangle2D <zh>ǩİΧ.</zh>
   * <en>The axis label bound.</en>
   *
   * @return IChartShape <zh>ͼζ.</zh>
   * <en>A {@link IChartShape}.</en>
   */
  protected IChartShape createAxisLabelChartShape(Rectangle2D axisLabelDrawRect) {
    RectangleTextChartShape shape = new RectangleTextChartShape(axisLabelDrawRect, getAxisLabel()) ;
    shape.setRotate(AngleUtil.conversionAngle(getAxisLabelAngle())) ;
    shape.setTextFont(getAxisLabelFont()) ;
    shape.setTextPaint(getAxisLabelPaint()) ;
    shape.setMargin(getAxisLabelInsets()) ;
    shape.setDrawBorder(false) ;

    shape.setTextHorizontalAlign(HorizontalAlignment.CENTER) ;
    shape.setTextVerticalAlign(VerticalAlignment.CENTER) ;

    return shape ;
  }

  /**
   * <zh>Ὺʼ˵Ŀհ,λΪٷֱ.</zh>
   * <en>Returns the lower margin for the axis.</en>
   *
   * @return <zh>Ὺʼ˿հ,λΪٷֱ.</zh>
   * <en>The margin (in percent).</en>
   */
  public double getLowerMargin() {
    return this.lowerMargin ;
  }

  /**
   * <zh>Ὺʼ˵Ŀհ(λΪٷֱ),м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the lower margin for the axis and sends an {@link AxisChangeEvent}
   * to all registered listeners.</en>
   *
   * @param margin <zh>Ὺʼ˵Ŀհ(λΪٷֱ),:0.05ʾ5%</zh>
   * <en>The margin as a percentage of the axis length (for example, 0.05 is five percent).</en>
   */
  public void setLowerMargin(double margin) {
    if (this.lowerMargin != margin) {
      this.lowerMargin = margin ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>˵Ŀհ,λΪٷֱ.</zh>
   * <en>Returns the upper margin for the axis.</en>
   *
   * @return <zh>˿հ,λΪٷֱ.</zh>
   * <en>The margin (in percent).</en>
   */
  public double getUpperMargin() {
    return this.upperMargin ;
  }

  /**
   * <zh>˵Ŀհ(λΪٷֱ),м{@link AxisChangeEvent}¼.</zh>
   * <en>Sets the upper margin for the axis and sends an {@link AxisChangeEvent}
   * to all registered listeners.</en>
   *
   * @param margin <zh>˵Ŀհ(λΪٷֱ),:0.05ʾ5%</zh>
   * <en>The margin as a percentage of the axis length (for example, 0.05 is five percent).</en>
   */
  public void setUpperMargin(double margin) {
    if (this.upperMargin != margin) {
      this.upperMargin = margin ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>,IAxis.draw()֮.</zh>
   * <en>Draw the grid line,only call after IAxis.draw().</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param dataArea Rectangle2D <zh>ͼdata() (Ϊnull).</zh>
   * <en>The area within which the data should be drawn (null not permitted).</en>
   * @param axisEdge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   * @param shapeCollection RenderShapeCollection <zh>ͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The grid line stroke.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The grid line paint (null not permitted).</en>
   */
  public abstract void drawGridLine(Graphics2D g2, Rectangle2D dataArea, RectangleEdge axisEdge,
                                    RenderShapeCollection shapeCollection, Stroke stroke,
                                    Paint paint) ;

  /**
   * <zh>ɾһ.</zh>
   * <en>Deregisters an object for notification of changes to the axis.</en>
   *
   * @param listener <zh>Ҫɾļ.</zh>
   * <en>The object to deregister.</en>
   */
  public void removeChangeListener(IAxisChangeListener listener) {
    this.listenerList.remove(IAxisChangeListener.class, listener) ;
  }

  /**
   * <zh>ɾм.</zh>
   * <en>Deregisters all object for notification of changes to the axis.</en>
   */
  public void removeAllChangeListener() {
    Object[] listeners = this.listenerList.getListenerList() ;

    for (int i = listeners.length - 2 ; i >= 0 ; i -= 2) {
      this.listenerList.remove(IAxisChangeListener.class, (IAxisChangeListener) listeners[i + 1]) ;
    }
  }

  /**
   * <zh>жǷмlistener.</zh>
   * <en>Returns true if the specified object is registered with
   * the dataset as a listener.  Most applications won't need to call this
   * method, it exists mainly for use by unit testing code.</en>
   *
   * @param listener  the listener.
   *
   * @return A boolean.
   */
  public boolean hasListener(EventListener listener) {
    List list = Arrays.asList(this.listenerList.getListenerList()) ;
    return list.contains(listener) ;
  }

  /**
   * <zh>ͨмᷢ˱仯.</zh>
   * <en>Notifies all registered listeners that the axis has changed.</en>
   */
  protected void fireAxisChanged() {
    notifyListeners(new AxisChangeEvent(this)) ;
  }

  /**
   * <zh>֪ͨеļ,귢˱仯</zh>
   * <en>Notifies all registered listeners that the axis has changed.
   * The AxisChangeEvent provides information about the change.</en>
   *
   * @param event  information about the change to the axis.
   */
  protected void notifyListeners(AxisChangeEvent event) {
    Object[] listeners = this.listenerList.getListenerList() ;
    for (int i = listeners.length - 2 ; i >= 0 ; i -= 2) {
      if (listeners[i] == IAxisChangeListener.class) {
        ((IAxisChangeListener) listeners[i + 1]).axisChanged(event) ;
      }
    }

  }

  /**
   * <zh>clone.</zh>
   * <en>Returns a clone of the axis.</en>
   *
   * @return A clone.
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    AbstractAxis clone = (AbstractAxis)super.clone() ;
    clone.tickLabelFontMap = new HashMap(this.tickLabelFontMap) ;
    clone.tickLabelPaintMap = new HashMap(this.tickLabelPaintMap) ;
    clone.listenerList = new EventListenerList() ;

    return clone ;
  }

  /**
   * <zh>hashCode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = HashcodeUtil.hashCode(this.axisVisible) ;
    result = 29 * result + (int) (100 * this.maximumCategoryLabelWidthRatio) ;
    result = 29 * result + (int) (29 * this.lowerMargin) ;
    result = 29 * result + (int) (100 * this.upperMargin) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLabel) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLabelFont) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLabelPaint) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLabelInsets) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLabelVisible) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLabelHorizontalAlign) ;
    result = 29 * result + (int) (100 * this.axisLabelAngle) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLineVisible) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLineStroke) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLinePaint) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axis3DFillPaint) ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickLabelsVisible) ;
    result = 29 * result + (int) (100 * this.tickLabelAngle) ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickLabelFont) ;
    result = 29 * result + this.tickLabelFontMap.size() ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickLabelPaint) ;
    result = 29 * result + this.tickLabelPaintMap.size() ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickLabelInsets) ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickLabelHorizontalAlign) ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickLabelVerticalAlign) ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickMarkVisible) ;
    result = 29 * result + (int) (100 * this.tickMarkInsideLength) ;
    result = 29 * result + (int) (100 * this.tickMarkOutsideLength) ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickMarkPaint) ;
    result = 29 * result + HashcodeUtil.hashCode(this.gridLineVisible) ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickMarkStroke) ;
    result = 29 * result + HashcodeUtil.hashCode(this.rowKeyFormatter) ;
    result = 29 * result + HashcodeUtil.hashCode(this.colKeyFormatter) ;
    result = 29 * result + HashcodeUtil.hashCode(this.dataFormatter) ;
    result = 29 * result + HashcodeUtil.hashCode(this.dataset) ;
    result = 29 * result + HashcodeUtil.hashCode(this.axisLabelType) ;
    result = 29 * result + (int)this.fixedAxisSpace ;
    result = 29 * result + HashcodeUtil.hashCode(this.drawEmptyLabelTickMark) ;

    return result ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this axis for equality with another object.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>The object (null permitted).</en>
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof AbstractAxis)) {
      return false ;
    }
    AbstractAxis that = (AbstractAxis) obj ;
    if (this.maximumCategoryLabelWidthRatio != that.maximumCategoryLabelWidthRatio) {
      return false ;
    }
    if (this.axisVisible != that.axisVisible) {
      return false ;
    }
    if (this.lowerMargin != that.lowerMargin) {
      return false ;
    }
    if (this.upperMargin != that.upperMargin) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLabel, that.axisLabel)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLabelFont, that.axisLabelFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLabelPaint, that.axisLabelPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLabelInsets, that.axisLabelInsets)) {
      return false ;
    }
    if (this.axisLabelVisible != that.axisLabelVisible) {
      return false ;
    }
    if (this.axisLabelAngle != that.axisLabelAngle) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLabelHorizontalAlign, that.axisLabelHorizontalAlign)) {
      return false ;
    }
    if (this.axisLineVisible != that.axisLineVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLineStroke, that.axisLineStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLinePaint, that.axisLinePaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axis3DFillPaint, that.axis3DFillPaint)) {
      return false ;
    }
    if (this.tickLabelsVisible != that.tickLabelsVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelFont, that.tickLabelFont)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelFontMap, that.tickLabelFontMap)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelPaint, that.tickLabelPaint)) {
      return false ;
    }
    if (!EqualsUtil.equalsPaintMap(this.tickLabelPaintMap, that.tickLabelPaintMap)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelInsets, that.tickLabelInsets)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelHorizontalAlign, that.tickLabelHorizontalAlign)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabelVerticalAlign, that.tickLabelVerticalAlign)) {
      return false ;
    }
    if (this.tickMarkVisible != that.tickMarkVisible) {
      return false ;
    }
    if (this.tickLabelAngle != that.tickLabelAngle) {
      return false ;
    }
    if (this.tickMarkInsideLength != that.tickMarkInsideLength) {
      return false ;
    }
    if (this.tickMarkOutsideLength != that.tickMarkOutsideLength) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickMarkPaint, that.tickMarkPaint)) {
      return false ;
    }
    if (this.gridLineVisible != that.gridLineVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickMarkStroke, that.tickMarkStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.dataset, that.dataset)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.rowKeyFormatter, that.rowKeyFormatter)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.colKeyFormatter, that.colKeyFormatter)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.dataFormatter, that.dataFormatter)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.axisLabelType, that.axisLabelType)) {
      return false ;
    }
    if (this.fixedAxisSpace != that.fixedAxisSpace) {
      return false ;
    }
    if (this.drawEmptyLabelTickMark != that.drawEmptyLabelTickMark) {
      return false ;
    }

    return true ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;
    SerialUtil.writePaint(stream, this.axisLabelPaint) ;
    SerialUtil.writePaint(stream, this.tickLabelPaint) ;
    SerialUtil.writeStroke(stream, this.axisLineStroke) ;
    SerialUtil.writePaint(stream, this.axisLinePaint) ;
    SerialUtil.writePaint(stream, this.axis3DFillPaint) ;
    SerialUtil.writeStroke(stream, this.tickMarkStroke) ;
    SerialUtil.writePaint(stream, this.tickMarkPaint) ;
    SerialUtil.writePaintMap(stream, this.tickLabelPaintMap) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;
    this.axisLabelPaint = SerialUtil.readPaint(stream) ;
    this.tickLabelPaint = SerialUtil.readPaint(stream) ;
    this.axisLineStroke = SerialUtil.readStroke(stream) ;
    this.axisLinePaint = SerialUtil.readPaint(stream) ;
    this.axis3DFillPaint = SerialUtil.readPaint(stream) ;
    this.tickMarkStroke = SerialUtil.readStroke(stream) ;
    this.tickMarkPaint = SerialUtil.readPaint(stream) ;
    this.listenerList = new EventListenerList() ;
    this.tickLabelPaintMap = SerialUtil.readPaintMap(stream) ;
  }
}
