package com.ijchart.xychart.axis.value ;

import java.io.Serializable ;
import java.text.DateFormat ;
import java.util.Date ;

import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.data.IntervalChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.tickunit.DateTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>Interval(ֵ)͵ֵ.</zh>
 * <en>The interval value axis.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class IntervalValueAxis extends NumberValueAxis implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1625594949243576914L ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public IntervalValueAxis(IMatrixDataset dataset) {
    super(null, dataset) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public IntervalValueAxis(String label, IMatrixDataset dataset) {
    super(label, dataset) ;
  }

  /**
   * <zh>ֱǩֵ.</zh>
   * <en>Calculate the number labels.</en>
   *
   * @param labelCount int <zh>ǩĸ.</zh>
   * <en>The number of labels</en>
   * @return String[]
   */
  protected String[] getNumberLabels(int labelCount) {
    IMatrixDataset dataset = getDataset() ;

    double minValue = 0, maxValue = 0 ; //С,ֵ
    if (getUserDefineDatasetRange() != null) {
      NumberRange range = getUserDefineDatasetRange() ;
      minValue = range.getLowerBound() ;
      maxValue = range.getUpperBound() ;
    } else {
      if (dataset.getRowCount() == 0) { //ݼΪ
        NumberRange range = getEmptyDatasetRange() ;
        minValue = range.getLowerBound() ;
        maxValue = range.getUpperBound() ;
      } else {
        int rows = dataset.getRowCount() ;

        for (int i = 0 ; i < rows ; i++) {
          int currRowCols = dataset.getColumnCount(i) ;
          for (int j = 0 ; j < currRowCols ; j++) {
            Object data = dataset.getData(i, j) ;
            if (data != null) {
              if (!(data instanceof IntervalChartData)) {
                throw new InvalidChartDataException(
                  "Invalid chart data type,musts be 'IntervalChartData'.") ;
              } else {
                IntervalChartData interval = (IntervalChartData) data ;
                if (interval.getStartData() == null || interval.getEndData() == null) {
                  continue ;
                }
                double startValue = interval.getStartValue() ;
                double endValue = interval.getEndValue() ;

                minValue = Math.min(minValue, Math.min(startValue, endValue)) ;
                maxValue = Math.max(maxValue, Math.max(startValue, endValue)) ;
              }
            }
          }
        }
      }

    }

    this.numberTicks = tickUnit.getTickUnits(minValue, maxValue, labelCount, isRangeIncludeZero()) ;
    String[] tickUnits = new String[numberTicks.length] ;
    System.arraycopy(this.numberTicks, 0, tickUnits, 0, this.numberTicks.length) ;

    IFormatter formatter = getTickValueFormatter() ;
    if (formatter != null) {
      for (int i = 0 ; i < tickUnits.length ; i++) {
        tickUnits[i] = formatter.format(this.numberTicks[i], i) ;
      }
    } else if (this.tickUnit instanceof DateTickUnit) {
      DateTickUnit dateUnit = (DateTickUnit)this.tickUnit ;
      dateUnit.setTimeZone(getTimeZone()) ;
      DateTickUnitType unitType = dateUnit.getUnitType() ;
      if (unitType == null) {
        unitType = DateTickUnit.calculateDateTickUnitType(minValue, maxValue, tickUnits.length) ;
      }
      DateFormat dateFormatter = dateUnit.getDateFormatter() ;
      if (dateFormatter == null) {
        dateFormatter = DateTickUnit.getDataFormatter(unitType) ;
      }

      for (int i = 0 ; i < tickUnits.length ; i++) {
        long value = Long.parseLong(this.numberTicks[i], 10) ;
        Date valueDate = new Date(value) ;
        tickUnits[i] = dateFormatter.format(valueDate) ;
      }
    }

    return (tickUnits == null) ? new String[0] : tickUnits ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof IntervalValueAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "IntervalValueAxis" ;
  }
}
