package com.ijchart.xychart.axis.value ;

import java.io.Serializable ;

import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.range.NumberRange ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ٲ͵ͼʹõֵ.</zh>
 * <en>The axis used for waterfall bar chart.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class WaterfallNumberValueAxis extends NumberValueAxis implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 606607862649138578L ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public WaterfallNumberValueAxis(IMatrixDataset dataset) {
    this(null, dataset) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public WaterfallNumberValueAxis(String label, IMatrixDataset dataset) {
    super(label, dataset) ;
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getDatasetRange() {
    if (getUserDefineDatasetRange() != null) {
      return getUserDefineDatasetRange() ;
    }

    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;

    IMatrixDataset dataset = getDataset() ;
    if (dataset != null) {
      int rows = dataset.getRowCount() ;
      for (int i = 0 ; i < rows ; i++) {
        double rowMinValue = Double.NaN ;
        double rowMaxValue = Double.NaN ;
        double totalValue = Double.NaN ;

        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          Object data = dataset.getData(i, j) ;
          if (data != null) {
            if (!(data instanceof NumberChartData)) {
              throw new InvalidChartDataException(
                "Invalid chart data type,musts be 'NumberChartData'.") ;
            } else {
              //ÿгһֵֵӺһֵбȽϴС
              double value = ((NumberChartData) data).getValue() ;
              if (!Double.isNaN(value)) {
                if (j == currRowCols - 1) {
                  if (Double.isNaN(rowMinValue)) {
                    rowMinValue = value ;
                  } else {
                    rowMinValue = Math.min(rowMinValue, value) ;
                  }

                  if (Double.isNaN(rowMaxValue)) {
                    rowMaxValue = value ;
                  } else {
                    rowMaxValue = Math.max(rowMaxValue, value) ;
                  }
                } else {
                  if (Double.isNaN(totalValue)) {
                    totalValue = value ;
                  } else {
                    totalValue += value ;
                  }

                  if (Double.isNaN(rowMinValue)) {
                    rowMinValue = totalValue ;
                  } else {
                    rowMinValue = Math.min(rowMinValue, totalValue) ;
                  }

                  if (Double.isNaN(rowMaxValue)) {
                    rowMaxValue = totalValue ;
                  } else {
                    rowMaxValue = Math.max(rowMaxValue, totalValue) ;
                  }

                }
              }
            }
          }
        }

        maxValue = Math.max(maxValue, rowMaxValue) ;

        if (Double.isNaN(minValue)) {
          minValue = rowMinValue ;
        } else {
          minValue = Math.min(minValue, rowMinValue) ;
        }

        if (Double.isNaN(maxValue)) {
          maxValue = rowMaxValue ;
        } else {
          maxValue = Math.max(maxValue, rowMaxValue) ;
        }

      }
    }

    if (Double.isNaN(minValue)) {
      minValue = 0 ;
    }
    if (Double.isNaN(maxValue)) {
      maxValue = 0 ;
    }

    return new NumberRange(minValue, maxValue) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof WaterfallNumberValueAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "WaterfallNumberValueAxis" ;
  }
}
