package com.ijchart.xychart.data ;

import java.io.Serializable ;

import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>(x,y,deltaX,deltaY)ĸֵͼݶ,ⲿǲ޸ĵ.</zh>
 * <en>The (x,y,deltaX,deltaY) ChartData object,this class is immutable.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class VectorChartData extends XYNumberChartData implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1676082811933304776L ;

  /**
   * <zh>x.</zh>
   * <en>The vector x.</en>
   */
  private Number deltaX ;

  /**
   * <zh>y.</zh>
   * <en>The vector y.</en>
   */
  private Number deltaY ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param x double <zh>xֵ.</zh>
   * <en>The x value.</en>
   * @param y double <zh>yֵ.</zh>
   * <en>The y value.</en>
   * @param deltaX double <zh>x.</zh>
   * <en>The vector x.</en>
   * @param deltaY double <zh>y.</zh>
   * <en>The vector y.</en>
   */
  public VectorChartData(double x, double y, double deltaX, double deltaY) {
    this(new Double(x), new Double(y), new Double(deltaX), new Double(deltaY)) ;
  }

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param x Number <zh>xֵ (Ϊnull).</zh>
   * <en>The x Number object (null not permitted).</en>
   * @param y Number <zh>yֵ (Ϊnull).</zh>
   * <en>The y Number object (null not permitted).</en>
   * @param deltaX Number <zh>x.</zh>
   * <en>The vector x.</en>
   * @param deltaY Number <zh>y.</zh>
   * <en>The vector y.</en>
   */
  public VectorChartData(Number x, Number y, Number deltaX, Number deltaY) {
    super(x, y) ;
    if (deltaX == null) {
      throw new IllegalArgumentException("Null 'deltaX' argument.") ;
    }
    if (deltaY == null) {
      throw new IllegalArgumentException("Null 'deltaY' argument.") ;
    }

    this.deltaX = deltaX ;
    this.deltaY = deltaY ;
  }

  /**
   * <zh>xֶ.</zh>
   * <en>Returns the x vector Number data.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getDeltaXData() {
    return this.deltaX ;
  }

  /**
   * <zh>yֶ.</zh>
   * <en>Returns the y vector Number data.</en>
   *
   * @return Number <zh>Ϊnull.</zh>
   * <en>Never null.</en>
   */
  public Number getDeltaYData() {
    return this.deltaY ;
  }

  /**
   * <zh>õdouble͵xֵ.</zh>
   * <en>Returns the double x vector value.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getDeltaX() {
    return this.deltaX.doubleValue() ;
  }

  /**
   * <zh>õdouble͵yֵ.</zh>
   * <en>Returns the double y vector value.</en>
   *
   * @return double <zh>double͵ֵ.</zh>
   * <en>The double value.</en>
   */
  public double getDeltaY() {
    return this.deltaY.doubleValue() ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (never null).</en>
   */
  public String getLabelText() {
    return "(" + formatNumber(getX()) + "," + formatNumber(getY()) + ","
      + formatNumber(getDeltaX()) + "," + formatNumber(getDeltaY()) + ")" ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return formatNumberForFunction(getX()) + "," + formatNumberForFunction(getY()) + ","
      + formatNumberForFunction(getDeltaX()) + "," + formatNumberForFunction(getDeltaY()) ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "x=" + formatNumber(getX()) + "&y=" + formatNumber(getY()) + "&deltaX="
      + formatNumber(getDeltaX()) + "&deltaY" + formatNumber(getDeltaY()) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof VectorChartData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    VectorChartData that = (VectorChartData) obj ;
    if (EqualsUtil.equals(this.deltaX, that.deltaX)) {
      return false ;
    }
    if (EqualsUtil.equals(this.deltaY, that.deltaY)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 37 * result + HashcodeUtil.hashCode(this.deltaX) ;
    result = 37 * result + HashcodeUtil.hashCode(this.deltaY) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[x=" + getX()
      + ",y=" + getY()
      + ",deltaX=" + getDeltaX()
      + ",deltaY=" + getDeltaY()
      + "]" ;
  }

}
