package com.ijchart.xychart.event ;

import java.io.Serializable ;
import java.util.EventObject ;

import com.ijchart.xychart.client.IJChart ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ͼƽ¼.</zh>
 * <en>An event that contains information about the drawing progress of a chart.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ChartProgressEvent extends EventObject implements Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -3253616779019448339L ;

  /**
   * <zh>¼:ʼͼ.</zh>
   * <en>Indicates calculate shapes started.</en>
   */
  public static final int PROGRESS_CALCULATE_STARTED = 1 ;

  /**
   * <zh>¼:ͼ.</zh>
   * <en>Indicates calculate shapes finished.</en>
   */
  public static final int PROGRESS_CALCULATE_FINISHED = 2 ;

  /**
   * <zh>¼:ʼͼ.</zh>
   * <en>Indicates drawing shapes started.</en>
   */
  public static final int PROGRESS_DRAWING_SHAPES_STARTED = 4 ;

  /**
   * <zh>¼:ʼƱͼ.</zh>
   * <en>Indicates drawing title shapes started.</en>
   */
  public static final int PROGRESS_DRAWING_TITLE_SHAPES_STARTED = 8 ;

  /**
   * <zh>¼:ʼͼͼ.</zh>
   * <en>Indicates drawing legend shapes started.</en>
   */
  public static final int PROGRESS_DRAWING_LEGEND_SHAPES_STARTED = 16 ;

  /**
   * <zh>¼:ʼPlotͼ.</zh>
   * <en>Indicates drawing plot shapes started.</en>
   */
  public static final int PROGRESS_DRAWING_PLOT_SHAPES_STARTED = 32 ;

  /**
   * <zh>¼:ͼ.</zh>
   * <en>Indicates drawing shapes finished.</en>
   */
  public static final int PROGRESS_DRAWING_SHAPES_FINISHED = 64 ;

  /**
   * <zh>¼.</zh>
   * <en>The type of event.</en>
   */
  private int eventType ;

  /**
   * <zh>¼Chart.</zh>
   * <en>The chart that generated the event. </en>
   */
  private IJChart chart ;

  /**
   * <zh>.</zh>
   * <en>Creates a new chart change event.</en>
   *
   * @param chart IJChart <zh>ͼ.</zh>
   * <en>The chart that generated the event.</en>
   * @param source Object <zh>¼Դ.</zh>
   * <en>The source of the event (could be the chart, a title, an axis etc.).</en>
   * @param eventType int <zh>¼.</zh>
   * <en>The type of event.</en>
   */
  public ChartProgressEvent(IJChart chart, Object source, int eventType) {
    super(source) ;

    this.chart = chart ;
    this.eventType = eventType ;
  }

  /**
   * <zh>ز¼chart.</zh>
   * <en>Returns the chart that generated the change event.</en>
   *
   * @return IJChart <zh>¼chart.</zh>
   * <en>The chart that generated the change event.</en>
   */
  public IJChart getChart() {
    return this.chart ;
  }

  /**
   * <zh>ò¼chart.</zh>
   * <en>Sets the chart that generated the change event.</en>
   *
   * @param chart IJChart <zh>¼chart.</zh>
   * <en>The chart that generated the event.</en>
   */
  public void setChart(IJChart chart) {
    this.chart = chart ;
  }

  /**
   * <zh>¼.</zh>
   * <en>Returns the event type.</en>
   *
   * @return int <zh>¼.</zh>
   * <en>The event type.</en>
   */
  public int getType() {
    return this.eventType ;
  }

  /**
   * <zh>¼.</zh>
   * <en>Sets the event type.</en>
   *
   * @param eventType int <zh>¼.</zh>
   * <en>The event type.</en>
   */
  public void setType(int eventType) {
    this.eventType = eventType ;
  }
}
