package com.ijchart.xychart.format ;

import java.io.Serializable ;
import java.text.DateFormat ;
import java.text.SimpleDateFormat ;
import java.util.Date ;
import java.util.Locale ;

import com.ijchart.xychart.util.EqualsUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʽΪַ.</zh>
 * <en>A Formatter provides support for formatting milliseconds.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class MillisecondsToDateFormatter implements IFormatter, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 6497265622117653748L ;

  /**
   * <zh>ʽ,ʽΪ"-- ʱ::"</zh>
   * <en>The date mask of long type.</en>
   */
  public static final String DATE_MASK_LONG = "yyyy-MM-dd HH:mm:ss" ;

  /**
   * <zh>ʽ,ʽΪ"--"</zh>
   * <en>The date mask of short type.</en>
   */
  public static final String DATE_MASK_SHORT = "yyyy-MM-dd" ;

  /**
   * <zh>ػ.</zh>
   * <en>A java.util.Locale instance.</en>
   */
  private Locale locale ;

  /**
   * <zh>ʽڵ.</zh>
   * <en>The mask.</en>.
   */
  private String mask ;

  /**
   * <zh>ʽڵjava.util.SimpleDateFormat.</zh>
   * <en>Instance of java.util.SimpleDateFormat for format date object.</en>
   */
  private SimpleDateFormat format ;

  /**
   * <zh>빹챾.</zh>
   * <en>Create a new DateFormatter by default locale and a date mask.</en>
   *
   * @param mask String <zh>.</zh>
   * <en>A date mask.</en>
   */
  public MillisecondsToDateFormatter(String mask) {
    this(Locale.getDefault(), mask) ;
  }

  /**
   * <zh>빹챾.</zh>
   * <en>Create a new MillisecondsToDateFormatter by a date mask.</en>
   *
   * @param locale Locale <zh>ػ.</zh>
   * <en>A java.util.Locale instance.</en>
   * @param mask String <zh>.</zh>
   * <en>A date mask.</en>
   */
  public MillisecondsToDateFormatter(Locale locale, String mask) {
    if (locale == null) {
      throw new IllegalArgumentException("Null 'locale' argument.") ;
    }
    if (mask == null) {
      throw new IllegalArgumentException("Null 'mask' argument.") ;
    }

    this.locale = locale ;
    this.mask = mask ;
    this.format = new SimpleDateFormat(mask, locale) ;
  }

  /**
   * <zh>ʽһjava.util.DateΪַ.ĳֵһϵж(ϵıǩ
   * ̶ֵ),ʽʱԸindexֵĲͬʽΪַͬ.indexʱûʹ,Ҫʹ,ɼ̳д.</zh>
   * <en>Formatted a java.util.Date object to a string. If a value is composed of a
   * series of objects (such as the coordinates of the axis labels or scale value),
   * the format could be based on index.Parameter index do not have to use,
   * to use,extends this class.</en>
   *
   * @param obj Object <zh>ҪʽĶ (Ϊnull).</zh>
   * <en>The object to be formatted (null permitted).</en>
   * @param index int <zh>һϵֵ˳,0ʼ.˲ûʹ.</zh>
   * <en>The index value of a series objects.
   * The first is at index 0, the next is at 1, and so on.This parameter is not used.</en>
   *
   * @return String <zh>ʽַ.</zh>
   * <en>A formatted date string.</en>
   */
  public String format(Object obj, int index) {
    if (obj == null) {
      return "" ;
    }

    if (obj instanceof Number) {
      return format.format(new Date(((Number) obj).longValue())) ;
    }

    String str = obj.toString() ;
    long mills = 0L ;
    try {
      mills = Long.parseLong(str, 10) ;
    } catch (NumberFormatException ex) {
      return str ;
    }

    return format.format(new Date(mills)) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this object for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵһ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof DateFormat)) {
      return false ;
    }

    MillisecondsToDateFormatter that = (MillisecondsToDateFormatter) obj ;
    if (!EqualsUtil.equals(this.locale, that.locale)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.mask, that.mask)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    return this.locale.hashCode() + 29 * mask.hashCode() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of this Range.</en>
   *
   * @return A String.
   */
  public String toString() {
    return "[locale=" + this.locale + ",mask=" + this.mask + "]" ;
  }

}
