package com.ijchart.xychart.function ;

import java.io.Serializable ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ݺ,ʽΪy=a*x^b.</zh>
 * <en>A function of the form y = a * x ^ b.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PowerFunction implements IFunction, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1757897659958286999L ;

  /**
   * <zh>ϵa.</zh>
   * <en>The 'a' coefficient.</en>
   */
  private double a ;

  /**
   * <zh>ϵb.</zh>
   * <en>The 'b' coefficient.</en>
   */
  private double b ;

  /**
   * <zh>ݺ.</zh>
   * <en>Constructs a new power function.</en>
   *
   * @param a double <zh>ϵa.</zh>
   * <en>The 'a' coefficient.</en>
   * @param b double <zh>ϵb.</zh>
   * <en>The 'b' coefficient.</en>
   */
  public PowerFunction(double a, double b) {
    this.a = a ;
    this.b = b ;
  }

  /**
   * <zh>ϵa.</zh>
   * <en>Returns the 'a' coefficient.</en>
   *
   * @return double <zh>ϵa.</zh>
   * <en>The 'a' coefficient.</en>
   */
  public double getA() {
    return this.a ;
  }

  /**
   * <zh>ϵb.</zh>
   * <en>Returns the 'b' coefficient.</en>
   *
   * @return double <zh>ϵb.</zh>
   * <en>The 'b' coefficient.</en>
   */
  public double getB() {
    return this.b ;
  }

  /**
   * <zh>xֵyֵ.</zh>
   * <en>Calculate the y value for a given inpu x.</en>
   *
   * @param x double <zh>xֵ.</zh>
   * <en>The x value.</en>
   * @return double <zh>yֵ.</zh>
   * <en>The y value.</en>
   */
  public double getY(double x) {
    return this.a * Math.pow(x, this.b) ;
  }

  /**
   * <zh>equals.</zh>
   * <en>Override equals method.</en>
   * @param obj Object
   * @return boolean
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof PowerFunction)) {
      return false ;
    }

    PowerFunction that = (PowerFunction) obj ;

    return this.a == that.a && this.b == that.b ;
  }

  /**
   * <zh>toString().</zh>
   * <en>Override toString method.</en>
   * @return String
   */
  public String toString() {
    return "[a=" + this.a + ",b=" + this.b + "]" ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object. </en>
   * @return  a hash code value for this object.
   */
  public int hashCode() {
    return (int) ((this.a + this.b) * 1000) ;
  }

}
