package com.ijchart.xychart.render.category ;

import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.render.util.Shape3DSortUtil ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>2D͵״ͼ.</zh>
 * <en>Draw 2D bar shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class Bar2DRender extends AbstractBaseValueCategoryRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8041338845692682282L ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public Bar2DRender(IMatrixDataset dataset) {
    super(dataset) ;
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param categoryAxis CategoryAxis <zh>category (Ϊnull).</zh>
   * <en>The category axis (null not permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param valueAxisEdge RectangleEdge <zh>valueλ (Ϊnull).</zh>
   * <en>The value axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(CategoryAxis categoryAxis, AbstractValueAxis valueAxis,
                           RectangleEdge valueAxisEdge, Rectangle2D dataArea,
                           ChartOrientation orientation, RenderShapeCollection renderShapes,
                           float alpha) {
    if (categoryAxis == null) {
      throw new IllegalArgumentException("Null 'categoryAxis' argument.") ;
    }
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (valueAxisEdge == null) {
      throw new IllegalArgumentException("Null 'valueAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }
    int columns = dataset.getColumnCount() ;
    int rows = dataset.getRowCount() ;

    double basePosition = valueAxis.valueToJava2D(dataArea, valueAxisEdge, getBaseValue(), true) ;
    if (basePosition == AbstractValueAxis.INVALID_COORDINATE_VALUE) { //ֵ᷶Χ
      return ;
    }

    this.fillPaints = getFillPaints(columns) ;
    this.borderPaints = getBorderPaints(columns) ;
    this.borderStrokes = getBorderStrokes(columns) ;

    boolean is3D = this instanceof Bar3DRender || this instanceof CylinderRender ;
    Shape3DSortUtil sortUtil = new Shape3DSortUtil() ;

    //״ͼ
    if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
      double gapSize = 0 ; //С
      if (rows > 1) {
        gapSize = dataArea.getWidth() * categoryAxis.getCategoryMargin() / (rows - 1) ;
      }
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin()
                          + ((rows == 1) ? 0 : categoryAxis.getCategoryMargin()) ; //еĿհ
      double categoryWidth = dataArea.getWidth() * (1 - axisMargin) / rows ; //ÿϵеĿ
      double categoryOneMargin = (dataArea.getWidth()
                                  * (1 - categoryAxis.getLowerMargin()
                                     - categoryAxis.getUpperMargin())
                                  * getItemMargin()) / rows ; //ÿcategoryĿհ

      double calculateBarWidth = (categoryWidth - categoryOneMargin) / columns ; //Ŀ
      double reallyBarWidth = Math.min(calculateBarWidth, getMaximumBarWidth()) ; //ʵʿ
      if (reallyBarWidth < getMinimumBarWidth()) { //ĿССƿ
        return ;
      }
      double barItemMargin = categoryOneMargin / columns ; //ÿ֮Ŀ

      for (int i = 0 ; i < rows ; i++) {
        sortUtil.clearShapes() ;
        double startX = dataArea.getX() + dataArea.getWidth() * categoryAxis.getLowerMargin()
                        + i * (categoryWidth + gapSize) ;

        for (int j = 0 ; j < columns ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            double y = valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;
            //ֵ᷶Χֵ
            if (y == AbstractValueAxis.INVALID_COORDINATE_VALUE || y == basePosition) {
              if (is3D && j == columns - 1) {
                add3DShapes(renderShapes, orientation, sortUtil) ;
              }
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            double x11 = startX + 0.5 * barItemMargin + 0.5 * (calculateBarWidth - reallyBarWidth)
                         + (calculateBarWidth + barItemMargin) * j ;
            Rectangle2D rect = new Rectangle2D.Double(x11, Math.min(y, basePosition),
                                                      reallyBarWidth, Math.abs(basePosition - y)) ;
            IChartShape shape = createItemShape(orientation, valueAxis, i, j, rect, fillPaints[j],
                                                borderPaints[j], borderStrokes[j], alpha, null) ;
            if (is3D) {
              sortUtil.addShape(Math.min(y, basePosition), shape) ;

              if (j == columns - 1) {
                add3DShapes(renderShapes, orientation, sortUtil) ;
              }
            } else {
              renderShapes.addDataItemShape(shape) ;
            }
          } else {
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
      double gapSize = 0 ; //С
      if (rows > 1) {
        gapSize = dataArea.getHeight() * categoryAxis.getCategoryMargin() / (rows - 1) ;
      }
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin()
                          + ((rows == 1) ? 0 : categoryAxis.getCategoryMargin()) ; //еĿհ
      double categoryWidth = dataArea.getHeight() * (1 - axisMargin) / rows ; //ÿϵеĿ
      double categoryOneMargin = (dataArea.getHeight()
                                  * (1 - categoryAxis.getLowerMargin()
                                     - categoryAxis.getUpperMargin())
                                  * getItemMargin()) / rows ; //ÿcategoryĿհ

      double calculateBarWidth = (categoryWidth - categoryOneMargin) / columns ; //Ŀ
      double reallyBarWidth = Math.min(calculateBarWidth, getMaximumBarWidth()) ; //ʵʿ
      if (reallyBarWidth < getMinimumBarWidth()) { //ĿССƿ
        return ;
      }
      double barItemMargin = categoryOneMargin / columns ; //ÿ֮Ŀ

      for (int i = 0 ; i < rows ; i++) {
        sortUtil.clearShapes() ;
        double startY = dataArea.getY() + dataArea.getHeight() * categoryAxis.getUpperMargin()
                        + i * (categoryWidth + gapSize) ;

        for (int j = 0 ; j < columns ; j++) {
          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            double x = valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;
            //ֵ᷶Χֵ
            if (x == AbstractValueAxis.INVALID_COORDINATE_VALUE || x == basePosition) {
              if (is3D && j == columns - 1) {
                add3DShapes(renderShapes, orientation, sortUtil) ;
              }
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            double y11 = startY + 0.5 * barItemMargin + 0.5 * (calculateBarWidth - reallyBarWidth)
                         + (calculateBarWidth + barItemMargin) * j ;
            Rectangle2D rect = new Rectangle2D.Double(Math.min(x, basePosition), y11,
                                                      Math.abs(basePosition - x), reallyBarWidth) ;
            IChartShape shape = createItemShape(orientation, valueAxis, i, j, rect, fillPaints[j],
                                                borderPaints[j], borderStrokes[j], alpha, null) ;
            if (is3D) {
              sortUtil.addShape(Math.min(x, basePosition), shape) ;

              if (is3D && j == columns - 1) {
                add3DShapes(renderShapes, orientation, sortUtil) ;
              }
            } else {
              renderShapes.addDataItemShape(shape) ;
            }
          } else {
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    }

  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof Bar2DRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "Bar2DRender" ;
  }

}
