package com.ijchart.xychart.render.category ;

import java.awt.geom.GeneralPath ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.render.IStackedRender ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ƶջʽXYͼ.</zh>
 * <en>Draw xy stacked area shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StackedAreaRender extends AbstractCategoryRender implements IStackedRender, Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 9019533399157015587L ;

  /**
   * <zh>Ƿ԰ٷֱʽʾ.</zh>
   * <en>A flag that controls whether the bars display values or percentages.</en>
   */
  private boolean renderAsPercentages ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public StackedAreaRender(IMatrixDataset dataset) {
    this(dataset, false) ;
  }

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param renderAsPercentages boolean <zh>Ƿ԰ٷֱʽʾ.</zh>
   * <en>A flag that controls whether the data values are rendered as percentages.</en>
   */
  public StackedAreaRender(IMatrixDataset dataset, boolean renderAsPercentages) {
    super(dataset) ;

    this.renderAsPercentages = renderAsPercentages ;
  }

  /**
   * <zh>԰ٷֱʽʾ,true,򷵻false.</zh>
   * <en>Returns true if the render displays each item value as a percentage
   * (so that the stacked bars add to 100%), and false otherwise.</en>
   *
   * @return A boolean. <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isRenderAsPercentages() {
    return this.renderAsPercentages ;
  }

  /**
   * <zh>Ƿ԰ٷֱʽʾ.</zh>
   * <en>Sets the flag that controls whether the render displays each item
   * value as a percentage (so that the stacked bars add to 100%), and sends
   * a {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param asPercentages <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setRenderAsPercentages(boolean asPercentages) {
    if (this.renderAsPercentages != asPercentages) {
      this.renderAsPercentages = asPercentages ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param categoryAxis CategoryAxis <zh>category (Ϊnull).</zh>
   * <en>The category axis (null not permitted).</en>
   * @param valueAxis AbstractValueAxis <zh>value (Ϊnull).</zh>
   * <en>The value axis (null not permitted).</en>
   * @param valueAxisEdge RectangleEdge <zh>valueλ (Ϊnull).</zh>
   * <en>The value axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(CategoryAxis categoryAxis, AbstractValueAxis valueAxis,
                           RectangleEdge valueAxisEdge, Rectangle2D dataArea,
                           ChartOrientation orientation, RenderShapeCollection renderShapes,
                           float alpha) {
    if (categoryAxis == null) {
      throw new IllegalArgumentException("Null 'categoryAxis' argument.") ;
    }
    if (valueAxis == null) {
      throw new IllegalArgumentException("Null 'valueAxis' argument.") ;
    }
    if (valueAxisEdge == null) {
      throw new IllegalArgumentException("Null 'valueAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int rows = dataset.getRowCount() ;
    int columns = dataset.getColumnCount() ;

    double basePosition = valueAxis.valueToJava2D(dataArea, valueAxisEdge, 0, false) ;
    if (basePosition == AbstractValueAxis.INVALID_COORDINATE_VALUE) { //ֵ᷶Χ
      return ;
    }

    double maxRowTotal = getDatasetMaxRowTotal() ;
    this.fillPaints = getFillPaints(rows) ;
    this.borderPaints = getBorderPaints(rows) ;
    this.borderStrokes = getBorderStrokes(rows) ;
    double[] allLowPos = new double[columns * 3] ;
    double[] allHighPos = new double[columns * 3] ;
    for (int i = 0 ; i < allLowPos.length ; i++) {
      allLowPos[i] = basePosition ;
      allHighPos[i] = basePosition ;
    }
    double[] posArr = new double[3] ;

    //״ͼ
    if (orientation.equals(ChartOrientation.VERTICAL)) { //ֱ
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin() ; //еĿհ
      double categoryWidth = dataArea.getWidth() * (1 - axisMargin) / columns ; //ÿϵеĿ
      double halfWidth = categoryWidth * 0.5 ;
      for (int i = 0 ; i < rows ; i++) {
        double startX = dataArea.getX() + dataArea.getWidth() * categoryAxis.getLowerMargin() ;
        int currRowCols = dataset.getColumnCount(i) ;
        arrToNull(posArr) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          double shapeX = startX + j * categoryWidth ;

          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            if (Double.isNaN(value)) {
              arrToNull(posArr) ;
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            value = (renderAsPercentages && maxRowTotal != 0) ? 100 * value / maxRowTotal : value ;
            double currYPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;

            double nextYPos = Double.NaN ;
            if (j < currRowCols - 1) {
              IChartData nextData = dataset.getData(i, j + 1) ;
              if (nextData != null && nextData instanceof NumberChartData) {
                double nextValue = ((NumberChartData) nextData).getValue() ;
                if (!Double.isNaN(nextValue)) {
                  nextValue = (renderAsPercentages && maxRowTotal != 0)
                              ? 100 * nextValue / maxRowTotal : nextValue ;
                  nextYPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, nextValue, false) ;
                }
              }
            }

            //ֵ᷶Χֵ
            if (currYPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || nextYPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              arrToNull(posArr) ;
              continue ;
            }

            boolean isCurrPositive = currYPos <= basePosition ;
            boolean isNextPositive = !Double.isNaN(nextYPos) && nextYPos <= basePosition ;
            double realCurrPos = isCurrPositive ? allLowPos[j * 3 + 1] + currYPos - basePosition
                                 : allHighPos[j * 3 + 1] + currYPos - basePosition ; //the real position of current data
            double realNextPos = Double.isNaN(nextYPos) ? Double.NaN
                                 : (isNextPositive
                                    ? allLowPos[(j + 1) * 3 + 1] + nextYPos - basePosition
                                    : allHighPos[(j + 1) * 3 + 1] + nextYPos - basePosition) ; //the real position of next data

            GeneralPath path = new GeneralPath() ;
            if (j == 0) {
              path.moveTo((float) shapeX, (float) realCurrPos) ;
              path.lineTo((float) (shapeX + halfWidth), (float) realCurrPos) ;
              posArr[0] = posArr[1] = realCurrPos ;

              if (Double.isNaN(nextYPos)) {
                if (j == currRowCols - 1) {
                  posArr[2] = realCurrPos ;
                } else {
                  posArr[2] = isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2] ;
                }
                path.lineTo((float) (shapeX + categoryWidth), (float) posArr[2]) ;
                path.lineTo((float) (shapeX + categoryWidth),
                            (float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2])) ;
              } else {
                posArr[2] = (realCurrPos + realNextPos) * 0.5 ;
                if (isCurrPositive && !isNextPositive) {
                  posArr[2] = allLowPos[j * 3 + 2] ;
                } else if (!isCurrPositive && isNextPositive) {
                  posArr[2] = allHighPos[j * 3 + 2] ;
                }
                path.lineTo((float) (shapeX + categoryWidth), (float) posArr[2]) ;
                path.lineTo((float) (shapeX + categoryWidth),
                            (float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2])) ;
              }
              path.lineTo((float) (shapeX + halfWidth),
                          (float) (isCurrPositive ? allLowPos[j * 3 + 1] : allHighPos[j * 3 + 1])) ;
              path.lineTo((float) shapeX,
                          (float) (isCurrPositive ? allLowPos[j * 3] : allHighPos[j * 3])) ;
              path.closePath() ;
            } else {
              posArr[0] = Double.isNaN(posArr[2]) ? realCurrPos : posArr[2] ;
              if (isCurrPositive) {
                posArr[0] = Math.min(posArr[0], allLowPos[j * 3]) ;
              } else {
                posArr[0] = Math.max(posArr[0], allHighPos[j * 3]) ;
              }
              path.moveTo((float) shapeX, (float) posArr[0]) ;

              posArr[1] = realCurrPos ;
              path.lineTo((float) (shapeX + halfWidth), (float) posArr[1]) ;
              if (Double.isNaN(nextYPos)) {
                if (j == currRowCols - 1) {
                  posArr[2] = realCurrPos ;
                } else {
                  posArr[2] = isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2] ;
                }
                path.lineTo((float) (shapeX + categoryWidth), (float) posArr[2]) ;
                path.lineTo((float) (shapeX + categoryWidth),
                            (float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2])) ;
              } else {
                posArr[2] = (realCurrPos + realNextPos) * 0.5 ;
                if (isCurrPositive && !isNextPositive) {
                  posArr[2] = allLowPos[j * 3 + 2] ;
                } else if (!isCurrPositive && isNextPositive) {
                  posArr[2] = allHighPos[j * 3 + 2] ;
                }

                path.lineTo((float) (shapeX + categoryWidth), (float) posArr[2]) ;
                path.lineTo((float) (shapeX + categoryWidth),
                            (float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2])) ;
              }
              path.lineTo((float) (shapeX + halfWidth),
                          (float) (isCurrPositive ? allLowPos[j * 3 + 1] : allHighPos[j * 3 + 1])) ;
              path.lineTo((float) shapeX,
                          (float) (isCurrPositive ? allLowPos[j * 3] : allHighPos[j * 3])) ;
              path.closePath() ;
            }

            Rectangle2D drawArea = null ;
            if (isCurrPositive) {
              drawArea = new Rectangle2D.Double(shapeX + halfWidth, posArr[1], 1,
                                                Math.abs(currYPos - basePosition)) ;
            } else {
              drawArea = new Rectangle2D.Double(shapeX + halfWidth,
                                                posArr[1] - Math.abs(currYPos - basePosition), 1,
                                                Math.abs(currYPos - basePosition)) ;
            }

            Object[] params = {path, drawArea} ;
            IChartShape shape = createItemShape(orientation, valueAxis, i, j, null, fillPaints[i],
                                                borderPaints[i], borderStrokes[i], alpha, params) ;
            renderShapes.addDataItemShape(shape) ;

            if (isCurrPositive) {
              allLowPos[j * 3] = posArr[0] ;
              allLowPos[j * 3 + 1] = posArr[1] ;
              allLowPos[j * 3 + 2] = posArr[2] ;
            } else {
              allHighPos[j * 3] = posArr[0] ;
              allHighPos[j * 3 + 1] = posArr[1] ;
              allHighPos[j * 3 + 2] = posArr[2] ;
            }
          } else {
            arrToNull(posArr) ;
            renderShapes.addDataItemShape(null) ;
          }
        }
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ˮƽ
      double axisMargin = categoryAxis.getLowerMargin() + categoryAxis.getUpperMargin() ; //еĿհ
      double categoryWidth = dataArea.getHeight() * (1 - axisMargin) / columns ; //ÿϵеĿ
      double halfWidth = categoryWidth * 0.5 ;
      for (int i = 0 ; i < rows ; i++) {
        double startY = dataArea.getY() + dataArea.getHeight() * categoryAxis.getUpperMargin() ;
        int currRowCols = dataset.getColumnCount(i) ;
        arrToNull(posArr) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          double shapeY = startY + j * categoryWidth ;

          IChartData data = dataset.getData(i, j) ;
          if (data != null && data instanceof NumberChartData) {
            double value = ((NumberChartData) data).getValue() ;
            if (Double.isNaN(value)) {
              arrToNull(posArr) ;
              renderShapes.addDataItemShape(null) ;
              continue ;
            }

            value = (renderAsPercentages && maxRowTotal != 0) ? 100 * value / maxRowTotal : value ;
            double currXPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, value, false) ;

            double nextXPos = Double.NaN ;
            if (j < currRowCols - 1) {
              IChartData nextData = dataset.getData(i, j + 1) ;
              if (nextData != null && nextData instanceof NumberChartData) {
                double nextValue = ((NumberChartData) nextData).getValue() ;
                if (!Double.isNaN(nextValue)) {
                  nextValue = (renderAsPercentages && maxRowTotal != 0)
                              ? 100 * nextValue / maxRowTotal : nextValue ;
                  nextXPos = valueAxis.valueToJava2D(dataArea, valueAxisEdge, nextValue, false) ;
                }
              }
            }

            //ֵ᷶Χֵ
            if (currXPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
                || nextXPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
              renderShapes.addDataItemShape(null) ;
              arrToNull(posArr) ;
              continue ;
            }

            boolean isCurrPositive = currXPos <= basePosition ;
            boolean isNextPositive = !Double.isNaN(nextXPos) && nextXPos <= basePosition ;
            double realCurrPos = isCurrPositive ? allLowPos[j * 3 + 1] + currXPos - basePosition
                                 : allHighPos[j * 3 + 1] + currXPos - basePosition ; //the real position of current data
            double realNextPos = Double.isNaN(nextXPos) ? Double.NaN
                                 : (isNextPositive
                                    ? allLowPos[(j + 1) * 3 + 1] + nextXPos - basePosition
                                    : allHighPos[(j + 1) * 3 + 1] + nextXPos - basePosition) ; //the real position of next data

            GeneralPath path = new GeneralPath() ;
            if (j == 0) {
              path.moveTo((float) realCurrPos, (float) shapeY) ;
              path.lineTo((float) realCurrPos, (float) (shapeY + halfWidth)) ;
              posArr[0] = posArr[1] = realCurrPos ;

              if (Double.isNaN(nextXPos)) {
                if (j == currRowCols - 1) {
                  posArr[2] = realCurrPos ;
                } else {
                  posArr[2] = isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2] ;
                }
                path.lineTo((float) posArr[2], (float) (shapeY + categoryWidth)) ;
                path.lineTo((float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2]),
                            (float) (shapeY + categoryWidth)) ;
              } else {
                posArr[2] = (realCurrPos + realNextPos) * 0.5 ;
                if (isCurrPositive && !isNextPositive) {
                  posArr[2] = allLowPos[j * 3 + 2] ;
                } else if (!isCurrPositive && isNextPositive) {
                  posArr[2] = allHighPos[j * 3 + 2] ;
                }
                path.lineTo((float) posArr[2], (float) (shapeY + categoryWidth)) ;
                path.lineTo((float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2]),
                            (float) (shapeY + categoryWidth)) ;
              }
              path.lineTo((float) (isCurrPositive ? allLowPos[j * 3 + 1] : allHighPos[j * 3 + 1]),
                          (float) (shapeY + halfWidth)) ;
              path.lineTo((float) (isCurrPositive ? allLowPos[j * 3] : allHighPos[j * 3]),
                          (float) shapeY) ;
              path.closePath() ;
            } else {
              posArr[0] = Double.isNaN(posArr[2]) ? realCurrPos : posArr[2] ;
              if (isCurrPositive) {
                posArr[0] = Math.min(posArr[0], allLowPos[j * 3]) ;
              } else {
                posArr[0] = Math.max(posArr[0], allHighPos[j * 3]) ;
              }
              path.moveTo((float) posArr[0], (float) shapeY) ;

              posArr[1] = realCurrPos ;
              path.lineTo((float) posArr[1], (float) (shapeY + halfWidth)) ;
              if (Double.isNaN(nextXPos)) {
                if (j == currRowCols - 1) {
                  posArr[2] = realCurrPos ;
                } else {
                  posArr[2] = isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2] ;
                }
                path.lineTo((float) posArr[2], (float) (shapeY + categoryWidth)) ;
                path.lineTo((float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2]),
                            (float) (shapeY + categoryWidth)) ;
              } else {
                posArr[2] = (realCurrPos + realNextPos) * 0.5 ;
                if (isCurrPositive && !isNextPositive) {
                  posArr[2] = allLowPos[j * 3 + 2] ;
                } else if (!isCurrPositive && isNextPositive) {
                  posArr[2] = allHighPos[j * 3 + 2] ;
                }

                path.lineTo((float) posArr[2], (float) (shapeY + categoryWidth)) ;
                path.lineTo((float) (isCurrPositive ? allLowPos[j * 3 + 2] : allHighPos[j * 3 + 2]),
                            (float) (shapeY + categoryWidth)) ;
              }
              path.lineTo((float) (isCurrPositive ? allLowPos[j * 3 + 1] : allHighPos[j * 3 + 1]),
                          (float) (shapeY + halfWidth)) ;
              path.lineTo((float) (isCurrPositive ? allLowPos[j * 3] : allHighPos[j * 3]),
                          (float) shapeY) ;
              path.closePath() ;
            }

            Rectangle2D drawArea = null ;
            if (isCurrPositive) {
              drawArea = new Rectangle2D.Double(posArr[1], shapeY + halfWidth,
                                                Math.abs(currXPos - basePosition), 1) ;
            } else {
              drawArea = new Rectangle2D.Double(posArr[1] - Math.abs(currXPos - basePosition),
                                                shapeY + halfWidth,
                                                Math.abs(currXPos - basePosition), 1) ;
            }

            Object[] params = {path, drawArea} ;
            IChartShape shape = createItemShape(orientation, valueAxis, i, j, null, fillPaints[i],
                                                borderPaints[i], borderStrokes[i], alpha, params) ;
            renderShapes.addDataItemShape(shape) ;

            if (isCurrPositive) {
              allLowPos[j * 3] = posArr[0] ;
              allLowPos[j * 3 + 1] = posArr[1] ;
              allLowPos[j * 3 + 2] = posArr[2] ;
            } else {
              allHighPos[j * 3] = posArr[0] ;
              allHighPos[j * 3 + 1] = posArr[1] ;
              allHighPos[j * 3 + 2] = posArr[2] ;
            }
          } else {
            arrToNull(posArr) ;
            renderShapes.addDataItemShape(null) ;
          }
        }
      }

    }

  }

  private static void arrToNull(double[] arr) {
    for (int i = 0 ; i < arr.length ; i++) {
      arr[i] = Double.NaN ;
    }
  }

  /**
   * <zh>ݼһ֮͵ֵ.</zh>
   * <en>Calculate the max total value of a dataset row.</en>
   *
   * @return double
   */
  private double getDatasetMaxRowTotal() {
    double maxColTotal = 0 ;

    IMatrixDataset dataset = getDataset() ;
    int rows = dataset.getRowCount() ;
    int cols = dataset.getColumnCount() ;

    if (dataset != null && this.renderAsPercentages) {
      for (int i = 0 ; i < cols ; i++) {
        double totalCol = 0 ;
        for (int j = 0 ; j < rows ; j++) {
          Object data = dataset.getData(j, i) ;
          if (data != null) {
            if (!(data instanceof NumberChartData)) {
              throw new InvalidChartDataException(
                "Invalid chart data type,musts be 'NumberChartData'.") ;
            } else { //Сֵиֵ֮,ֵֵ֮
              NumberChartData numberData = (NumberChartData) data ;
              totalCol += Math.abs(numberData.getValue()) ;
            }
          }
        }

        maxColTotal = Math.max(maxColTotal, totalCol) ;
      }
    }

    return maxColTotal ;
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    Object[] params = (Object[]) appendParam ;
    GeneralPath path = (GeneralPath) params[0] ;
    Rectangle2D drawArea = (Rectangle2D) params[1] ;

    GeneralPathChartShape shape = new GeneralPathChartShape(path, drawArea, null) ;
    shape.setText(text) ;
    shape.setForceInsideTextNoWrap(true) ;

    return shape ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof StackedAreaRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    StackedAreaRender that = (StackedAreaRender) obj ;
    if (this.renderAsPercentages != that.renderAsPercentages) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() * 37 + HashcodeUtil.hashCode(this.renderAsPercentages) ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[renderAsPercentages=" + this.renderAsPercentages + "]" ;
  }

}
