package com.ijchart.xychart.render.xy ;

import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.IXYChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>XYͼͼ.</zh>
 * <en>A render that draws dot shape for each data item
 *  (for use with the {@link com.ijchart.xychart.plot.XYPlot} class).</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYDotRender extends AbstractXYRender implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 7954551675645972588L ;

  /**
   * <zh>ȱʡĵĿ.</zh>
   * <en>The default dot width.</en>
   */
  public static final double DEFAULT_DOT_WIDTH = 2 ;

  /**
   * <zh>ȱʡĵĸ߶.</zh>
   * <en>The default dot height.</en>
   */
  public static final double DEFAULT_DOT_HEIGHT = 2 ;

  /**
   * <zh>Ŀ.</zh>
   * <en>The dot width.</en>
   */
  private double dotWidth ;

  /**
   * <zh>ĸ߶.</zh>
   * <en>The dot height.</en>
   */
  private double dotHeight ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public XYDotRender(IMatrixDataset dataset) {
    this(dataset, DEFAULT_DOT_WIDTH, DEFAULT_DOT_HEIGHT) ;
  }

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param dotWidth <zh>Ŀ.</zh>
   * <en>The dot width.</en>
   * @param dotHeight <zh>ĸ߶.</zh>
   * <en>The dot height.</en>
   */
  public XYDotRender(IMatrixDataset dataset, double dotWidth, double dotHeight) {
    super(dataset) ;

    this.dotWidth = dotWidth ;
    this.dotHeight = dotHeight ;
  }

  /**
   * <zh>صĿ.</zh>
   * <en>Returns the dot width.</en>
   *
   * @return double <zh>.</zh>
   * <en>The width.</en>
   */
  public double getDotWidth() {
    return this.dotWidth ;
  }

  /**
   * <zh>õĿ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the dot width and sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to
   *  all registered listeners.</en>
   *
   * @param width double <zh>.</zh>
   * <en>The width.</en>
   */
  public void setDotWidth(double width) {
    if (this.dotWidth != width) {
      this.dotWidth = width ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>صĸ߶.</zh>
   * <en>Returns the dot height.</en>
   *
   * @return double <zh>߶.</zh>
   * <en>The height.</en>
   */
  public double getDotHeight() {
    return this.dotHeight ;
  }

  /**
   * <zh>õĸ߶,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the dot height and sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to
   *  all registered listeners.</en>
   *
   * @param height double <zh>߶.</zh>
   * <en>The height.</en>
   */
  public void setDotHeight(double height) {
    if (this.dotHeight != height) {
      this.dotHeight = height ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }

    int rows = dataset.getRowCount() ;
    this.fillPaints = getFillPaints(rows) ;
    getBorderPaints(rows) ;
    this.borderPaints = fillPaints ;
    this.borderStrokes = getBorderStrokes(rows) ;

    //ߺͼ
    for (int i = 0 ; i < rows ; i++) {
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData data = dataset.getData(i, j) ;
        if (data != null && data instanceof IXYChartData) {
          double xValue = ((IXYChartData) data).getX() ;
          double x = xAxis.valueToJava2D(dataArea, xAxisEdge, xValue, false) ;
          double yValue = ((IXYChartData) data).getY() ;
          double y = yAxis.valueToJava2D(dataArea, yAxisEdge, yValue, false) ;
          //ֵ᷶Χֵ
          if (x == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || y == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            renderShapes.addDataItemShape(null) ;
            continue ;
          }

          double halfWidth = this.dotWidth * 0.5 ;
          double halfHeight = this.dotHeight * 0.5 ;
          Rectangle2D rect = null ;
          if (orientation.equals(ChartOrientation.VERTICAL)) {
            rect = new Rectangle2D.Double(x - halfWidth, y - halfHeight,
                                          this.dotWidth, this.dotHeight) ;
          } else {
            rect = new Rectangle2D.Double(y - halfHeight, x - halfWidth,
                                          this.dotHeight, this.dotWidth) ;
          }

          RectangleTextChartShape rectShape = new RectangleTextChartShape(rect, null) ;
          IChartShape chartShape = createItemShape(orientation, yAxis, i, j, null,
                                                   fillPaints[i], borderPaints[i],
                                                   borderStrokes[i], alpha, rectShape) ;
          renderShapes.addDataItemShape(chartShape) ;
        } else {
          renderShapes.addDataItemShape(null) ;
        }
      }
    }

  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof XYDotRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    XYDotRender that = (XYDotRender) obj ;
    if (this.dotWidth != that.dotWidth) {
      return false ;
    }
    if (this.dotHeight != that.dotHeight) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = result * 37 + this.dotWidth ;
    result = result * 37 + this.dotHeight ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "XYDotRender" ;
  }

}
