package com.ijchart.xychart.shape ;

import java.awt.Color ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StrokeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class GranttChartShape extends RectangleTextChartShape implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 7657341253377361689L ;

  /**
   * <zh>ͼ.</zh>
   * <en>The grantt area.</en>
   */
  private transient Rectangle2D granttArea ;

  /**
   * <zh>.</zh>
   * <en>The complete area.</en>
   */
  private transient Rectangle2D completeArea ;

  /**
   * <zh>δ.</zh>
   * <en>The incomplete area.</en>
   */
  private transient Rectangle2D incompleteArea ;

  /**
   * <zh>Paint.</zh>
   * <en>The complete area paint.</en>
   */
  private transient Paint completeAreaPaint ;

  /**
   * <zh>δPaint.</zh>
   * <en>The incomplete area paint.</en>
   */
  private transient Paint incompleteAreaPaint ;

  /**
   * <zh>ͼͼ.</zh>
   * <en>Construct a grantt chart shape.</en>
   *
   * @param granttArea Rectangle2D <zh>ͼ (Ϊnull).</zh>
   * <en>The grantt area (null not permitted).</en>
   * @param completeArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The complete area (null permitted).</en>
   * @param incompleteArea Rectangle2D <zh>δ (Ϊnull).</zh>
   * <en>The incomplete area (null permitted).</en>
   */
  public GranttChartShape(Rectangle2D granttArea, Rectangle2D completeArea,
                          Rectangle2D incompleteArea) {
    this(granttArea, completeArea, incompleteArea, null) ;
  }

  /**
   * <zh>ͼͼ.</zh>
   * <en>Construct a grantt chart shape.</en>
   *
   * @param granttArea Rectangle2D <zh>ͼ (Ϊnull).</zh>
   * <en>The grantt area (null not permitted).</en>
   * @param completeArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The complete area (null permitted).</en>
   * @param incompleteArea Rectangle2D <zh>δ (Ϊnull).</zh>
   * <en>The incomplete area (null permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The draw text (null permitted).</en>
   */
  public GranttChartShape(Rectangle2D granttArea, Rectangle2D completeArea,
                          Rectangle2D incompleteArea, String text) {
    super(granttArea, text) ;

    this.granttArea = granttArea ;
    this.completeArea = completeArea ;
    this.incompleteArea = incompleteArea ;

    this.completeAreaPaint = null ;
    this.incompleteAreaPaint = null ;
  }

  /**
   * <zh>ظͼ.</zh>
   * <en>Returns the grantt area.</en>
   *
   * @return Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The rectangle (never null).</en>
   */
  public Rectangle2D getGanttArea() {
    return this.granttArea ;
  }

  /**
   * <zh>øͼ,֪ͨ仯.</zh>
   * <en>The grantt area,and sends a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all
   * registered listeners.</en>
   *
   * @param granttArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The rectangle (null not permitted).</en>
   */
  public void setGranttArea(Rectangle2D granttArea) {
    if (granttArea == null) {
      throw new IllegalArgumentException("Null 'granttArea' argument.") ;
    }

    this.granttArea = granttArea ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>.</zh>
   * <en>Returns the complete area.</en>
   *
   * @return Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The rectangle (maybe null).</en>
   */
  public Rectangle2D getCompleteArea() {
    return this.completeArea ;
  }

  /**
   * <zh>,֪ͨ仯.</zh>
   * <en>Sets the complete area,and sends a {@link com.ijchart.xychart.event.ChartShapeChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param completeArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The rectangle (null permitted).</en>
   */
  public void setCompleteArea(Rectangle2D completeArea) {
    if (!(this.completeArea == null && completeArea == null)) {
      this.completeArea = completeArea ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>δ.</zh>
   * <en>Returns the incomplete area.</en>
   *
   * @return Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The rectangle (maybe null).</en>
   */
  public Rectangle2D getIncompleteArea() {
    return this.incompleteArea ;
  }

  /**
   * <zh>δ,֪ͨ仯.</zh>
   * <en>Sets the incomplete area,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param incompleteArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The rectangle (null permitted).</en>
   */
  public void setIncompleteArea(Rectangle2D incompleteArea) {
    if (!(this.incompleteArea == null && incompleteArea == null)) {
      this.incompleteArea = incompleteArea ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>Paint.</zh>
   * <en>Returns the complete area paint.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getCompleteAreaPaint() {
    return this.completeAreaPaint ;
  }

  /**
   * <zh>Paint,֪ͨ仯.</zh>
   * <en>Sets the complete area paint,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setCompleteAreaPaint(Paint paint) {
    if (!(this.completeAreaPaint == null && paint == null)) {
      this.completeAreaPaint = paint ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>δPaint.</zh>
   * <en>Returns the incomplete area paint.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getIncompleteAreaPaint() {
    return this.incompleteAreaPaint ;
  }

  /**
   * <zh>δPaint,֪ͨ仯.</zh>
   * <en>Sets the incomplete area paint,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setIncompleteAreaPaint(Paint paint) {
    if (!(this.incompleteAreaPaint == null && paint == null)) {
      this.incompleteAreaPaint = paint ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    //grantt area
    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    Rectangle2D drawArea = getDrawArea() ;
    Rectangle2D cloneArea = new Rectangle2D.Double(0, 0, drawArea.getWidth(), drawArea.getHeight()) ;
    g2.translate(drawArea.getX(), drawArea.getY()) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;
//    g2.setClip(cloneArea) ;

    float strokeWidth = 0 ;
    if (isDrawBorder()) {
      strokeWidth = StrokeUtil.getStrokeWidth(getBorderStroke()) ;
    }

    drawBackground(g2, cloneArea) ;
    drawBorder(g2, new Rectangle2D.Double(strokeWidth / 2, strokeWidth / 2,
                                          drawArea.getWidth() - strokeWidth,
                                          drawArea.getHeight() - strokeWidth)) ;
    g2.rotate( -getRotate()) ;

    state.restoreTransform() ;
    drawArea(g2, this.completeArea, this.completeAreaPaint) ; //
    state.restoreTransform() ;
    drawArea(g2, this.incompleteArea, this.incompleteAreaPaint) ; //δ

    //
    state.restoreTransform() ;
    g2.translate(drawArea.getX(), drawArea.getY()) ;
    g2.rotate(getRotate()) ;

    drawTextWithTrimMarginRect(g2, cloneArea) ;

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;

    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>area.</zh>
   * <en>Draw a rect.</en>
   * @param g2 Graphics2D
   * @param area Rectangle2D
   * @param paint Paint
   */
  private void drawArea(Graphics2D g2, Rectangle2D area, Paint paint) {
    if (area != null) {
      Paint drawPaint = (paint == null) ? Color.WHITE : paint ;
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;

      Rectangle2D drawArea = getDrawArea() ;
      Rectangle2D completeCloneArea = new Rectangle2D.Double(area.getX() - drawArea.getX(),
                                                             area.getY() - drawArea.getY(),
                                                             area.getWidth(), area.getHeight()) ;
      g2.translate(drawArea.getX(), drawArea.getY()) ;
      g2.rotate(getRotate()) ;

      g2.setPaint(drawPaint) ;
      g2.fill(completeCloneArea) ;

      g2.rotate( -getRotate()) ;
      state.restoreOldState() ;
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    Rectangle2D drawArea = getDrawArea() ;
    AffineTransform transform = AffineTransform.getTranslateInstance(drawArea.getX(), drawArea.getY()) ;
    transform.rotate(getRotate()) ;

    Rectangle2D rect = new Rectangle2D.Double(0, 0, drawArea.getWidth(), drawArea.getHeight()) ;
    return transform.createTransformedShape(rect) ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_GRANTT ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writeShape(stream, this.granttArea) ;
    SerialUtil.writeShape(stream, this.completeArea) ;
    SerialUtil.writeShape(stream, this.incompleteArea) ;

    SerialUtil.writePaint(stream, this.completeAreaPaint) ;
    SerialUtil.writePaint(stream, this.incompleteAreaPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.granttArea = (Rectangle2D) SerialUtil.readShape(stream) ;
    this.completeArea = (Rectangle2D) SerialUtil.readShape(stream) ;
    this.incompleteArea = (Rectangle2D) SerialUtil.readShape(stream) ;

    this.completeAreaPaint = SerialUtil.readPaint(stream) ;
    this.incompleteAreaPaint = SerialUtil.readPaint(stream) ;
  }

}
