package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.Graphics2D ;
import java.awt.Image ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.AnchorUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StrokeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ھڲʾһͼƬ.</zh>
 * <en>Draw an image in a rectangle.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class RectangleImageChartShape extends AbstractImageChartShape implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3707797035200370907L ;

  /**
   * <zh>ȱʡǷͼƬ.</zh>
   * <en>A default flag controls whether or not to fill the area by image.</en>
   */
  public static final boolean DEFAULT_IMAGE_FILL_AREA = false ;

  /**
   * <zh>ͼƬľ.</zh>
   * <en>The image draw area.</en>
   */
  private transient Rectangle2D drawArea ;

  /**
   * <zh>ǷԶͼƬ.</zh>
   * <en>A boolean flag controls whether or not fill area by image.</en>
   */
  private boolean imageFill ;

  /**
   * <zh>drawAreaʾͼƬimage.</zh>
   * <en>Draw image in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param image Image <zh>ҪʾͼƬ (Ϊnull).</zh>
   * <en>The image (null permitted).</en>
   */
  public RectangleImageChartShape(Rectangle2D drawArea, Image image) {
    this(drawArea, image, false) ;
  }

  /**
   * <zh>drawAreaʾͼƬimage.</zh>
   * <en>Draw image in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param image Image <zh>ҪʾͼƬ (Ϊnull).</zh>
   * <en>The image (null permitted).</en>
   * @param drawBorder <zh>Ƿͼε</zh>
   * <en>A flag that controls whether or not the shape border is visible.</en>
   */
  public RectangleImageChartShape(Rectangle2D drawArea, Image image, boolean drawBorder) {
    super(image) ;
    this.drawArea = drawArea ;
    setDrawBorder(drawBorder) ;

    this.imageFill = DEFAULT_IMAGE_FILL_AREA ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    Rectangle2D cloneArea = new Rectangle2D.Double(0, 0, drawArea.getWidth(), drawArea.getHeight()) ;
    g2.translate(drawArea.getX(), drawArea.getY()) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;
//    g2.clip(cloneArea) ;

    float strokeWidth = 0 ;
    if (isDrawBorder()) {
      strokeWidth = StrokeUtil.getStrokeWidth(getBorderStroke()) ;
    }

    drawBackground(g2, cloneArea) ;
    drawBorder(g2, new Rectangle2D.Double(strokeWidth / 2, strokeWidth / 2,
                                          drawArea.getWidth() - strokeWidth,
                                          drawArea.getHeight() - strokeWidth)) ;

    g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getImageAlpha())) ;

    Image image = getImage() ;
    if (image != null) {
      cloneArea.setRect(0, 0, cloneArea.getWidth() - strokeWidth * 2,
                        cloneArea.getHeight() - strokeWidth * 2) ;

      RectangleInsets margin = getMargin() ;
      margin.trim(cloneArea) ;
      g2.translate(margin.getLeft() + strokeWidth, margin.getTop() + strokeWidth) ;

      int imageWidth = image.getWidth(null) ;
      int imageHeight = image.getHeight(null) ;
      if (imageWidth > 0 && imageHeight > 0) { //ͼƬѾ
        //
        if (isImageFill()
            || (imageWidth > cloneArea.getWidth() && imageHeight > cloneArea.getHeight())) {
          Image scaledImage = image.getScaledInstance((int) cloneArea.getWidth(),
                                                      (int) cloneArea.getHeight(),
                                                      Image.SCALE_SMOOTH) ;
          g2.drawImage(scaledImage, 0, 0,
                       (int) cloneArea.getWidth(), (int) cloneArea.getHeight(), null) ;
        } else {
          Image newImage = image ;
          int newImageWidth = (int) Math.min(imageWidth, cloneArea.getWidth()) ; //ͼƬ
          int newImageHeight = (int) Math.min(imageHeight, cloneArea.getHeight()) ; //ͼƬ߶
          if (imageWidth > cloneArea.getWidth() || imageHeight > cloneArea.getHeight()) {
            newImage = image.getScaledInstance(newImageWidth, newImageHeight, Image.SCALE_SMOOTH) ;
          }

          double startX = 0, startY = 0 ; //ʼx,yλ
          HorizontalAlignment hAlign = getImageHorizontalAlign() ;
          if (hAlign.equals(HorizontalAlignment.LEFT)) {
            startX = 0 ;
          } else if (hAlign.equals(HorizontalAlignment.CENTER)) {
            startX = (cloneArea.getWidth() - newImageWidth) / 2 ;
          } else if (hAlign.equals(HorizontalAlignment.RIGHT)) {
            startX = cloneArea.getWidth() - newImageWidth ;
          }

          VerticalAlignment vAlign = getImageVerticalAlign() ;
          if (vAlign.equals(VerticalAlignment.TOP)) {
            startY = 0 ;
          } else if (vAlign.equals(VerticalAlignment.CENTER)) {
            startY = (cloneArea.getHeight() - newImageHeight) / 2 ;
          } else if (vAlign.equals(VerticalAlignment.BOTTOM)) {
            startY = cloneArea.getHeight() - newImageHeight ;
          }

          g2.drawImage(newImage, (int) startX, (int) startY, newImageWidth, newImageHeight, null) ;
        }
      }
    }

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such
   * as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }

    if (isSelected()) {
      Graphics2DState state = new Graphics2DState(g2) ;
      state.saveOldState() ;

      g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getSelectRectAlpha())) ;

      double rectWidth = getSelectRectWidth() ;
      g2.translate(this.drawArea.getX(), this.drawArea.getY()) ;
      g2.rotate(getRotate()) ;

      //ê
      g2.setPaint(getSelectAnchorPaint()) ;
      AnchorUtil.drawAnchor(g2, this.drawArea, rectWidth) ;

      g2.rotate( -getRotate()) ;
      state.restoreOldState() ;
    }

  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      AffineTransform transform = new AffineTransform() ;
      transform.translate(this.drawArea.getX(), this.drawArea.getY()) ;
      transform.rotate(getRotate()) ;
      Rectangle2D newArea = new Rectangle2D.Double(0, 0, this.drawArea.getWidth(),
                                                   this.drawArea.getHeight()) ;

      transformedShape = transform.createTransformedShape(newArea) ;
    }
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      this.drawArea.setRect(drawArea.getX() + tx, drawArea.getY() + ty,
                            drawArea.getWidth(), drawArea.getHeight()) ;

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    return new Point2D.Double(this.drawArea.getX(), this.drawArea.getY()) ;
  }

  /**
   * <zh>ʱͼεĵ.</zh>
   * <en>Returns the lock coordinates of this shape.</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getLockPoint() {
    Rectangle2D area = getDrawArea() ;
    Rectangle2D newArea = new Rectangle2D.Double(0, 0, area.getWidth(), area.getHeight()) ;

    AffineTransform transform = AffineTransform.getTranslateInstance(area.getX(), area.getY()) ;
    transform.rotate(getRotate()) ;
    Shape rotateShape = transform.createTransformedShape(newArea) ;
    Rectangle2D rotateBound = rotateShape.getBounds2D() ;

    return new Point2D.Double(rotateBound.getCenterX(), rotateBound.getCenterY()) ;
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    calculateTransformedShape() ;
    return transformedShape ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   *
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   *   (Ϊnull).</zh>
   *  <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_IMAGE_IN_RECTANGLE ;
  }

  /**
   * <zh>ֿд.</zh>
   * <en>Sets the text draw area.</en>
   *
   * @param drawArea Rectangle2D <zh>εֿд (Ϊnull).</zh>
   * <en>The text draw area (null not permitted).</en>
   */
  public void setDrawArea(Rectangle2D drawArea) {
    if (drawArea == null) {
      throw new IllegalArgumentException("Null 'drawArea' argument.") ;
    }
    this.drawArea = drawArea ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ֵĿд.</zh>
   * <en>Returns the text draw area.</en>
   *
   * @return Rectangle2D <zh>εֿд.</zh>
   * <en>The text draw area.</en>
   */
  public Rectangle2D getDrawArea() {
    return drawArea ;
  }

  /**
   * <zh>ǷͼƬԶ.</zh>
   * <en>Returns whether or not fill area by image.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isImageFill() {
    return this.imageFill ;
  }

  /**
   * <zh>ǷͼƬԶ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets whether or not fill area by image,and sends a
   *  {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all
   * registered listeners.</en>
   *
   * @param fill boolean <zh>ǷͼƬԶ.</zh>
   * <en>Sets whether or not fill  area by image.</en>
   */
  public void setImageFill(boolean fill) {
    if (this.imageFill != fill) {
      this.imageFill = fill ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>tooltipͼӦƫƵλ.</zh>
   * <en>Returns the translate point of tooltip shape.</en>
   *
   * @return Point2D <zh> (Ϊnull).</zh>
   * <en>The translate point (null permitted).</en>
   */
  protected Point2D getTooltipShapeTranslatePoint() {
    if (this.drawArea != null) {
      return new Point2D.Double(this.drawArea.getX(), this.drawArea.getY()) ;
    }

    return null ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof RectangleImageChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    RectangleImageChartShape that = (RectangleImageChartShape) obj ;
    if (!EqualsUtil.equals(this.drawArea, that.drawArea)) {
      return false ;
    }
    if (this.imageFill != that.imageFill) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.drawArea) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.imageFill) ;

    return hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    Image image = getImage() ;

    return "[drawArea=" + drawArea + ",image="
      + ((image == null) ? "null" : (image.getWidth(null) + " x " + image.getHeight(null))) + "]" ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;
    SerialUtil.writeShape(stream, this.drawArea) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;
    this.drawArea = (Rectangle2D) SerialUtil.readShape(stream) ;
  }

}
