package com.ijchart.xychart.shape ;

import java.awt.AlphaComposite ;
import java.awt.Composite ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StrokeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ƹƱ(stock)ͼ.</zh>
 * <en>Draw a stock {@link com.ijchart.xychart.shape.IChartShape}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StockChartShape extends AbstractRectangleTextChartShape implements
  Serializable, Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 8230063775354436222L ;

  /**
   * <zh>volumeζ.</zh>
   * <en>The volume rectangle.</en>
   */
  private transient Rectangle2D volumeRect ;

  /**
   * <zh>volume paint.</zh>
   * <en>The volume paint.</en>
   */
  private transient Paint volumePaint ;

  /**
   * <zh>open-closeֵ.</zh>
   * <en>The open-close rectangle.</en>
   */
  private transient Rectangle2D openCloseRect ;

  /**
   * <zh>open-closeα߿paint.</zh>
   * <en>The paint used to draw open-close rectangle border.</en>
   */
  private transient Paint openCloseBorderPaint ;

  /**
   * <zh>open-closeεpaint.</zh>
   * <en>The paint used to fill open-close rectangle border.</en>
   */
  private transient Paint openCloseFillPaint ;

  /**
   * <zh>high-lowֵӦֱ.</zh>
   * <en>The high-low line.</en>
   */
  private transient Line2D highLowLine ;

  /**
   * <zh>high-lowֱߵpaint.</zh>
   * <en>The paint used to fill high-low rectangle border.</en>
   */
  private transient Paint highLowPaint ;

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param volumeRect Rectangle2D <zh>volumeζ (Ϊnull).</zh>
   * <en>The volume rectangle (null permitted).</en>
   * @param volumePaint Paint <zh>volume paint (Ϊnull).</zh>
   * <en>The volume paint (null permitted).</en>
   * @param openCloseRect Rectangle2D <zh>open-closeֵ (Ϊnull).</zh>
   * <en>The open-close rectangle (null not permitted).</en>
   * @param openCloseBorderPaint Paint <zh>open-closeα߿paint (Ϊnull).</zh>
   * <en>The paint used to draw open-close rectangle border (null not permitted).</en>
   * @param openCloseFillPaint Paint <zh>open-closeεpaint (Ϊnull).</zh>
   * <en>The paint used to fill open-close rectangle border (null not permitted).</en>
   * @param highLowLine Line2D <zh>high-lowֵӦֱ (Ϊnull).</zh>
   * <en>The high-low line (null not permitted).</en>
   * @param highLowPaint Paint <zh>high-lowֱߵpaint (Ϊnull).</zh>
   * <en>The paint used to fill high-low rectangle border (null not permitted).</en>
   */
  public StockChartShape(String text, Rectangle2D volumeRect, Paint volumePaint,
                         Rectangle2D openCloseRect, Paint openCloseBorderPaint,
                         Paint openCloseFillPaint, Line2D highLowLine, Paint highLowPaint) {
    super(openCloseRect, text) ;

    if (openCloseRect == null) {
      throw new IllegalArgumentException("Null 'openCloseRect' argument.") ;
    }
    if (openCloseBorderPaint == null) {
      throw new IllegalArgumentException("Null 'openCloseBorderPaint' argument.") ;
    }
    if (openCloseFillPaint == null) {
      throw new IllegalArgumentException("Null 'openCloseFillPaint' argument.") ;
    }
    if (highLowLine == null) {
      throw new IllegalArgumentException("Null 'highLowLine' argument.") ;
    }
    if (highLowPaint == null) {
      throw new IllegalArgumentException("Null 'highLowPaint' argument.") ;
    }

    this.volumeRect = volumeRect ;
    this.volumePaint = volumePaint ;
    this.openCloseRect = openCloseRect ;
    this.openCloseBorderPaint = openCloseBorderPaint ;
    this.openCloseFillPaint = openCloseFillPaint ;
    this.highLowLine = highLowLine ;
    this.highLowPaint = highLowPaint ;
  }

  /**
   * <zh>volumeζ.</zh>
   * <en>Returns the volume rectangle.</en>
   *
   * @return Rectangle2D <zh>ζ (Ϊnull).</zh>
   * <en>The rectangle (maybe null).</en>
   */
  public Rectangle2D getVolumeRect() {
    return this.volumeRect ;
  }

  /**
   * <zh>volumeζ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the volume rectangle,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param rect Rectangle2D <zh>ζ (Ϊnull).</zh>
   * <en>The rectangle (null permitted).</en>
   */
  public void setVolumeRect(Rectangle2D rect) {
    if (!(this.volumeRect == null && rect == null)) {
      this.volumeRect = rect ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>volume paint.</zh>
   * <en>Returns the volume paint.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (maybe null).</en>
   */
  public Paint getVolumePaint() {
    return this.volumePaint ;
  }

  /**
   * <zh>volume paint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the volume paint,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   */
  public void setVolumePaint(Paint paint) {
    if (!(this.volumePaint == null && paint == null)) {
      this.volumePaint = paint ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>open-closeֵ.</zh>
   * <en>Returns the open-close rectangle.</en>
   *
   * @return Rectangle2D <zh>ζ (Ϊnull).</zh>
   * <en>The rectangle (never null).</en>
   */
  public Rectangle2D getOpenCloseRect() {
    return this.openCloseRect ;
  }

  /**
   * <zh>open-closeֵζ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the open-close rectangle,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param rect Rectangle2D <zh>ζ (Ϊnull).</zh>
   * <en>The rectangle (null not permitted).</en>
   */
  public void setOpenCloseRect(Rectangle2D rect) {
    if (rect == null) {
      throw new IllegalArgumentException("Null 'rect' argument.") ;
    }

    this.openCloseRect = rect ;
    fireChartShapeChanged() ;

  }

  /**
   * <zh>ػopen-closeα߿paint.</zh>
   * <en>Returns the paint used to draw open-close rectangle border.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getOpenCloseBorderPaint() {
    return this.openCloseBorderPaint ;
  }

  /**
   * <zh>ûopen-closeα߿paint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw open-close rectangle border,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setOpenCloseBorderPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.openCloseBorderPaint = paint ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>open-closeεpaint.</zh>
   * <en>Returns the paint used to draw open-close rectangle border.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getOpenCloseFillPaint() {
    return this.openCloseFillPaint ;
  }

  /**
   * <zh>volume paint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the volume paint,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setOpenCloseFillPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.openCloseFillPaint = paint ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>high-lowֵӦֱ.</zh>
   * <en>Returns the high-low line.</en>
   *
   * @return Line2D <zh>ֱ߶ (Ϊnull).</zh>
   * <en>The line (never null).</en>
   */
  public Line2D getHighLowLine() {
    return this.highLowLine ;
  }

  /**
   * <zh>high-lowֵӦֱ,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the high-low line,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param line Line2D <zh>ֱ߶ (Ϊnull).</zh>
   * <en>The line (null not permitted).</en>
   */
  public void setHighLowLine(Line2D line) {
    if (line == null) {
      throw new IllegalArgumentException("Null 'line' argument.") ;
    }

    this.highLowLine = line ;
    fireChartShapeChanged() ;
  }

  /**
   * <zh>ػhigh-lowֱߵpaint.</zh>
   * <en>Returns the paint used to fill high-low rectangle border.</en>
   *
   * @return Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (never null).</en>
   */
  public Paint getHighLowPaint() {
    return this.highLowPaint ;
  }

  /**
   * <zh>ûhigh-lowֱߵpaint,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the paint used to fill high-low rectangle border,and sends a
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null not permitted).</en>
   */
  public void setHighLowPaint(Paint paint) {
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    this.highLowPaint = paint ;
    fireChartShapeChanged() ;

  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    double openCloseRectX = this.openCloseRect.getX() ;
    double openCloseRectY = this.openCloseRect.getY() ;

    g2.translate(openCloseRectX, openCloseRectY) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;

    Composite originalComposite = g2.getComposite() ;
    g2.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, getFillAlpha())) ;

    //draw volume rect
    if (this.volumeRect != null && this.volumePaint != null) {
      g2.setPaint(this.volumePaint) ;
      Rectangle2D newVolumeRect = new Rectangle2D.Double(this.volumeRect.getX() - openCloseRectX,
                                                         this.volumeRect.getY() - openCloseRectY,
                                                         this.volumeRect.getWidth(),
                                                         this.volumeRect.getHeight()) ;
      g2.fill(newVolumeRect) ;
    }
    g2.setComposite(originalComposite) ;

    //draw high low line
    Line2D newHighLowLine = new Line2D.Double(this.highLowLine.getX1() - openCloseRectX,
                                              this.highLowLine.getY1() - openCloseRectY,
                                              this.highLowLine.getX2() - openCloseRectX,
                                              this.highLowLine.getY2() - openCloseRectY) ;
    g2.setPaint(this.highLowPaint) ;
    g2.draw(newHighLowLine) ;

    //draw openCloseRect
    Rectangle2D newOpenCloseRect = new Rectangle2D.Double(0, 0, this.openCloseRect.getWidth(),
                                                          this.openCloseRect.getHeight()) ;
    g2.setPaint(this.openCloseFillPaint) ;
    g2.fill(newOpenCloseRect) ;
    g2.setPaint(this.openCloseBorderPaint) ;
    g2.draw(newOpenCloseRect) ;

    drawTextWithTrimMarginRect(g2, newOpenCloseRect) ;

    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such
   * as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (isSelected()) {
      Point2D.Double[] points = new Point2D.Double[] {
                                new Point2D.Double(0, 0),
                                new Point2D.Double(this.openCloseRect.getWidth(), 0),
                                new Point2D.Double(0, this.openCloseRect.getHeight()),
                                new Point2D.Double(this.openCloseRect.getWidth(),
                                                   this.openCloseRect.getHeight())
      } ;

      drawAnchorByPoints(g2, this.openCloseRect.getX(), this.openCloseRect.getY(), points) ;
    }
  }

  /**
   * <zh>ƶͼ,X᷽ƫtx,Y᷽ƫty,м
   * {@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Concatenates the current Graphics2D Transform with a translation transform.and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners.
   * Subsequent rendering is translated by the specified
   * distance relative to the previous position.
   * This is equivalent to calling transform(T), where T is an
   * AffineTransform represented by the following matrix:
   * <pre>
   *		[   1    0    tx  ]
   *		[   0    1    ty  ]
   *		[   0    0    1   ]
   * </pre>.</en>
   *
   * @param tx double <zh>X᷽ľ.</zh>
   * <en>The distance to translate along the x-axis.</en>
   * @param ty double <zh>Y᷽ľ.</zh>
   * <en>The distance to translate along the y-axis.</en>
   */
  public void translate(double tx, double ty) {
    if (!(tx == 0 && ty == 0)) {
      if (this.volumeRect != null) {
        this.volumeRect.setRect(this.volumeRect.getX() + tx, this.volumeRect.getY() + ty,
                                this.volumeRect.getWidth(), this.volumeRect.getHeight()) ;
      }

      this.openCloseRect.setRect(this.openCloseRect.getX() + tx, this.openCloseRect.getY() + ty,
                                 this.openCloseRect.getWidth(), this.openCloseRect.getHeight()) ;

      this.highLowLine.setLine(this.highLowLine.getX1() + tx, this.highLowLine.getY1() + ty,
                               this.highLowLine.getX2() + tx, this.highLowLine.getY2() + ty) ;

      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼϽǵ(ͼεת).</zh>
   * <en>Returns the top left corner coordinates of this shape (not consider rotation).</en>
   *
   * @return Point2D <zh>ֵ.</zh>
   * <en>The coordinate.</en>
   */
  public Point2D getTopLeftPoint() {
    double volumeRectX = (this.volumeRect == null) ? 0 : this.volumeRect.getMinX() ;
    double volumeRectY = (this.volumeRect == null) ? 0 : this.volumeRect.getMinY() ;

    double openCloseRectX = this.openCloseRect.getMinX() ;
    double openCloseRectY = this.openCloseRect.getMinY() ;

    double highLowLineX1 = this.highLowLine.getX1() ;
    double highLowLineX2 = this.highLowLine.getX2() ;
    double highLowLineY1 = this.highLowLine.getY1() ;
    double highLowLineY2 = this.highLowLine.getY2() ;

    double minX = Math.min(Math.min(volumeRectX, openCloseRectX),
                           Math.min(highLowLineX1, highLowLineX2)) ;
    double minY = Math.min(Math.min(volumeRectY, openCloseRectY),
                           Math.min(highLowLineY1, highLowLineY2)) ;

    return new Point2D.Double(minX, minY) ;
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      double openCloseRectX = this.openCloseRect.getX() ;
      double openCloseRectY = this.openCloseRect.getY() ;

      GeneralPath path = new GeneralPath() ;
      if (this.volumeRect != null && this.volumePaint != null) {
        Rectangle2D newVolumeRect = new Rectangle2D.Double(this.volumeRect.getX() - openCloseRectX,
                                                           this.volumeRect.getY() - openCloseRectY,
                                                           this.volumeRect.getWidth(),
                                                           this.volumeRect.getHeight()) ;
        path.append(newVolumeRect.getPathIterator(null), false) ;
      }

      Line2D newHighLowLine = new Line2D.Double(this.highLowLine.getX1() - openCloseRectX,
                                                this.highLowLine.getY1() - openCloseRectY,
                                                this.highLowLine.getX2() - openCloseRectX,
                                                this.highLowLine.getY2() - openCloseRectY) ;
      Shape shape = StrokeUtil.lineToRectangle(newHighLowLine, getBorderStroke(), 0) ;
      path.append(shape, false) ;

      Rectangle2D newOpenCloseRect = new Rectangle2D.Double(0, 0, this.openCloseRect.getWidth(),
                                                            this.openCloseRect.getHeight()) ;
      path.append(newOpenCloseRect.getPathIterator(null), false) ;

      AffineTransform transform = new AffineTransform() ;
      transform.translate(openCloseRectX, openCloseRectY) ;
      transform.rotate(getRotate()) ;
      transformedShape = transform.createTransformedShape(path) ;
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    double openCloseRectX = this.openCloseRect.getX() ;
    double openCloseRectY = this.openCloseRect.getY() ;

    GeneralPath path = new GeneralPath() ;
    if (this.openCloseRect != null) {
      Rectangle2D newOpenCloseRect = new Rectangle2D.Double(0, 0, this.openCloseRect.getWidth(),
                                                            this.openCloseRect.getHeight()) ;
      path.append(newOpenCloseRect.getPathIterator(null), true) ;
    } else if (this.volumeRect != null) {
      Rectangle2D newVolumeRect = new Rectangle2D.Double(this.volumeRect.getX() - openCloseRectX,
                                                         this.volumeRect.getY() - openCloseRectY,
                                                         this.volumeRect.getWidth(),
                                                         this.volumeRect.getHeight()) ;
      path.append(newVolumeRect.getPathIterator(null), true) ;
    }

    AffineTransform transform = new AffineTransform() ;
    transform.translate(openCloseRectX, openCloseRectY) ;
    transform.rotate(getRotate()) ;

    return transform.createTransformedShape(path) ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   *
   * @return ShapeType <zh>{@link ShapeType}ඨĳ (Ϊnull).</zh>
   * <en>The constant value in class {@link ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_STOCK ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false
   *  otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    if (!(obj instanceof StockChartShape)) {
      return false ;
    }

    StockChartShape that = (StockChartShape) obj ;
    if (!EqualsUtil.equals(this.volumeRect, that.volumeRect)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.volumePaint, that.volumePaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.openCloseRect, that.openCloseRect)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.openCloseBorderPaint, that.openCloseBorderPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.openCloseFillPaint, that.openCloseFillPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.highLowLine, that.highLowLine)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.highLowPaint, that.highLowPaint)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.volumeRect) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.volumePaint) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.openCloseRect) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.openCloseBorderPaint) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.openCloseFillPaint) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.highLowLine) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.highLowPaint) ;

    return (int) hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "StockChartShape" ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the output stream.
   *
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writeShape(stream, this.volumeRect) ;
    SerialUtil.writePaint(stream, this.volumePaint) ;

    SerialUtil.writeShape(stream, this.openCloseRect) ;
    SerialUtil.writePaint(stream, this.openCloseBorderPaint) ;
    SerialUtil.writePaint(stream, this.openCloseFillPaint) ;

    SerialUtil.writeShape(stream, this.highLowLine) ;
    SerialUtil.writePaint(stream, this.highLowPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.volumeRect = (Rectangle2D) SerialUtil.readShape(stream) ;
    this.volumePaint = SerialUtil.readPaint(stream) ;

    this.openCloseRect = (Rectangle2D) SerialUtil.readShape(stream) ;
    this.openCloseBorderPaint = SerialUtil.readPaint(stream) ;
    this.openCloseFillPaint = SerialUtil.readPaint(stream) ;

    this.highLowLine = (Line2D) SerialUtil.readShape(stream) ;
    this.highLowPaint = SerialUtil.readPaint(stream) ;
  }
}
